/* ceclipse.h
   (C) 01.03.98 Oliver Obst, Uni Koblenz

   use this file to program external eclipse prolog predicates with c or c++.
   some warning messages may appear during compilation (that's ok).
   
   $Id: ceclipse.h,v 1.4 1999/05/05 14:58:43 frvit Exp $

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
 
*/
#ifndef CECLIPSE_H
#define CECLIPSE_H

#ifdef __cplusplus
# if ECLIPSE >= 40
#  define EC_EXTERNAL
#  define EC_EMBED
#  define Winapi
#  include "eclipseconfig.h"
#  include "ec_public.h"
#  include "types.h"
#  include "embed.h"
# elif ECLIPSE >= 36
#  include "eclipseconfig.h"
#  include "sepia.h"
#  include "eclipseclass.h"
//#  include "embed.h"
#  include "error.h"
# else
#  error "please use ECLiPSe 3.7.1 or ECLiPSe 4.0.1"
# endif
#else
# include <eclipse.h>
#endif

#ifdef __cplusplus
#define Prolog extern "C"
#else
#define Prolog

#define PERROR			-1	/* general error */
#define UNIFY_OVNI		-2	/* unknown object type */
#if 0	/* these are already defined */
#define INSTANTIATION_FAULT	-4	/* variable instead of constant */
#define TYPE_ERROR		-5	/* wrong type */
#define RANGE_ERROR		-6	/* out of range */
#endif
#define BAD_FORMAT_STRING	-7	/* in printf */
#define BAD_ARGUMENT_LIST	-8	/* in printf */

#define META_TERM_UNIFY		-11	/* metaterm unification */
#define	FAIL_TO_PAR_CHP		(-16)
#define	CREATING_PAR_CHP	(-15)
#define RECOMP_FAILED		-17	/* failure during oracle following */
#define ARITH_EXCEPTION		-20	/* arithmetic exception(0 division) */
#define COMPARE_TRAP		-23	/* non-number in comparison	*/
#define ARITH_TYPE_ERROR	-24	/* non-number arithmetic operand */
#define INTEGER_OVERFLOW	-25
#define ARITY_LIMIT		-31	/* arity limit exceeded */
#define EVENT_IGNORED		-32
#define EVENT_QUEUE_FULL	-33
#define STALE_HANDLE		-40	/* stale external object handle */
#define NOGLOBAL		-41	/* global var. or array does not
					 * exist */
#define ARRAY_EXISTS		-42
#define MULT_POST_INF           -43     /* multiple definiton postfix/infix*/
#define LOCAL_REC		-44	/* local record exists		*/
#define	NO_LOCAL_REC		-45	/* no local record		*/

#define NOENTRY			-60	/* entry does not exist */
#define INCONSISTENCY		-62
#define	NOT_DYNAMIC		-63	/* should be dynamic */
#define ALREADY_DYNAMIC		-64
#define ALREADY_DEFINED		-65	/* probably not necessary */
#define REDEF_SYS		-66
#define NOT_LOADED		-67
#define CALLING_UNDEFINED	-68
#define CALLING_AUTOLOAD	-69
#define ACCESSING_UNDEF_DYN_PROC -70	/* no defined dynamic at all */

#define UNDEF_OPERATOR		-72

#define MODULENAME		-80
#define DIRECTIVE		-81
#define LOCKED			-82
#define	ALREADY_LOCAL		-87	/* when not expected	*/
#define	ALREADY_EXPORTED	-88	/* when not expected	*/
#define	ALREADY_GLOBAL		-89	/* when not expected	*/
#define NO_TOOL			-91
#define LOCAL_EXISTS		-92
#define EXPORT_EXISTS		-93
#define IMPORT_EXISTS		-94
#define GLOBAL_EXISTS		-95
#define ALREADY_IMPORT		-96
#define MODULE_EXISTS		-97
#define WRONG_UNLOCK_STRING	-98
#define IMPORT_CLASH		-99
#define	ACCESSING_NON_LOCAL	-100
#define SELF_DESTRUCTION	-101

#define	ERR_PARSE		-110

#define PUNCTUATION             -111
#define ILL_QUOTED		-112
#define PRECEDENCE              -113
#define UNEXPECTED              -114
#define ENDOFFILE               -115
#define	LEX_ERROR		-116
#define BRACKET                 -117
#define	ENDOFCLAUSE		-118
#define	POSTINF			-119
#define	SOLOCH			-120
#define BLANK			-121
#define MULTI_META		-122

#define	PREFINF			-124
#define	UNCLOSBR 		-125
#define GRAMMAR_HEAD		-126
#define GRAMMAR_BODY		-127
#define TRANS_ERROR		-128
#define	ILLEGAL_HEAD		-130
#define	ILLEGAL_GOAL		-131
#define VARIABLE_GOAL		ILLEGAL_GOAL
#define WRONG_TYPE              -132
#define	LIBRARY			-133
#define	CONSECUTIVE		-134
#define	PROTECT_REDEF		-135
#define	BUILT_IN_REDEF		-136
#define	INCONSISTENT_REDEF	-137
#define	QUERY_FLOUNDERED	-138
#define COMPILED_FILE		-139
#define UNDEFINED		-140	/* unknown instruction */
#define UNIMPLEMENTED		-141
#define NOT_AVAILABLE		-142
#define QUERY_FAILED		-143
#define IF_CUT			-144	/* cut inside condition	*/
#define MULTIFILE		-145
#define BAD_PRAGMA		-148
#define GLOBAL_TR_EXISTS	-160
#define TR_IN_MOD		-161
#define NO_TR			-162
#define	ONE_SQ_AQ		-163
#define SYS_ERROR		-170	/* (host) system error; errno usually
					 *  holds the real error code.
					 * The global variable whose did is
					 * "d_errno_" should be set by the
					 * macro "Set_Errno" to the system
					 * "errno" describing the error.
					 */
#define NO_FILE			-171
#define FILE_NOT_OPEN		-172
#define MPS_ERROR		-176
#define WIN32_ERROR		-177
#define PEOF			-190	/* end of file in an IO predicate */
#define OUT_ERROR		-191	/* like SYS_ERROR for output functions*/
#define STREAM_MODE		-192 	/* wrong stream mode */
#define STREAM_SPEC		-193
#define TOO_MANY_NAMES		-194
#define SYSTEM_STREAM		-196
#define INCORRECT_USER		-197
#define READ_PAST_EOF		-198	/* reading after PEOF was reached */
#define	REMEMBER		-210
#define NOCODE			-211	/* no code in the descriptor	*/
#define ILLEGAL_RETURN		-212	/* bad return code from external */
#define	TRACING_NODBG		-250
#define	DEBUG_USER_EVENT	-251
#define	EC_DEBUG_EVENT		-252

#define UNEXPECTED_EOF		-260
#define INVALID_SS		-261
#define CANT_ALLOCATE		-262
#define WRONG_LEVEL		-263
#define NOT_DUMP_FILE		-264
#define BAD_DUMP_VERSION	-265
#define NOT_IMPLEMENTED		-267
#define NOT_IN_PARALLEL		-268

#define	UNDEF_ATTR	        -270
#define	ATTR_FORMAT	        -271
#define	BAD_DELAY_CONDITION     -272
#define	BAD_RESTORE_WL		-274

#define KEGI_GENERAL_SUNVIEW	-330
#define KEGI_GENERAL_X11	-331
#endif


/* ceclipse error checking */
Prolog int ec_err( int resin, int *resout )
{
        if ( *resout == PSUCCEED )
                *resout = resin;

        return ( resin != PSUCCEED );
}


/* ceclipse error checking */
Prolog int ec_ok( int resin, int *resout )
{
        if ( *resout == PSUCCEED )
                *resout = resin;

        return ( resin == PSUCCEED );
}


/* return PFAIL if input is false, else return input value */
Prolog int ec_bool( const int b, int res )
{
        return b ? res : PFAIL;
}


/* type-check integer or double, return double */
Prolog int ec_get_number( const pword pnumber, double *cnumber )
{
        int res;
        long n;

        if (( res = ec_get_long( pnumber, &n )) == TYPE_ERROR ) {
                res = ec_get_double( pnumber, cnumber );
        } else {
                if ( res == PSUCCEED ) {
                        *cnumber = n;
                }
        }
        return res;
}


/* return did name */
Prolog char* ec_did_name(dident d)
{
        return ((char *)(((struct dict_item *)(d))->string + 1));
}


#endif /* CECLIPSE_H */
