/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (c) Mar 03 1998 Oliver Obst
   internal types and constants for all robolog modules
   $Id: constants.h,v 1.4 1999/05/05 13:43:23 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _CONSTANTS_H
#define _CONSTANTS_H

#include "maths.h"

// used in control
const int       MaxStringSize           =    4096;
const int       TeamnameLen             =      16;
const int       FilenameLen             =     256;
const int       HostnameLen             =      64;

// used in objarray and msgarray
const int       MaxObject               =     255; // max. objects at a cycle
const int       MaxMessage              =       5; // max. messages at a cycle

const double    dir_self                =    9998;
const double    dir_referee             =    9999;

// used in listproc
const int       st_before_kick_off      =       1;
const int       st_kick_off_l           =       2;
const int       st_kick_off_r           =       3;
const int       st_kick_in_l            =       4;
const int       st_kick_in_r            =       5;
const int       st_corner_kick_l        =       6;
const int       st_corner_kick_r        =       7;
const int       st_goal_kick_l          =       8;
const int       st_goal_kick_r          =       9;
const int       st_free_kick_l          =      10;
const int       st_free_kick_r          =      11;
const int       st_play_on              =      12;
const int       st_half_time            =      13;
const int       st_time_up              =      14;
const int       st_extend               =      15;
const int       st_foul_l_              =      16;
const int       st_foul_r_              =      17;
const int       st_goal_l_              =      18;
const int       st_goal_r_              =      19;
const int       st_goalie_catch_ball_l  =      20;
const int       st_goalie_catch_ball_r  =      21;
const int       no_side                 =       0;
const int       left_side               =       1;
const int       right_side              =       2;


// used in objarr
const double    abs_unknown     =  9000;
const double    bearing_unknown = -9999;
const short     id_goal_left    =     0;
const short     id_goal_right   =     1;
const short     id_flag_l_t     =     0;
const short     id_flag_l_b     =     1;
const short     id_flag_c_t     =     2;
const short     id_flag_c_b     =     3;
const short     id_flag_r_t     =     4;
const short     id_flag_r_b     =     5;
const short     id_pflag_l_t    =     6;
const short     id_pflag_l_c    =     7;
const short     id_pflag_l_b    =     8;
const short     id_pflag_r_t    =     9;
const short     id_pflag_r_c    =    10;
const short     id_pflag_r_b    =    11;
const short     id_flag_c       =    12;
const short     id_gflag_l_t    =    13;
const short     id_gflag_l_b    =    14;
const short     id_gflag_r_t    =    15;
const short     id_gflag_r_b    =    16;
const short     id_oflag_t_l_10 =    17;
const short     id_oflag_t_l_20 =    18;
const short     id_oflag_t_l_30 =    19;
const short     id_oflag_t_l_40 =    20;
const short     id_oflag_t_l_50 =    21;
const short     id_oflag_t_r_10 =    22;
const short     id_oflag_t_r_20 =    23;
const short     id_oflag_t_r_30 =    24;
const short     id_oflag_t_r_40 =    25;
const short     id_oflag_t_r_50 =    26;
const short     id_oflag_b_l_10 =    27;
const short     id_oflag_b_l_20 =    28;
const short     id_oflag_b_l_30 =    29;
const short     id_oflag_b_l_40 =    30;
const short     id_oflag_b_l_50 =    31;
const short     id_oflag_b_r_10 =    32;
const short     id_oflag_b_r_20 =    33;
const short     id_oflag_b_r_30 =    34;
const short     id_oflag_b_r_40 =    35;
const short     id_oflag_b_r_50 =    36;
const short     id_oflag_l_t_10 =    37;
const short     id_oflag_l_t_20 =    38;
const short     id_oflag_l_t_30 =    39;
const short     id_oflag_l_b_10 =    40;
const short     id_oflag_l_b_20 =    41;
const short     id_oflag_l_b_30 =    42;
const short     id_oflag_r_t_10 =    43;
const short     id_oflag_r_t_20 =    44;
const short     id_oflag_r_t_30 =    45;
const short     id_oflag_r_b_10 =    46;
const short     id_oflag_r_b_20 =    47;
const short     id_oflag_r_b_30 =    48;
const short     id_oflag_t_0    =    49;
const short     id_oflag_b_0    =    50;
const short     id_oflag_l_0    =    51;
const short     id_oflag_r_0    =    52;
const short     id_line_l       =     0;
const short     id_line_r       =     1;
const short     id_line_t       =     2;
const short     id_line_b       =     3;
const short     id_unknown      = -9999;

// used in self
const Cvector   Cv_unknown(bearing_unknown, bearing_unknown);

// used in control, knowledge
enum Angle { normal, wide, narrow };
enum Quality { high, low };
/* view widths */
#define v_normal                       2
#define v_wide                         3
#define v_narrow                       1
/* view qualities */
#define v_high                         2
#define v_low                          1


const double DOUBLE_EPS = 0.001;

// used in objarr, objinfo
enum Objtype { unknown, ball, player, partner, opponent, goal, flag, line };

// used in sclient
enum Msgtype { self, referee, other, all };

// lines
enum line_t { SL_left, SL_right, SL_top, SL_bottom, SL_no_line };

typedef unsigned char unum_t;

enum marker_t {
  Goal_L,
  Goal_R,

  Flag_C,
  Flag_CT,
  Flag_CB,
  Flag_LT,
  Flag_LB,
  Flag_RT,
  Flag_RB,

  Flag_PLT,
  Flag_PLC,
  Flag_PLB,
  Flag_PRT,
  Flag_PRC,
  Flag_PRB,

  Flag_GLT,
  Flag_GLB,
  Flag_GRT,
  Flag_GRB,

  Flag_TL50,
  Flag_TL40,
  Flag_TL30,
  Flag_TL20,
  Flag_TL10,
  Flag_T0,
  Flag_TR10,
  Flag_TR20,
  Flag_TR30,
  Flag_TR40,
  Flag_TR50,

  Flag_BL50,
  Flag_BL40,
  Flag_BL30,
  Flag_BL20,
  Flag_BL10,
  Flag_B0,
  Flag_BR10,
  Flag_BR20,
  Flag_BR30,
  Flag_BR40,
  Flag_BR50,

  Flag_LT30,
  Flag_LT20,
  Flag_LT10,
  Flag_L0,
  Flag_LB10,
  Flag_LB20,
  Flag_LB30,

  Flag_RT30,
  Flag_RT20,
  Flag_RT10,
  Flag_R0,
  Flag_RB10,
  Flag_RB20,
  Flag_RB30,

  No_Marker
};
#endif
