/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   control interface for the server connection
   Copyright (C) Oct 22 1997 Oliver Obst
   $Id: control.h,v 1.4 1999/05/05 13:47:35 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _CONTROL_H
#define _CONTROL_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "constants.h"
#include "fifo.h"
#include "oclient.h"
#include "listproc.h"

class Control : public OClient
{
public:
        Control(Listproc *l, const char *server, int port, double ver);
        virtual ~Control();

        void    init(bool goalie);
        virtual void moveto(double x, double y);
        virtual void turn(double moment);
        virtual void turn_neck(double angle_d);
        virtual void dash(double power);
        virtual void kick(double power, double direction);
        virtual void say(const char *text=0);
        void    goalie_catch(double direction);
        void    changeview(Angle width=normal, Quality quality=high);
        void    sense_body();

        void    transceive();
        void    dispatch();
        void    receive();
        
        bool    get_data(char *data);
        void    wait4outq(int level);
        void    dequeue();
        int     outqlen();

protected:
        Fifo      	outq;
        char    	tmpstr[ MaxStringSize ];
        char            netstr[ MaxStringSize ];
        Listproc        *sensor;

private:
        char    	inq[ MaxStringSize ];
        bool    	have_data;
};
#endif /* _CONTROL_H */
