/* -*- mode: c -*-
   
   this file is part of the robolog libraries
   Copyright (C) Apr 24 1999 Oliver Obst
   GNU prolog interface for the robolog c library
   $Id$

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <signal.h>
 
#include <robolog.h>
#include <gprolog.h>

/**
 * atan2 for prolog.
 * This function calculates the arc tangent of the two
 * numbers x and y. It is similar to calculating the arc
 * tangent of y / x, except that the signs of both arguments
 * are used to determine the quadrant of the result. (from man atan2(3))
 *
 * Prolog: atan2(+X, +Y, ?Result)
 */
Bool p_atan2(double y, double x, FIOArg *result)
{
        result->value.d = RSC_atan2(y,x);
        return TRUE;
}

/**
 * deg2rad for prolog.
 * Converts a number from degrees to radiants.
 *
 * Prolog: deg2rad(+Deg, ?Rad)
 */
Bool p_deg2rad(double deg, FIOArg *rad)
{
        rad->value.d = RSC_deg2rad(deg);
        return TRUE;
}     

/**
 * rad2deg for prolog.
 * Converts a number from radiants to degrees.
 *
 * Prolog: rad2deg(+Rad, ?Deg)
 */
Bool p_rad2deg(double rad, FIOArg *deg)
{
        deg->value.d = RSC_rad2deg(rad);
        return TRUE;
}     

/**
 * server registration for normal players.
 *
 * Prolog: init_player(+Teamname, +Servername, +Portnr, +Version)
 */
Bool p_init_player(char *teamname, char *servername,
                   int portnr, double version)
{
        RSC_init_player(teamname, servername, portnr, version);
        return TRUE;
}

/**
 * server registration for goalies.
 *
 * Prolog: init_goalie(+Teamname, +Servername, +Portnr, +Version)
 */
Bool p_init_goalie(char *teamname, char *servername,
                   int portnr, double version)
{
        RSC_init_goalie(teamname, servername, portnr, version);
        return TRUE;
}

/**
 * stops IRQ and deletes client
 *
 * Prolog: stop/0
 */
Bool p_stop()
{
        RSC_stop();
        return TRUE;
}

/**
 * open_log(+Filename[, +Append, +Flushfreq])   
 * start a normal logfile 'Filename', which will be flushed every
 * 'Flushfreq' lines.
 * If 'Append' is false, an existing file 'Filename' will be overwritten.
 */
Bool p_open_log(char *filename)
{
        /* FIXME: add append and flush freq. */        
        if (RSC_open_log(filename) != 0) {
                Pl_Err_Permission(permission_operation_output,
                                  permission_type_stream, filename);
        }
        return TRUE;
}

/**
 * open_audiolog(+Filename[, +Append, +Flushfreq])   
 * start an audio logfile 'Filename', which will be flushed every
 * 'Flushfreq' lines.
 * If 'Append' is false, an existing file 'Filename' will be overwritten.
 */
Bool p_open_audiolog(char *filename)
{
        /* FIXME: add append and flush freq. */        
        if (RSC_open_audiolog(filename) != 0) {
                Pl_Err_Permission(permission_operation_output,
                                  permission_type_stream, filename);
        }
        return TRUE;
}

static void Timer_Handler()
{
        printf("Oli: %s\n","TRralola, ein test test");
        fflush(NULL);
}


Bool p_oli()
{
        printf("Oli\n");
        signal(14, Timer_Handler);
        return TRUE;
}

