/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   knowledge interface definition
   Copyright (C) Apr 03 1998 Oliver Obst
   $Id: knowledge.C,v 1.3 1999/02/03 08:10:25 frvit Exp $


   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if STDC_HEADERS
# include <string.h>
#endif
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#include "constants.h"
#include "soccerconst.h"
#include "knowledge.h"
#include "objinfo.h"

Knowledge::Knowledge(int)
{
        teamname[0] = '\000';
        opponentname[0] = '\000';
        
        n_goals_l = n_goals_r = 0;

        view_q = high;
        view_w = normal;
        
        mynumber = 0;
        myside = 0;
        mystatus = 0;

        stamina = 2000;
        effort = 1;
        recovery = 1;
        neck = 0;
        
        speed = 0;
        kick_count = dash_count = turn_count = turn_neck_count = say_count = 0;

        goalie[0] = goalie[1] = 0;
}

Knowledge::Knowledge(const Knowledge &k)
{
        view_q = k.view_q;
        view_w = k.view_w;
        n_goals_l = k.n_goals_l;
        n_goals_r = k.n_goals_r;
        strcpy(teamname, k.teamname);
        strcpy(opponentname, k.opponentname);
        mynumber = k.mynumber;
        myside = k.myside;
        mystatus = k.mystatus;
        stamina = k.stamina;
        effort = k.effort;
        version = k.version;
        recovery = k.recovery;
        neck = k.neck;
        speed = k.speed;
        kick_count = k.kick_count;
        dash_count = k.dash_count;
        turn_count = k.turn_count;
        turn_neck_count = k.turn_neck_count;
        say_count = k.say_count;
        goalie[0] = k.goalie[0];
        goalie[1] = k.goalie[1];
}


Knowledge::~Knowledge()
{
}
        
// --- inline public ----------------------------------------------------------
inline void Knowledge::dash(double power)
{
        if (power < 0) 
                stamina = (version < 5 ?
                           max(stamina + power, 0) :
                           max(stamina + power + power, 0));
        else
                stamina = max(stamina - power, 0);

        dash_count++;
}

// --- inline public ----------------------------------------------------------
inline void Knowledge::turn_neck(double angle_d)
{
        double newneck = neck + angle_d;
        turn_neck_count++;
        
        if (angle_d < 0) 
                neck = (newneck > min_neck_angle ? newneck : min_neck_angle);
        else
                neck = (newneck < max_neck_angle ? newneck : max_neck_angle);
}

// --- public -----------------------------------------------------------------
void Knowledge::set_teamname(const char *name, int team)
{
        if (team == 0) {
                strncpy(teamname, name, TeamnameLen - 1);
                teamname[ TeamnameLen - 1 ] = '\000';        
        } else {
                strncpy(opponentname, name, TeamnameLen - 1);
                opponentname[ TeamnameLen - 1 ] = '\000'; 
        }
}

// --- public -----------------------------------------------------------------
void Knowledge::set_goalie(bool b)
{
        if (b) goalie[0] = mynumber;
}

// --- public -----------------------------------------------------------------
void Knowledge::set_goalie(int nr, int team)
{
        if (team == 0) goalie[0] = nr;
        else goalie[1] = nr;
}

// --- public -----------------------------------------------------------------
void Knowledge::set_view_quality(Quality q)
{
        view_q = q;
}

// --- public -----------------------------------------------------------------
void Knowledge::set_view_width(Angle a)
{
        view_w = a;
}

// --- public -----------------------------------------------------------------
void Knowledge::set_goals(int side, int goals)
{
        if (side == 0) n_goals_l = goals;
        else n_goals_r = goals;
}

// --- public -----------------------------------------------------------------
char* Knowledge::get_teamname(int team) const
{
        if (team == 0) return (char *) teamname;
        else return (char *) opponentname;
}

// --- public -----------------------------------------------------------------
bool Knowledge::is_goalie() const
{
        return (goalie[0] == mynumber);
}

// --- public -----------------------------------------------------------------
int Knowledge::get_goalie(int team=0) const
{
        if (team == 0) return goalie[0];
        else return goalie[1];
}
