/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   knowledge interface definition
   Copyright (C) Apr 03 1998 Oliver Obst
   $Id: knowledge.h,v 1.4 1999/05/05 13:58:43 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _KNOWLEDGE_H
#define _KNOWLEDGE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "constants.h"
#include "maths.h"

class Knowledge 
{
public:
        Knowledge(int i=0);
        Knowledge(const Knowledge &k);
        virtual ~Knowledge();

        void    set_teamname(const char *name, int team=0);
        void    set_goalie(bool b=true);
        void    set_goalie(int nr, int team=0);
        void    set_view_quality(Quality q);
        void    set_view_width(Angle a);
        void    set_goals(int side, int goals);

        inline  void    set_number(int n) { mynumber = n; }
        inline  void    set_side(int n) { myside = n; }
        inline  void    set_status(int n) { mystatus = n; }
        inline  void    set_stamina(double n) { stamina = n; }
        inline  void    set_effort(double d) { effort = d; }
        inline  void    set_recovery(double d) { recovery = d; }
        inline  void    set_version(double ver) { version = ver; }
        inline  void    set_speed(double d) { speed = d; }
        inline  void    set_neck(double d) { neck = d; }
        inline  void    set_kicks(int n) { kick_count = n; }
        inline  void    set_dashs(int n) { dash_count = n; }
        inline  void    set_turns(int n) { turn_count = n; }
        inline  void    set_turn_necks(int n) { turn_neck_count = n; }
        
        
        inline  void    set_says(int n) { say_count = n; }
        
        char*   get_teamname(int team=0) const;

        inline  int     get_number() const { return mynumber; }        
        bool    is_goalie() const;
        int     get_goalie(int team=0) const;
        inline  int     get_side() const { return myside; }
        inline  Quality get_view_quality() const { return view_q; }
        inline  Angle   get_view_width() const { return view_w; }
        inline  int     get_status() const { return mystatus; }
        
        inline  double  get_stamina() const { return stamina; }
        inline  double  get_effort() const { return effort; }
        inline  double  get_recovery() const { return recovery; }
        inline  double  get_version() const { return version; }
        inline  double  get_neck() const { return neck; }

        inline  double  get_speed() const { return speed; }
        inline  int     get_kicks() const { return kick_count; }
        inline  int     get_dashs() const { return dash_count; }
        inline  int     get_turns() const { return turn_count; }
        inline  int     get_turn_necks() const { return turn_neck_count; }
                                                        
        inline  int     get_says() const { return say_count; }
        inline  int     get_goals(int side)
                { return (side == id_goal_left ? n_goals_l : n_goals_r); }

        inline  void    dash(double power);
        inline  void    turn() { turn_count++; }
        inline  void    turn_neck(double angle_d);
        inline  void    kick() { kick_count++; }
        inline  void    say() { say_count++; }
        

private:
        Quality view_q;
        Angle   view_w;        
        
        int     n_goals_l;
        int     n_goals_r;
        
        char    teamname[ TeamnameLen ];
        char    opponentname[ TeamnameLen ];
        int     mynumber;
        int     myside;
        int     mystatus;

        double  stamina;
        double  effort;
        double  recovery;
        double  version;
        double  neck;
        
        double  speed;
        int     kick_count;
        int     dash_count;
        int     turn_count;
        int     turn_neck_count;
        int     say_count;

        int     goalie[2];
};
#endif /* _KNOWLEDGE_H */
