/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   Copyright (C) Nov 09 1997 Oliver Obst
   list processor (parsing and storing server data)
   $Id: listproc.C,v 1.6 1999/05/05 14:03:05 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if HAVE_STRINGS_H
# include <strings.h>
#endif

#if STDC_HEADERS
# include <string.h>
# include <stdlib.h>
#else
# if HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#include "listproc.h"
#include "objarray.h"
#include "msgarray.h"
#include "knowledge.h"
#include "debug.h"

// --- public constructor -----------------------------------------------------
Listproc::Listproc()
{
        strcpy(myname, "");
        mynamelen = 0;
        set_opname = true;
        version = 0;
        time = 0;
        moved = msgup = objup = goalie = false;
       
        oa = 0;
        ma = 0;
        k = 0;
}

// --- public constructor -----------------------------------------------------
Listproc::Listproc(ObjArray *oa0, MsgArray *ma0, Knowledge *k0)
{
        strcpy(myname, "");
        mynamelen = 0;
        version = 0;
        time = 0;
        moved = msgup = objup = goalie = false;
        
        set_opname = true;
        
        oa = oa0;
        ma = ma0;
        k = k0;
}

// --- public copy constructor ------------------------------------------------
Listproc::Listproc(const Listproc &l) 
{
        strcpy(myname, l.myname);
        mynamelen = l.mynamelen;
        version = l.version;
        time = l.time;
        moved = l.moved;
        msgup = l.msgup;
        objup = l.objup;
        goalie = l.goalie;
        set_opname = l.set_opname;
        
        oa = l.oa;
        ma = l.ma;
        k = l.k;
}

// --- public destructor ------------------------------------------------------
Listproc::~Listproc()
{
}

// --- public -----------------------------------------------------------------
void Listproc::set_teamname(const char *name, bool keeper)
{
        strcpy(myname, "(player ");
        strcat(myname, name);
        mynamelen = strlen(myname);
        k->set_teamname(name,0);
        goalie = keeper;
}

// --- public -----------------------------------------------------------------
void Listproc::set_base(ObjArray *oa0, MsgArray *ma0, Knowledge *k0)
{
        oa = oa0;
        ma = ma0;
        k = k0;
}

// --- public -----------------------------------------------------------------
char* Listproc::get_teamname(bool init)
{
        if (init) return myname+8;
        else return myname;
}

// --- public -----------------------------------------------------------------
void Listproc::parse(const char *string)
{
        char    *list = 0;
        char    *s = 0;

        ObjInfo oi;

        int l = 0;
        
        if (string != 0) {
                l = rindex(string, ')') - string + 1;
                if (l > 2) {
                        list = new char[ l + 1 ];
                        ROBOBUG1(ROBOBUG_INFO, LISTPROC_MEMORY,
                                 "parse(const char *): list = new char[%u]",
                                 l+1);
                        strncpy(list, string, l);
                        list[ l ] = '\0';
                }
        }

        if (l < 3 ) return;

        int t;
        MsgInfo msg;
        
        switch (list[3]) {
        case 'e' : // see
                oa->clear();
                objup = true;

                t = strtol(list + 5, &s, 10);
                set_time(t);

                while (s != 0) {
                        if ((s = index(s, '(')) != 0) {
                                s++;
                                oi = identify(&s);
                                bearing(&s, oi);
                                oa->add_object(oi);
                        }
                }
                break;
        case 'i': // init
                if (*(list + 6) == 'l') {
                        k->set_side(left_side);
                }
                else if (*(list + 6) == 'r') {
                        k->set_side(right_side);
                } else k->set_side(no_side);

                k->set_number(strtol(list + 8, &s, 10));
                k->set_goalie(goalie);
                k->set_status(judgement(&s));
                break;
        case 'a': // hear
                ma->clear();
                msgup = true;
                
                t = strtol(list + 6, &s, 10);
                set_time(t);

                int sender, msglen;

                char tmp[ MaxStringSize ];
                
                s += strspn(s, " ");
                if (strncmp(s, "referee", 7) == 0) {
                        msg.set(dir_referee);
                        sender = 1;
                        s += 8;
                }
                else if (strncmp(s, "self", 4) == 0) {
                        msg.set(dir_self);
                        sender = 0;
                        s += 5;
                } else {
                        msg.set(strtod(s, &s));
                        sender = 2;
                        s++;
                } 

                msglen = rindex(s, ')') - s;
                 
                if (msglen > 0) {
                        strncpy(tmp, s, msglen);
                        tmp[ msglen ] = '\0';
                        msg.set(tmp);

                        switch (sender) {
                        case 1 : // set status and add message
                                k->set_status(judgement(&s));
                        case 0 :

                        case 2 :
                                ma->add_message(msg);
                                break;
                        default:
                                s = rindex(s, ')');
                        }
                }
                break;
        case 'n': // sense_body
                set_time(strtol(list + 12, &s, 10));
                while (s != 0) {
                        if ((s = index(s, '(')) != 0) {
                                s++;
                                sense_body(&s);
                        }
                }
                break;
        default: // unknown string
                ROBOBUG1(ROBOBUG_WARN, LISTPROC_MEMORY,
                         "received unexpected string (%s)", list);
        }
        delete list;
        ROBOBUG(ROBOBUG_INFO, LISTPROC_MEMORY,
                "parse(const char *): delete list");
}

void Listproc::sense_body(char **stringp)
{
        if (*stringp == 0 || strlen(*stringp) < 4)
                return;

        ROBOBUG1(ROBOBUG_INFO, LISTPROC_MEMORY,
                 "sense_body (%s)", *stringp);

        // view_mode
        if ((*stringp)[3] == 'w') {
                (*stringp) += 10;
                
                if (!strncmp(*stringp, "low", 3)) {
                        k->set_view_quality(low);
                        (*stringp) += 4;
                }
                else if (!strncmp(*stringp, "high", 4)) {
                        k->set_view_quality(high);
                        (*stringp) += 5;
                }

                if (!strncmp(*stringp, "normal)", 7)) {
                        k->set_view_width(normal);
                        (*stringp) += 7;
                }
                else if (!strncmp(*stringp, "narrow)", 7)) {
                        k->set_view_width(narrow);
                        (*stringp) += 7;
                }
                else if (!strncmp(*stringp, "wide)", 5)) {
                        k->set_view_width(wide);
                        (*stringp) += 5;
                }

                if (!strncmp(*stringp, " (", 2))
                        (*stringp) += 2;
        }

        // stamina
        if ((*stringp)[3] == 'm') {
                k->set_stamina(strtol(*stringp + 8, stringp, 10));
                k->set_effort(strtod(*stringp, stringp));

                if (!strncmp(*stringp, ") (", 3))
                        (*stringp) += 3;
                else
                        (*stringp)++;
        }

        // speed
        if ((*stringp)[3] == 'e') {
                k->set_speed(strtod(*stringp + 6, stringp));

                if (!strncmp(*stringp, ") (", 3))
                        (*stringp) += 3;
                else
                        (*stringp)++;
        }

        // head_angle
        if ((*stringp)[3] == 'd') {
                k->set_neck(strtod(*stringp + 11, stringp));

                if (!strncmp(*stringp, ") (", 3))
                        (*stringp) += 3;
                else
                        (*stringp)++;
        }

        // kick
        if ((*stringp)[3] == 'k') {
                k->set_kicks(strtol(*stringp + 5, stringp, 10));

                if (!strncmp(*stringp, ") (", 3))
                        (*stringp) += 3;
                else
                        (*stringp)++;
        }

        // dash
        if ((*stringp)[3] == 'h') {
                k->set_dashs(strtol(*stringp + 5, stringp, 10));
                if (!strncmp(*stringp, ") (", 3))
                        (*stringp) += 3;
                else
                        (*stringp)++;
        }

        // turn
        if (((*stringp)[3] == 'n') && ((*stringp)[4] == ' ')) {
                k->set_turns(strtol(*stringp + 5, stringp, 10));

                if (!strncmp(*stringp, ") (", 3))
                        (*stringp) += 3;
                else
                        (*stringp)++;
        }        

        // say
        if ((*stringp)[3] == ' ') {        
                k->set_says(strtol(*stringp + 4, stringp, 10));

                if (!strncmp(*stringp, ") (", 3))
                        (*stringp) += 3;
                else
                        (*stringp)++;
        }

        // turn_neck
        if (((*stringp)[3] == 'n') && ((*stringp)[4] == '_')) {        
                k->set_turn_necks(strtol(*stringp + 10, stringp, 10));

                if (!strncmp(*stringp, ") (", 3))
                        (*stringp) += 3;
                else
                        (*stringp)++;
        }

        ROBOBUG1(ROBOBUG_INFO, LISTPROC_MEMORY,
                 "sense_body (%s)", *stringp);                 
}

ObjInfo Listproc::identify(char **stringp)
{
        ObjInfo oi(unknown);

        if (*stringp == 0) return oi;
        
        if (strncasecmp(*stringp, myname, mynamelen) == 0) {
                oi.set_name(partner);
                *stringp += mynamelen;
                if (**stringp == ' ')
                        oi.set_id((short) strtol(*stringp, stringp, 10));
                else
                        oi.set_id(id_unknown);
                (*stringp)++;
        }
        else if (strncmp(*stringp, "(player ", 8) == 0
                 || strncmp(*stringp, "(Player ", 8) == 0) { 
                oi.set_name(opponent);

                if (set_opname) {
                        size_t l = strcspn(*stringp + 8, " )");
                        char *othername = new char[l + 1];
                        strncpy(othername, *stringp + 8, l);
                        othername[l] = '\0';
                        k->set_teamname(othername, 1);
                        delete othername;
                        set_opname = false;
                }

                *stringp = strpbrk(*stringp + 8, " )");

                if (**stringp == ' ')
                        oi.set_id((short) strtol(*stringp, stringp, 10));
                else
                        oi.set_id(id_unknown);
                (*stringp)++;
        }
        else if (strncmp(*stringp, "(player)", 8) == 0
                 || strncmp(*stringp, "(Player)", 8) == 0) {
                oi.set_name(player);
                oi.set_id(id_unknown);
                *stringp += 8;
        }
        else if (strncmp(*stringp, "(goal", 5) == 0
                 || strncmp(*stringp, "(Goal", 5) == 0) {
                oi.set_name(goal);
                if (*(*stringp + 5) == ' ') {
                        if (*(*stringp + 6) == 'l')
                                oi.set_id(id_goal_left);
                        else
                                oi.set_id(id_goal_right);
                        *stringp += 8;
                } else {
                        oi.set_id(id_unknown);
                        *stringp += 6;
                }
        }
        else if (strncmp(*stringp, "(ball)", 6) == 0
                 || strncmp(*stringp, "(Ball)", 6) == 0) {
                oi.set_name(ball);
                oi.set_id(id_unknown);
                *stringp += 6;
        }
        else if (strncmp(*stringp, "(flag g ", 8) == 0) {
                oi.set_name(flag);
                switch (*(*stringp + 8)) {
                case 'l' :
                        if (*(*stringp + 10) == 't')
                                oi.set_id(id_gflag_l_t);
                        else
                                oi.set_id(id_gflag_l_b);
                        *stringp += 12;
                        break;
                case 'r' :
                        if (*(*stringp + 10) == 't')
                                oi.set_id(id_gflag_r_t);
                        else
                                oi.set_id(id_gflag_r_b);
                        *stringp +=12;
                        break;
                default :
                        oi.set_id(id_unknown);
                        *stringp = index(*stringp, ')') + 1;  
                }
        }
        else if (strncmp(*stringp, "(flag p ", 8) == 0) {
                oi.set_name(flag);
                switch (*(*stringp + 10)) {
                case 't' :
                        if (*(*stringp + 8) == 'l')
                                oi.set_id(id_pflag_l_t);
                        else
                                oi.set_id(id_pflag_r_t);
                        *stringp += 12;
                        break;
                case 'c' :
                        if (*(*stringp + 8) == 'l')
                                oi.set_id(id_pflag_l_c);
                        else
                                oi.set_id(id_pflag_r_c);
                        *stringp +=12;
                        break;
                case 'b' :
                        if (*(*stringp + 8) == 'l')
                                oi.set_id(id_pflag_l_b);
                        else
                                oi.set_id(id_pflag_r_b);
                        *stringp += 12;
                        break;
                default :
                        oi.set_id(id_unknown);
                        *stringp = index(*stringp, ')') + 1;  
                }
        }
        else if (strncmp(*stringp, "(flag ",6) == 0) {
                oi.set_name(flag);
                switch (*(*stringp + 6)) {
                // flag b
                case 'b' :
                        // flag b 0
                        switch (*(*stringp + 8)) {
                        case '0' :
                                oi.set_id(id_oflag_b_0);
                                *stringp += 10;
                                break;
                        // flag b l
                        case 'l' :
                                // flag b l 10
                                switch (*(*stringp + 10)) {
                                case '1' :
                                        oi.set_id(id_oflag_b_l_10);
                                        *stringp += 13;
                                        break;
                                // flag b l 20
                                case '2' :
                                        oi.set_id(id_oflag_b_l_20);
                                        *stringp += 13;
                                        break;
                                // flag b l 30
                                case '3' :
                                        oi.set_id(id_oflag_b_l_30);
                                        *stringp += 13;
                                        break;
                                // flag b l 40
                                case '4' :
                                        oi.set_id(id_oflag_b_l_40);
                                        *stringp += 13;
                                        break;
                                // flag b l 50
                                case '5' :
                                        oi.set_id(id_oflag_b_l_50);
                                        *stringp += 13;
                                        break;
                                default :
                                        oi.set_id(id_unknown);
                                        *stringp = index(*stringp, ')') + 1;
                                }
                                break;
                        // flag b r
                        case 'r' :
                                switch (*(*stringp + 10)) {
                                // flag b r 10
                                case '1' :
                                        oi.set_id(id_oflag_b_r_10);
                                        *stringp += 13;
                                        break;
                                // flag b r 20
                                case '2' :
                                        oi.set_id(id_oflag_b_r_20);
                                        *stringp += 13;
                                        break;
                                // flag b r 30
                                case '3' :
                                        oi.set_id(id_oflag_b_r_30);
                                        *stringp += 13;
                                        break;
                                // flag b r 40
                                case '4' :
                                        oi.set_id(id_oflag_b_r_40);
                                        *stringp += 13;
                                        break;
                                // flag b r 50
                                case '5' :
                                        oi.set_id(id_oflag_b_r_50);
                                        *stringp += 13;
                                        break;
                                default :
                                        oi.set_id(id_unknown);
                                        *stringp = index(*stringp, ')') + 1;
                                }
                                break;
                        default :
                                oi.set_id(id_unknown);
                                *stringp = index(*stringp, ')') + 1;
                        }
                        break;
                // flag c
                case 'c' :
                        // flag c
                        if (*(*stringp + 7) == ')') {
                                oi.set_id(id_flag_c);
                                *stringp += 8;
                        } else {
                                // flag c t / flag c b
                                if (*(*stringp + 8) == 't')
                                        oi.set_id(id_flag_c_t);
                                else
                                        oi.set_id(id_flag_c_b);
                                *stringp +=10;
                        }
                        break;
                // flag l
                case 'l' :
                        switch (*(*stringp + 8)) {
                        // flag l 0
                        case '0' :
                                oi.set_id(id_oflag_l_0);
                                *stringp += 10;
                                break;
                        // flag l t
                        case 't' :
                                // flag l t
                                if (*(*stringp + 9) == ')') {
                                        oi.set_id(id_flag_l_t);
                                        *stringp += 10;
                                } else {
                                        switch (*(*stringp + 10)) {
                                        // flag l t 10
                                        case '1' :
                                                oi.set_id(id_oflag_l_t_10);
                                                *stringp += 13;
                                                break;
                                        // flag l t 20
                                        case '2' :
                                                oi.set_id(id_oflag_l_t_20);
                                                *stringp += 13;
                                                break;
                                        // flag l t 30
                                        case '3' :
                                                oi.set_id(id_oflag_l_t_30);
                                                *stringp += 13;
                                                break;
                                        default :
                                                oi.set_id(id_unknown);
                                                *stringp = index(*stringp, ')')
                                                        + 1;
                                        }
                                }
                                break;
                                // flag l b 
                        case 'b' :
                                // flag l b
                                if (*(*stringp + 9) == ')') {
                                        oi.set_id(id_flag_l_b);
                                        *stringp += 10;
                                } else {
                                        switch (*(*stringp + 10)) {
                                        // flag l b 10
                                        case '1' :
                                                oi.set_id(id_oflag_l_b_10);
                                                *stringp += 13;
                                                break;
                                        // flag l b 20
                                        case '2' :
                                                oi.set_id(id_oflag_l_b_20);
                                                *stringp += 13;
                                                break;
                                        // flag l b 30
                                        case '3' :
                                                oi.set_id(id_oflag_l_b_30);
                                                *stringp += 13;
                                                break;
                                        default :
                                                oi.set_id(id_unknown);
                                                *stringp = index(*stringp, ')')
                                                        + 1;
                                        }
                                }
                                break;
                        default :
                                oi.set_id(id_unknown);
                                *stringp = index(*stringp, ')') + 1;
                        }
                        break;                        
                // flag r 
                case 'r' :
                        switch (*(*stringp + 8)) {
                        // flag r 0
                        case '0' :
                                oi.set_id(id_oflag_r_0);
                                *stringp += 10;
                                break;
                        // flag r t
                        case 't' :
                                // flag r t
                                if (*(*stringp + 9) == ')') {
                                        oi.set_id(id_flag_r_t);
                                        *stringp += 10;
                                } else {
                                        switch (*(*stringp + 10)) {
                                        // flag r t 10
                                        case '1' :
                                                oi.set_id(id_oflag_r_t_10);
                                                *stringp += 13;
                                                break;
                                        // flag r t 20
                                        case '2' :
                                                oi.set_id(id_oflag_r_t_20);
                                                *stringp += 13;
                                                break;
                                        // flag r t 30
                                        case '3' :
                                                oi.set_id(id_oflag_r_t_30);
                                                *stringp += 13;
                                                break;
                                        default :
                                                oi.set_id(id_unknown);
                                                *stringp = index(*stringp, ')')
                                                        + 1;
                                        }
                                }
                                break;
                        // flag r b 
                        case 'b' :
                                // flag r b
                                if (*(*stringp + 9) == ')') {
                                        oi.set_id(id_flag_r_b);
                                        *stringp += 10;
                                } else {
                                        switch (*(*stringp + 10)) {
                                        // flag r b 10
                                        case '1' :
                                                oi.set_id(id_oflag_r_b_10);
                                                *stringp += 13;
                                                break;
                                        // flag r b 20
                                        case '2' :
                                                oi.set_id(id_oflag_r_b_20);
                                                *stringp += 13;
                                                break;
                                        // flag r b 30
                                        case '3' :
                                                oi.set_id(id_oflag_r_b_30);
                                                *stringp += 13;
                                                break;
                                        default :
                                                oi.set_id(id_unknown);
                                                *stringp = index(*stringp, ')')
                                                        + 1;
                                        }
                                }
                                break;
                        default :
                                oi.set_id(id_unknown);
                                *stringp = index(*stringp, ')') + 1;
                        }
                        break;
                // flag t
                case 't' :
                        switch (*(*stringp + 8)) {
                        // flag t 0
                        case '0' :
                                oi.set_id(id_oflag_t_0);
                                *stringp += 10;
                                break;
                        // flag t l
                        case 'l' :
                                switch (*(*stringp + 10)) {
                                // flag t l 10
                                case '1' :
                                        oi.set_id(id_oflag_t_l_10);
                                        *stringp += 13;
                                        break;
                                // flag t l 20
                                case '2' :
                                        oi.set_id(id_oflag_t_l_20);
                                        *stringp += 13;
                                        break;
                                // flag t l 30
                                case '3' :
                                        oi.set_id(id_oflag_t_l_30);
                                        *stringp += 13;
                                        break;
                                // flag t l 40
                                case '4' :
                                        oi.set_id(id_oflag_t_l_40);
                                        *stringp += 13;
                                        break;
                                // flag t l 50
                                case '5' :
                                        oi.set_id(id_oflag_t_l_50);
                                        *stringp += 13;
                                        break;
                                default :
                                        oi.set_id(id_unknown);
                                        *stringp = index(*stringp, ')') + 1;
                                }
                                break;
                        // flag t r        
                        case 'r' :
                                switch (*(*stringp + 10)) {
                                // flag t r 10
                                case '1' :
                                        oi.set_id(id_oflag_t_r_10);
                                        *stringp += 13;
                                        break;
                                // flag t r 20
                                case '2' :
                                        oi.set_id(id_oflag_t_r_20);
                                        *stringp += 13;
                                        break;
                                // flag t r 30
                                case '3' :
                                        oi.set_id(id_oflag_t_r_30);
                                        *stringp += 13;
                                        break;
                                // flag t r 40
                                case '4' :
                                        oi.set_id(id_oflag_t_r_40);
                                        *stringp += 13;
                                        break;
                                // flag t r 50
                                case '5' :
                                        oi.set_id(id_oflag_t_r_50);
                                        *stringp += 13;
                                        break;
                                default :
                                        oi.set_id(id_unknown);
                                        *stringp = index(*stringp, ')') + 1;
                                }
                                break;
                        default :
                                oi.set_id(id_unknown);
                                *stringp = index(*stringp, ')') + 1;
                        }
                        break;
                default :
                        oi.set_id(id_unknown);
                        *stringp = index(*stringp, ')') + 1;  
                }
        }
        else if (strncmp(*stringp, "(flag)", 6) == 0
                 || strncmp(*stringp, "(Flag)", 6) == 0) {
                oi.set_name(flag);
                oi.set_id(id_unknown);
                *stringp += 6;
        }
        else if (strncmp(*stringp, "(line ", 6) == 0) {
                oi.set_name(line);
                switch (*(*stringp + 6)) {
                case 'l' :
                        oi.set_id(id_line_l);
                        *stringp += 8;
                        break;
                case 'r' :
                        oi.set_id(id_line_r);
                        *stringp += 8;
                        break;
                case 't' :
                        oi.set_id(id_line_t);
                        *stringp += 8;
                        break;
                case 'b' :
                        oi.set_id(id_line_b);
                        *stringp += 8;
                        break;
                default :
                        oi.set_id(id_unknown);
                        *stringp = index(*stringp, ')') + 1;
                }
        }
        else if (strncmp(*stringp, "(line)", 6) == 0
                 || strncmp(*stringp, "(Line)", 6) == 0) {
                oi.set_name(line);
                oi.set_id(id_unknown);
                *stringp += 6;
        } else {
                oi.set_name(unknown);
                oi.set_id(id_unknown);
                if ((*stringp = index(*stringp, ')')) != 0)
                        (*stringp)++;
        }

        return oi;
}

void Listproc::bearing(char **stringp, ObjInfo &oi) 
{
        oi.set_xy(bearing_unknown, bearing_unknown);

        char *p;
        double d;

        if (*stringp != 0) {
                d = strtod(*stringp, &p);
                if ((p != 0) && p[0] == ')') {
                        oi.set_dist(bearing_unknown);
                        oi.set_dir(d);
                        *stringp = p;
                } else {
                        oi.set_dist(d);
     
                        if (p == *stringp)
                                oi.set_dist(bearing_unknown);
                        else
                                *stringp = p;
        
                        if (*stringp != 0) {
                                oi.set_dir(strtod(*stringp, &p), true);
                        }
                        else
                                p = 0;
        
                        if (p == *stringp)
                                oi.set_dir(bearing_unknown);
                        else
                                *stringp = p;
                }
        }

        if (*stringp != 0)
                oi.set_ddist(strtod(*stringp, &p));
        else
                p = 0;
        
        if (p == *stringp)
                oi.set_ddist(bearing_unknown);
        else
                *stringp = p;


        if (*stringp != 0)
                oi.set_ddir(strtod(*stringp, &p), true);
        else
                p = 0;
        
        if (p == *stringp)
                oi.set_ddir(bearing_unknown);
        else
                *stringp = p;

        
        if (*stringp != 0)
                oi.set_bodydir(strtod(*stringp, &p));
        else
                p = 0;
        
        if (p == *stringp)
                oi.set_bodydir(bearing_unknown);
        else
                *stringp = p;

        
        if (*stringp != 0)
                oi.set_facedir(strtod(*stringp, &p));
        else
                p = 0;
        
        if (p == *stringp)
                oi.set_facedir(bearing_unknown);
        else
                *stringp = p;
        
        
        if (*stringp != 0)
                if ((*stringp = index(*stringp, ')')) != 0)
                        (*stringp)++;        
}

int Listproc::judgement(char **stringp)
{
        int st = 0;

        if (strncmp(*stringp, "before_kick_off", 15) == 0)
                st = st_before_kick_off;
        if (strncmp(*stringp, "kick_off_l", 10) == 0)
                st = st_kick_off_l;
        if (strncmp(*stringp, "kick_off_r", 10) == 0)
                st = st_kick_off_r;
        if (strncmp(*stringp, "kick_in_l", 9) == 0)
                st = st_kick_in_l;
        if (strncmp(*stringp, "kick_in_r", 9) == 0)
                st = st_kick_in_r;
        if (strncmp(*stringp, "corner_kick_l", 13) == 0)
                st = st_corner_kick_l;
        if (strncmp(*stringp, "corner_kick_r", 13) == 0)
                st = st_corner_kick_r;
        if (strncmp(*stringp, "goal_kick_l", 11) == 0)
                st = st_goal_kick_l;
        if (strncmp(*stringp, "goal_kick_r", 11) == 0)
                st = st_goal_kick_r;
        if (strncmp(*stringp, "free_kick_l", 11) == 0)
                st = st_free_kick_l;
        if (strncmp(*stringp, "free_kick_r", 11) == 0)
                st = st_free_kick_r;
        if (strncmp(*stringp, "play_on", 7) == 0)
                st = st_play_on;
        if (strncmp(*stringp, "half_time", 9) == 0)
                st = st_half_time;
        if ((strncmp(*stringp, "time_up", 7) == 0)
            || (strncmp(*stringp, "time_over", 9) == 0))
                st = st_time_up;
        if (strncmp(*stringp, "extend", 6) == 0)
                st = st_extend;
        if (strncmp(*stringp, "foul_l", 6) == 0)
                st = st_foul_l_;
        if (strncmp(*stringp, "foul_r", 6) == 0)
                st = st_foul_r_;
        if (strncmp(*stringp, "goalie_catch_ball_l", 19) == 0)
                st = st_goalie_catch_ball_l;
        if (strncmp(*stringp, "goalie_catch_ball_r", 19) == 0)
                st = st_goalie_catch_ball_r;

        if (strncmp(*stringp, "goal_l", 6) == 0) {
                st = st_goal_l_;
                k->set_goals(0, atoi(*stringp + 7));
        }
        if (strncmp(*stringp, "goal_r", 6) == 0 ) {
                st = st_goal_r_;
                k->set_goals(1, atoi(*stringp + 7));
        }

        *stringp = strpbrk(*stringp, " )");
        
        return st;
}
