/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   Copyright (C) Nov 09 1997 Oliver Obst
   $Id: listproc.h,v 1.4 1999/05/05 14:05:25 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _LISTPROC_H
#define _LISTPROC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "constants.h"
#include "objinfo.h"
#include "objarray.h"
#include "msgarray.h"
#include "knowledge.h"

/**
 * A list processor for parsing and storing server data.
 *
 * @author      Oliver Obst <frvit@uni-koblenz.de>
 * @version     $Id: listproc.h,v 1.4 1999/05/05 14:05:25 frvit Exp $
 * @short       A list processor for parsing and storing server data.
 **/ 
class Listproc
{
public:
        Listproc();
        Listproc(ObjArray *oa0, MsgArray *ma0, Knowledge *k0);
        Listproc(const Listproc &l);
        virtual ~Listproc();

        void    set_teamname(const char *name, bool keeper);
        void    set_base(ObjArray *oa0, MsgArray *ma0, Knowledge *k0);
        void    parse(const char *string);
        void    sense_body(char **stringp);

        char    *get_teamname(bool init=true);
        
        inline  void    set_time(int t) { time = t; }
        inline  void    set_moved(bool move=true) { moved=move; }
        inline  void    set_version(double v) { version = v; }

        inline  int     get_time() { return time; }
        inline  bool    get_moved() { return moved; }
        inline  double  get_version() const { return version; }
        inline  bool    get_newobj()
                { bool b = objup; objup=false; return b; }
        inline  bool    get_newmsg()
                { bool b = msgup; msgup=false; return b; }

        inline  void    dash(double power) { k->dash(power); }
        
protected:
        ObjInfo identify(char **stringp);
        void    bearing(char **stringp, ObjInfo &oi);
        int     judgement(char **stringp);

private:
        ObjArray        *oa;
        MsgArray        *ma;
        Knowledge       *k;

        char    myname[ TeamnameLen + 9 ];
        int     mynamelen;
        double  version;
        int     time;
        bool    moved;
        bool    msgup;
        bool    objup;
        bool    goalie;
        bool    set_opname;
};

#endif /* _LISTPROC_H */
