/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Sep 01 1998 Oliver Obst
   array to store information about messages
   $Id: msgarray.C,v 1.2 1999/05/05 14:12:06 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "msginfo.h"
#include "msgarray.h"

// --- public constructor -----------------------------------------------------
MsgArray::MsgArray(int msize)
{
        size = msize;
        entries = refmsgs = ownmsgs = 0;
        messages = new MsgInfo[size];
        emsg = new MsgInfo;
}

// --- public copy constructor ------------------------------------------------
MsgArray::MsgArray(const MsgArray &ma)
{
        size = ma.size;
        entries = ma.entries;
        refmsgs = ma.refmsgs;
        ownmsgs = ma.ownmsgs;
        emsg = new MsgInfo;
        messages = new MsgInfo[size];
        *emsg = *ma.emsg;
        for (int i=0; i < entries; i++)
                messages[i] = ma.messages[i];
}

// --- destructor -------------------------------------------------------------
MsgArray::~MsgArray()
{
        delete emsg;
        delete[] messages;
}

// --- public -----------------------------------------------------------------
bool MsgArray::add_message(MsgInfo &m)
{
        if (entries < size) {
                messages[entries] = m;
                if (m.dir() == dir_self) ownmsgs++;
                else if (m.dir() == dir_referee) refmsgs++;
                entries++;
                return true;
        }
        else return false;
}

// --- public -----------------------------------------------------------------
void MsgArray::clear()
{
        entries = ownmsgs = refmsgs = 0;
}

// --- public -----------------------------------------------------------------
MsgInfo& MsgArray::referee(int i) const
{
        if (i < refmsgs) {
                int j = -1;
                int k = -1;
                while (j < i) {
                        k++;
                        if (messages[k].dir() == dir_referee) j++;
                }
                return messages[k];
        }
        else return *emsg;
}

// --- public -----------------------------------------------------------------
MsgInfo& MsgArray::self(int i) const
{
        if (i < ownmsgs) {
                int j = -1;
                int k = -1;
                while (j < i) {
                        k++;
                        if (messages[k].dir() == dir_self) j++;
                }
                return messages[k];
        }
        else return *emsg;
}
                

// --- public -----------------------------------------------------------------
MsgInfo& MsgArray::others(int i) const
{
        if (i < entries - ownmsgs - refmsgs) {
                int j = -1;
                int k = -1;
                while (j < i) {
                        k++;
                        if (messages[k].dir() < 360.0) j++;
                }
                return messages[k];
        }
        else return *emsg;
}
                

// --- public operator --------------------------------------------------------
MsgInfo& MsgArray::operator[](int i) const
{
        if (i < entries) return messages[i];
        else return *emsg;
}

// --- public operator --------------------------------------------------------
MsgArray& MsgArray::operator=(const MsgArray &ma)
{
        size = ma.size;
        entries = ma.entries;
        refmsgs = ma.refmsgs;
        ownmsgs = ma.ownmsgs;
        *emsg = *ma.emsg;
        for (int i=0; i < entries; i++)
                messages[i] = ma.messages[i];
        return (*this);
}

#if 0
// --- friend operator --------------------------------------------------------
ostream& operator <<(ostream &os, MsgArray &ma)
{
        int i;
        os << "[";
        for (i=0; i < ma.get_entries(); i++) {
                os << ma.messages[i];
                if (i+1 < ma.get_entries())
                        os << ", ";
        }
        os << "]";
        
        return os;
}
#endif 
