/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Apr 08 1998 Oliver Obst
   class to store information about a single message
   $Id: msginfo.C,v 1.2 1999/05/05 14:13:46 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if STDC_HEADERS
# include <string.h>
#endif
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#include "msginfo.h"

// --- public constructor -----------------------------------------------------
MsgInfo::MsgInfo(const char *text, double dir)
{
        if (text == 0 || strlen(text) == 0)
                message = 0;
        else message = strdup(text);
        direction = dir;
}


// --- public constructor -----------------------------------------------------
MsgInfo::MsgInfo(const MsgInfo &m)
{
        //	cerr << "MsgInfo copy constructor ";
        if (m.message == 0) message = 0;
        else message = strdup(m.message);
//        //	cerr << ">\n>" << m.message << "<\n";
        direction = m.direction;
        //	cerr << "finished\n";
}


// --- public destructor ------------------------------------------------------
MsgInfo::~MsgInfo()
{
        delete message;
}


// --- public -----------------------------------------------------------------
MsgInfo& MsgInfo::set(const MsgInfo &m)
{
        if (message != 0) delete message;
        if (m.message != 0) message = strdup(m.message);
        else message = 0;
//        cerr << ">\n>" << m.message << "<\n";
        direction = m.direction;
        return *this;
}

// --- public -----------------------------------------------------------------
MsgInfo& MsgInfo::set(const char *text)
{
        if (message != 0) delete message;
        if (text != 0 && strlen(text) > 0) message = strdup(text);
        else message = 0;
//        cerr << ">\n>" << message << "<\n";
        return *this;
}

// --- public -----------------------------------------------------------------
MsgInfo& MsgInfo::set(double dir)
{
        direction = dir;
        return *this;
}

// --- public -----------------------------------------------------------------
MsgInfo& MsgInfo::set(const char *text, double dir)
{
        if (message != 0) delete message;
        if (text != 0 && strlen(text) > 0) message = strdup(text);
        else message = 0;
//        cerr << ">\n>" << message << "<\n";
        direction = dir;
        return *this;
}


// --- public -----------------------------------------------------------------
void MsgInfo::get(char *text, double *dir) const
{
        if (message == 0) strcpy(text, "");
        else strcpy(text, message);

//        cerr << ":" << message << ".\n";

        *dir = direction;
}

// --- public -----------------------------------------------------------------
MsgInfo& MsgInfo::operator =(const MsgInfo &m)
{
        //	cerr << "* MsgInfo operator = <" << m << ">\n";
        //	cerr << "*   m.message == 0 ? ";
        delete message;
        if (m.message == 0) {
                //	cerr << "jau.\n";
                message = 0;
        } else {
                //	cerr << "n.\n";
                message = strdup(m.message);
        }
        //	cerr << "*   message copied, now direction..." << m.dir() << "\n";
        direction = m.dir();
        //	cerr << "* finished\n";
        return *this;
}

// --- friend operator --------------------------------------------------------
int operator ==(const MsgInfo &m0, const MsgInfo &m1)
{
        return (!strcmp(m0.message, m1.message)
                && m0.direction == m1.direction);
}

#if 0
// --- friend operator --------------------------------------------------------
ostream& operator <<(ostream &os, MsgInfo &mi)
{
        return os << "(" << mi.message << " (" << mi.direction << "))";
}
#endif
