/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Apr 08 1998 Oliver Obst
   class to store information about a single message
   $Id: msginfo.h,v 1.2 1999/05/05 14:14:35 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _MSGINFO_H
#define _MSGINFO_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "constants.h"

/**
 * A class to store information about a message.
 *
 * This class stores the message text and the direction of the message.
 * Direction is between -180 and 180 for other players messages or
 * <code>self</code> for the clients own messages and <code>referee</code>
 * for referee messages.
 *
 * @author      Oliver Obst <frvit@uni-koblenz.de>
 * @version     $Id: msginfo.h,v 1.2 1999/05/05 14:14:35 frvit Exp $
 * @short       A class to store information about a message.
 **/
class MsgInfo
{
public:
/**
 * MsgInfo constructor.
 *
 * @param text          the message text
 * @param dir           the direction of the message
 **/       
        MsgInfo(const char *text=0, double dir=bearing_unknown);
/**
 * MsgInfo copy constructor.
 *
 * @param m             other MsgInfo.
 **/
        MsgInfo(const MsgInfo &m);
/**
 * Destructor. Free allocated memory.
 **/
        virtual ~MsgInfo();

/**
 * Sets the MsgInfo.
 *
 * @param m             MsgInfo.
 * @return              MsgInfo.
 **/
        MsgInfo&        set(const MsgInfo &m);

/**
 * Sets the MsgInfo text.
 *
 * @param text          Message text.
 * @return              MsgInfo.
 **/
        MsgInfo&        set(const char *text);

/**
 * Sets the MsgInfo direction.
 *
 * @param dir           Message direction
 * @return              MsgInfo.
 **/
        MsgInfo&        set(double dir);

/**
 * Sets the MsgInfo.
 *
 * @param text          Message text.
 * @param dir           Message direction
 * @return              MsgInfo.
 **/
        MsgInfo&        set(const char *text, double dir);
        
/**
 * Gets the MsgInfo text and direction.
 *
 * @param text          Message text.
 * @param dir           Message direction.
 **/
        void    get(char *text, double *dir) const;
        
/**
 * Gets the MsgInfo text.
 *
 * @param text          Message text.
 **/
        void    text(char *txt) const;
        
/**
 * Gets the MsgInfo direction.
 *
 * @return              message direction.
 **/
        inline double   dir() const { return direction; }

/**
 * Sets the MsgInfo.
 * @see msginfo#set
 *
 * @param m             MsgInfo.
 * @return              MsgInfo.
 **/
        MsgInfo& operator =(const MsgInfo &m);

/**
 * Compares two MsgInfo's.
 *
 * @param m0            MsgInfo.
 * @param m1            MsgInfo.
 * @return              <code>true</code> if both the messages and the
 * directions are equal.
 **/
        friend int operator ==(const MsgInfo &m0, const MsgInfo &m1);

#if 0
/**
 * print MsgInfo to output
 *
 * @return 
 */
        friend ostream& operator <<(ostream &os, MsgInfo &mi);
#endif           
        
private:
        char    *message;
        double  direction;
};

#endif /* _MSGINFO_H */
