/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (c) Nov 09 1997 Oliver Obst
   object array to store information about objects
   $Id: objarray.C,v 1.4 1999/05/05 14:17:15 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "debug.h"
#include "objarray.h"

// --- public constructor -----------------------------------------------------
ObjArray::ObjArray(int)
{
        clear();
}

// --- public destructor ------------------------------------------------------
ObjArray::~ObjArray()
{
}

// --- public -----------------------------------------------------------------
bool ObjArray::add_object(const ObjInfo &oi)
{
        bool ok = true;
        
        switch (oi.name()) {
        case ball :
                if (n_balls < max_balls) {
                        oi_balls[n_balls] = oi;
                        n_balls++;
                }
                else ok = false;
                break;
        case player :
                if (n_players < max_players) {
                        oi_players[n_players] = oi;
                        n_players++;
                }
                else ok = false;
                break;
        case partner :
                if (n_partners < max_teamplayers) {
                        oi_partners[n_partners] = oi;
                        n_partners++;
                }
                else ok = false;
                break;
        case opponent :
                if (n_opponents < max_teamplayers) {
                        oi_opponents[n_opponents] = oi;
                        n_opponents++;
                }
                else ok = false;
                break;
        case goal :
                if (n_goals < max_goals) {
                        oi_goals[n_goals] = oi;
                        n_goals++;
                        if (oi.id() > id_unknown) n_kgoals++;
                }
                else ok = false;
                break;
        case flag :
                if (n_flags < max_flags) {
                        oi_flags[n_flags] = oi;
                        n_flags++;
                        if (oi.id() > id_unknown) n_kflags++;
                }
                else ok = false;
                break;
        case line :
                if (n_lines < max_lines) {
                        oi_lines[n_lines] = oi;
                        n_lines++;
                        if (oi.id() > id_unknown) n_klines++;
                }
                else ok = false;
                break;
        default :
                if (n_unknowns < max_unknowns) {                        
                        oi_unknowns[n_unknowns] = oi;
                        n_unknowns++;
                }
                else ok = false;
        }

        return ok;
}

// --- public -----------------------------------------------------------------
void ObjArray::clear()
{
        entries = 0;
        n_unknowns = 0;
        n_lines = n_flags = n_goals = 0;
        n_klines = n_kflags = n_kgoals = 0;
        n_balls = n_players = n_partners = n_opponents = 0;

        pos.set(bearing_unknown, bearing_unknown);
        bodyangle = headangle = bearing_unknown;
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::moving(int i)
{
        if (i < get_moving()) return (*this)[i];
        else {
                ROBOBUG2(ROBOBUG_ERROR, OBJARRAY_MOVING,
                         "segfault (i is %i, should be less than %i)",
                         i, get_moving());
                return oi_none;
        }        
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::nonmoving(int i)
{
        if (i < get_nonmoving()) return (*this)[get_moving() + i];
        else {
                ROBOBUG2(ROBOBUG_ERROR, OBJARRAY_NONMOV,
                         "segfault (i is %i, should be less than %i)",
                         i, get_nonmoving());
                return oi_none;
        } 
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::lines(int i)
{
        if (i < n_lines) return oi_lines[i];
        else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_LINES,                
                         "segfault (%i)", i);
                return oi_none;
        }
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::flags(int i)
{
        if (i < n_flags) return oi_flags[i];
        else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_FLAGS,
                         "segfault (%i)", i);
                return oi_none;
        }
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::goals(int i)
{
        if (i < n_goals) return oi_goals[i];
        else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_GOALS,                
                         "segfault (%i)", i);
                return oi_none;
        }
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::balls(int i)
{
        if (i < n_balls) return oi_balls[i];
        else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_BALLS,
                         "segfault (%i)", i);
                return oi_none;
        }
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::points(int i)
{
        if (i < n_flags + n_goals) {
                if (i < n_flags) return oi_flags[i];
                else return oi_goals[i - n_flags];
        } else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_POINTS,
                         "segfault (%i)", i);
                return oi_none;
        }
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::players(int i)
{
        if (i < n_players) return oi_players[i];
        else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_PLAYER,
                         "segfault (%i)", i);
                return oi_none;
        }
}

// --- public -----------------------------------------------------------------
ObjInfo& ObjArray::partners(int i)
{
        if (i < n_partners) return oi_partners[i];
        else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_PARTNR,
                         "segfault (%i)", i);
                return oi_none;
        }
}

ObjInfo& ObjArray::opponents(int i)
{
        if (i < n_opponents) return oi_opponents[i];
        else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_OPPON,
                         "segfault (%i)", i);
                return oi_none;
        }
}

ObjInfo& ObjArray::unknowns(int i)
{
        if (i < n_unknowns) return oi_unknowns[i];
        else {
                ROBOBUG1(ROBOBUG_ERROR, OBJARRAY_UNKNOW,
                         "segfault (%i)", i);
                return oi_none;
        }
}

// --- public operator --------------------------------------------------------
// return a single object
ObjInfo& ObjArray::operator[](int i)
{
        int k;
        int j = n_lines;
        if (i < j) return oi_lines[i];

        k = n_lines;
        j += n_flags;
        if (i < j) return oi_flags[i - k];

        k += n_flags;
        j += n_goals;
        if (i < j) return oi_goals[i - k];
        
        k += n_goals;
        j += n_players;
        if (i < j) return oi_players[i - k];

        k += n_players;
        j += n_balls;
        if (i < j) return oi_balls[i - k];

        k += n_balls;
        j += n_partners;
        if (i < j) return oi_partners[i - k];

        k += n_partners;
        j += n_opponents;
        if (i < j) return oi_opponents[i - k];

        k += n_opponents;
        j += n_unknowns;
        if (i < j) return oi_unknowns[i - k];

        ROBOBUG2(ROBOBUG_ERROR, OBJARRAY_SQRBRK,
                 "segfault (%i, should be less than %i)", i, entries);
        return oi_none;
}

// --- public -----------------------------------------------------------------
inline Cvector ObjArray::mypos()
{
        return pos;
}

// --- public -----------------------------------------------------------------
inline double ObjArray::mybodyangle()
{
        return bodyangle == bearing_unknown ? bodyangle : rad2deg(bodyangle);
}

// --- public -----------------------------------------------------------------
inline double ObjArray::myheadangle()
{
        return headangle == bearing_unknown ? headangle : rad2deg(headangle);
}

// --- public -----------------------------------------------------------------
//      double  xpos, ypos, beta       own absolute position
void ObjArray::calc_abs(double xpos, double ypos, double alpha, double beta)
{
        pos.set(xpos, ypos);
        bodyangle = alpha;
        headangle = beta;

        if (xpos < -9000 || ypos < -9000 || beta < -9000) return;

        int i;
        for (i=0; i < n_balls; i++)
                oi_balls[i].calc_abs(xpos, ypos, beta);
        for (i=0; i < n_players; i++)
                oi_players[i].calc_abs(xpos, ypos, beta);
        for (i=0; i < n_partners; i++)
                oi_partners[i].calc_abs(xpos, ypos, beta);
        for (i=0; i < n_opponents; i++)
                oi_opponents[i].calc_abs(xpos, ypos, beta);
}
