/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   array to store information about objects
   Copyright (C) Nov 09 1997 Oliver Obst
   $Id: objarray.h,v 1.3 1999/05/05 14:15:44 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _OBJARRAY_H
#define _OBJARRAY_H

#include "constants.h"
#include "soccerconst.h"
#include "maths.h"
#include "objinfo.h"

class ObjArray
{
public:
        ObjArray(int i=0);
        virtual ~ObjArray();

        bool    add_object(const ObjInfo &oi);
        void    clear();


        inline  int     get_entries() const { return entries; }
        inline  int     get_moving() const
                { return n_players + n_partners + n_opponents + n_balls; }
        inline  int     get_nonmoving() const
                { return n_flags + n_goals + n_lines; }
        inline  int     get_lines() const { return n_lines; }
        inline  int     get_known_lines() const { return n_klines; }
        
        inline  int     get_points() const { return n_flags + n_goals; }
        inline  int     get_known_points() const { return n_kflags+n_kgoals; }
                        
        inline  int     get_flags() const { return n_flags; }
        inline  int     get_known_flags() const { return n_kflags; }
        inline  int     get_goals() const { return n_goals; }
        inline  int     get_known_goals() const { return n_kgoals; }
        inline  int     get_balls() const { return n_balls; }
        inline  int     get_partners() const { return n_partners; }
        inline  int     get_opponents() const { return n_opponents; }
        inline  int     get_players() const { return n_players; }
        inline  int     get_unknowns() const { return n_unknowns; }

        ObjInfo& moving(int i);
        ObjInfo& nonmoving(int i);
        ObjInfo& lines(int i);
        ObjInfo& flags(int i);
        ObjInfo& goals(int i);
        ObjInfo& points(int i);
        ObjInfo& balls(int i);
        ObjInfo& partners(int i);
        ObjInfo& opponents(int i);
        ObjInfo& players(int i);
        ObjInfo& unknowns(int i);
        
        ObjInfo& operator[](int i);

        Cvector mypos();
        double  mybodyangle();
        double  myheadangle();
        
        void    calc_abs(double xpos, double ypos, double alpha, double beta);
        
protected:
        // for backward compatibility
        inline void    add_moving(ObjInfo &oi) { add_object(oi); }
        inline void    add_nonmoving(ObjInfo &oi) { add_object(oi); }
        inline void    delete_all() { clear(); }
        
private:
	static const int max_unknowns = 10;
        
        int     entries;

        int     n_lines, n_klines;
        int     n_flags, n_kflags;
        int     n_goals, n_kgoals;
        int     n_balls;
        int     n_players;
        int     n_partners;
        int     n_opponents;
        int     n_unknowns;
        
        ObjInfo oi_lines[max_lines];
        ObjInfo oi_flags[max_flags];
        ObjInfo oi_goals[max_goals];
        ObjInfo oi_balls[max_balls];
        ObjInfo oi_players[max_players];
        ObjInfo oi_partners[max_teamplayers];
        ObjInfo oi_opponents[max_teamplayers];
        ObjInfo oi_unknowns[max_unknowns];

        ObjInfo oi_none;
        Cvector pos;
        double  bodyangle;
        double  headangle;
};
#endif /* _OBJARRAY_H */

