/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Apr 08 1998 Oliver Obst
   class to store information about a single object
   $Id: objinfo.C,v 1.3 1999/05/05 14:21:19 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if STDC_HEADERS
# include <string.h>
#endif
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#include "objinfo.h"

ObjInfo::ObjInfo(Objtype name, short id, double dist, double dir,
                 double ddist, double ddir,
                 double bodydir, double facedir, double x, double y)
{
        myname  = name;
        myid    = id;
        mybodydir = bodydir;
        myfacedir = facedir;
        set_dist(dist, dir, true);
        set_ddist(ddist, ddir, true);
        set_xy(x, y);
}


ObjInfo::~ObjInfo()
{
}


// --- public -----------------------------------------------------------------
inline void ObjInfo::set_name(const Objtype name)
{
        myname = name;
}


// --- public -----------------------------------------------------------------
inline void ObjInfo::set_id(const short id)
{
        myid = id;
}

// --- public -----------------------------------------------------------------
inline void ObjInfo::set_bodydir(const double dir)
{
        mybodydir = dir;
}

// --- public -----------------------------------------------------------------
inline void ObjInfo::set_facedir(const double dir)
{
        myfacedir = dir;
}


// --- public -----------------------------------------------------------------
void ObjInfo::set_dir(double dir, bool deg)
{
        mydist.set_dir(dir, deg);
}


// --- public -----------------------------------------------------------------
void ObjInfo::set_dist(const double dist)
{
        mydist.set_dist(dist);
}


// --- public -----------------------------------------------------------------
void ObjInfo::set_dist(double dist, double dir, bool deg)
{
        mydist.set(dist, dir, deg);
}


// --- public -----------------------------------------------------------------
inline void ObjInfo::set_dist(const Pvector &d)
{
        mydist = d;
}


// --- public -----------------------------------------------------------------
void ObjInfo::set_ddir(double ddir, bool deg)
{
        myddist.set_dir(ddir, deg);
}


// --- public -----------------------------------------------------------------
void ObjInfo::set_ddist(const double ddist)
{
        myddist.set_dist(ddist);
}


// --- public -----------------------------------------------------------------
void ObjInfo::set_ddist(double ddist, double ddir, bool deg)
{
        myddist.set(ddist, ddir, deg);
}


// --- public -----------------------------------------------------------------
inline void ObjInfo::set_ddist(const Pvector &dd)
{
        myddist = dd;
}


// --- public -----------------------------------------------------------------
void ObjInfo::set_xy(const double x, const double y)
{
        myxy.set(x, y);
}


// --- public -----------------------------------------------------------------
inline void ObjInfo::set_xy(const Cvector &xy)
{
        myxy = xy;
}


// --- public -----------------------------------------------------------------
void ObjInfo::set_unknown()
{
        myid = id_unknown;
        mydist.set(bearing_unknown, bearing_unknown, true);
        myddist.set(bearing_unknown, bearing_unknown, true);
        myxy.set(bearing_unknown, bearing_unknown);
        mybodydir = bearing_unknown; myfacedir = bearing_unknown;
}


// --- public -----------------------------------------------------------------
// calculate absolute position, xpos, ypos client position,
// beta client view angle (rad) 
void ObjInfo::calc_abs(double xpos, double ypos, double beta)
{
        double alpha = mydist.dir(false) + beta;
        myxy.set(cos(alpha) * mydist.dist() + xpos,
                 sin(alpha) * mydist.dist() + ypos);        
}


// --- public -----------------------------------------------------------------
inline Objtype ObjInfo::name() const
{
        return myname;
}


// --- public -----------------------------------------------------------------
inline short ObjInfo::id() const
{
        return myid;
}


// --- public -----------------------------------------------------------------
inline double ObjInfo::dist() const
{
        return mydist.dist();
}


// --- public -----------------------------------------------------------------
inline double ObjInfo::dir(bool deg)
{
        return mydist.dir(deg);
}


// --- public -----------------------------------------------------------------
inline double ObjInfo::ddist() const
{
        return myddist.dist();
}


// --- public -----------------------------------------------------------------
inline double ObjInfo::ddir(bool deg)
{
        return myddist.dir(deg);
}

// --- public -----------------------------------------------------------------
inline double ObjInfo::bodydir()
{
        return mybodydir;
}

// --- public -----------------------------------------------------------------
inline double ObjInfo::facedir()
{
        return myfacedir;
}


// --- public -----------------------------------------------------------------
inline double ObjInfo::x()
{
        return myxy.x();
}


// --- public -----------------------------------------------------------------
inline double ObjInfo::y()
{
        return myxy.y();
}


// --- public operator --------------------------------------------------------
ObjInfo& ObjInfo::operator =(const ObjInfo &oi)
{
        myname  = oi.myname;
        myid    = oi.myid;
        mydist  = oi.mydist;
        myddist = oi.myddist;
        mybodydir = oi.mybodydir;
        myfacedir = oi.myfacedir;        
        myxy    = oi.myxy;
        return *this;
}


// --- friend operator --------------------------------------------------------
// cannot use `const ObjInfo ...', because Pvector operator == may change
// its operands
int operator ==(ObjInfo &oi0, ObjInfo &oi1)
{
        return (oi0.myname == oi1.myname &&
                oi0.myid == oi1.myid &&
                oi0.mydist == oi1.mydist &&
                oi0.myddist == oi1.myddist &&
                oi0.mybodydir == oi1.mybodydir &&
                oi0.myfacedir == oi1.myfacedir &&                
                oi0.myxy == oi1.myxy );
}

// --- friend operator --------------------------------------------------------
ostream& operator <<(ostream &os, ObjInfo &oi)
{
        os << "(";
        
        switch (oi.name()) {
        case unknown :
                os << "unknown_";
                break;
        case ball :
                os << "ball_";
                break;
        case player :
                os << "player_";
                break;
        case partner :
                os << "partner_";
                break;
        case opponent :
                os << "opponent_";
                break;
        case goal :
                os << "goal_";
                break;
        case flag :
                os << "flag_";
                break;
        case line :
                os << "line_";
                break;
        }
        
        return os << oi.id() << ", Dst: " << oi.dist() << ", Dir: "
                  << oi.dir() << ")";
}

#if 0
// Object ---------------------------------------------------------------------

double Object::dist() const
{
        if ((abs_valid() > 0)) /* && Mem->MyConf() */
                return Mem->DistanceTo(gpos); /* From global   */

        /* dump_core("dump"); */
        my_error("Can't get_dist %d",type);
        return 0;

        if (pdtime < Mem->CurrentTime-1)              /* In sight      */
                my_error("Can't get_dist %d",type);

        printf("Using in-sight estimate of dist\n");
        return dist;                                    /* From relative */ 
}

/********************************************************************************/

AngleDeg Object::get_ang()
{
  if ( pos_valid() || Mem->MyConf() )     
    return Mem->AngleTo(gpos);                                /* From global   */

  my_error("Can't get_ang %d",type);
  return 0;

  if ( patime < Mem->CurrentTime-1 )
    my_error("Can't get_ang %d",type);

  /* In sight */
  printf("Using in-sight estimate of ang\n");
  return ang;                                                 /* From relative */  
}

/********************************************************************************/

Vector Object::get_rel_pos() /* relative */
{
  if ( rtime != Mem->CurrentTime ){
    rpos = (gpos - Mem->MyPos()).rotate(-Mem->MyAng());
    rtime = Mem->CurrentTime;
  }
  return rpos;
}

/********************************************************************************/

void Object::set_polar(float d, float a, Time time)
{ 
  dist = d;
  ang  = a;
  pdtime = patime = time;
  seen = TRUE;
  seen_time = time;
}

/********************************************************************************/

void Object::set_angle(AngleDeg a, Time time)
{
  ang = a;
  patime = time;
  seen = TRUE;
  seen_time = time;
}

/********************************************************************************/

void Object::set_chinfo(float dist, float dir, Time time)
{
  distch = dist;
  dirch  = dir;
  chtime = time;
  seen_moving = TRUE;
}

/********************************************************************************/

void Object::update()
{
  seen = seen_moving = FALSE;
}

/********************************************************************************/

void Object::reset ()
{
  chtime = pdtime = patime = 0;
  seen = FALSE;
  seen_time = -3;
}  

/********************************************************************************/

void Object::sanitize_times()
{
  Mem->sanitize_time(seen_time);
  Mem->sanitize_time(pdtime);
  Mem->sanitize_time(patime);
  Mem->sanitize_time(chtime);
}  

/********************************************************************************/

Bool Object::in_view_range(Time time, float angle_buffer, float distance_buffer)
{
  if ( !pos_valid() || !Mem->MyConf() ) return FALSE;
  if ( distance_buffer != 0 ) my_error("Shouldn't have valid distance_buffer at this point");

  Vector tmp_pos = get_rel_pos();
  /* angle_buffer is a distance I should move forward before drawing the angle */
  tmp_pos -= Vector(angle_buffer,0);  

  if ( fabs(tmp_pos.dir()) <= Mem->MyViewAngle(time) ) return TRUE;

  /* if ( fabs(get_ang()) <= Mem->MyViewAngle() - angle_buffer ) return TRUE; */

  return FALSE; 
}
#endif // if 0
