/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Apr 08 1998 Oliver Obst
   class to store information about a single object
   $Id: objinfo.h,v 1.4 1999/05/05 14:23:52 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _OBJINFO_H
#define _OBJINFO_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream.h>

#include "maths.h"
#include "timer.h"
#include "constants.h"

/**
 * A test is a test is a test.
   */
   
class ObjInfo
{
public:
        ObjInfo(Objtype name=unknown, short id=id_unknown,
                double dist=bearing_unknown, double dir=bearing_unknown,
                double ddist=bearing_unknown, double ddir=bearing_unknown,
                double bodydir=bearing_unknown, double facedir=bearing_unknown,
                double x=bearing_unknown, double y=bearing_unknown);
        virtual ~ObjInfo();

        void    set_name(const Objtype name);
        void    set_id(const short id);
        void    set_bodydir(const double dir);
        void    set_facedir(const double dir);
        void    set_dist(const Pvector &d);
        void    set_dist(double dist, double dir, bool deg=true);
        void    set_dist(const double dist);
        void    set_dir(double dir, bool deg=true);
        void    set_ddist(const Pvector &dd);
        void    set_ddist(double ddist, double ddir, bool deg=true);
        void    set_ddist(const double ddist);
        void    set_ddir(double ddir, bool deg=true);
        void    set_xy(const Cvector &xy);
        void    set_xy(const double x, const double y);
        void    set_unknown();
        void    calc_abs(double xpos, double ypos, double beta);

        Objtype name() const;
        short   id() const;
        double  dist() const;
        double  dir(bool deg=true);
        double  ddist() const;
        double  ddir(bool deg=true);
        double  bodydir();
        double  facedir();
        double  x();
        double  y();

        ObjInfo& operator =(const ObjInfo &oi);

        friend int operator ==(ObjInfo &oi0, ObjInfo &oi1);
        friend ostream& operator <<(ostream &os, ObjInfo &oi);

private:
        Objtype myname;
        short   myid;
        Pvector mydist;
        Pvector myddist;
        double  mybodydir;
        double  myfacedir;
        Cvector myxy;
};

#if 0
class Object 
{
public:
        Object(Objtype name=unknown);

        Pvector polar() const;
        double  dist() const;
        double  dir(bool deg=true) const;

        Pvector change() const;
        double  ddist() const;
        double  ddir(bool deg=true) const;
        
        Cvector rel() const;
        Cvector abs() const;
        
        inline double abs_x() const { return abs().x(); }
        inline double abs_y() const { return abs().y(); }
        inline double abs_valid() const { return (gpos_conf >= min_conf
                                                  ? gpos_conf : 0); }

        // change the position
        // no effect until update() is called
        virtual void set_polar(const Pvector &p, Timer time);
        virtual void set_dist(double dist, Timer time);
        virtual void set_dir(double dir, Timer time);
        virtual void set_change(const Pvector &p, Timer time);

        virtual void update();
        virtual void reset();
        virtual void sanitize_times();
        
protected:
        virtual bool in_view_range(Timer time, Pvector buffer);
        inline Timer ptime() { return (dist_t < dir_t ? dist_t : dir_t); }
        
        Objtype myname;

        double max_conf;
        double min_conf;
        
        bool seen;
        bool seen_moving;
        Timer seen_time;

        Pvector mydist;
        Timer dist_t;
        Timer dir_t;

        Pvector myddist;
        Timer ddist_t;

        Cvector myrpos;
        Timer rpos_t;
        
        Cvector mygpos;
        Timer gpos_t;
        double gpos_conf;
};
#endif // if 0
#endif /* _OBJINFO_H */

