/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   read soccer server parameter
   Copyright (C) Apr 06 1999 Oliver Obst
   $Id: parameter.C,v 1.1 1999/05/05 15:02:30 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#if HAVE_STRINGS_H
# include <strings.h>
#endif

#if STDC_HEADERS
# include <string.h>
#endif

#include <stdio.h>

#include "debug.h"
#include "parameter.h"

// --- public constructor -----------------------------------------------------
Parameter::Parameter()
{
        // no option flags for these
	SP_pitch_length    		=  105.0;
	SP_pitch_width 	 		=   68.0;
	SP_pitch_margin	 		=    5.0;
	SP_penalty_area_length 		=   16.5;
	SP_penalty_area_width		=   40.32;
	SP_goal_area_length		=    5.5;
	SP_goal_area_width		=   18.32;
	SP_penalty_spot_dist		=   11.0;
	SP_corner_arc_r		        =    1.0;
	SP_free_kick_buffer		=    9.15;
	SP_after_goal_wait  		=   50;
	SP_feel_distance 		=    3.0;
	SP_num_lines     		=    4;
	SP_num_markers   		=   55;
	SP_unum_far_length		=   20.0;
	SP_unum_too_far_length		=   40.0;
	SP_team_far_length		=   40.0;
	SP_team_too_far_length		=   60.0;
	SP_version	 		=    4.06;
	SP_team_size     		=   11;
	SP_half	    		        =    1;        
        sprintf(SP_host, "localhost");

        // server.conf starts here        
	SP_goal_width    		=   14.02;       
	SP_player_size   		=    0.3;
	SP_player_decay  		=    0.4;
	SP_player_rand 		        =    0.1;
	SP_player_weight		=   60.0;
	SP_player_speed_max		=    1.0;
	SP_stamina_max                  = 3500.0;
	SP_stamina_inc_max		=   20.0;
	SP_recover_dec_thr		=    0.3;
	SP_recover_dec   		=    0.002;
	SP_recover_min   		=    0.5;
	SP_effort_dec_thr		=    0.3;
	SP_effort_dec    		=    0.005;	
	SP_effort_inc_thr		=    0.6;	
	SP_effort_inc    		=    0.01;
	SP_effort_min    		=    0.6;
	SP_hear_max                     =    2;
	SP_hear_inc                     =    1;
	SP_hear_decay    		=    2;
	SP_inertia_moment		=    5.0;
        SP_sense_body_step              =  100;
	SP_catchable_area_l  		=    2.0;	
	SP_catchable_area_w  		=    1.0;	
	SP_catch_probability		=    1.0;
	SP_catch_ban_cycle		=    5;        
	SP_ball_size     		=    0.085;		
	SP_ball_decay    		=    0.94;
	SP_ball_rand     		=    0.05;		
	SP_ball_weight   		=    0.2;
	SP_ball_speed_max		=    2.7;
	SP_wind_dir		        =    0.0;	
	SP_wind_force    		=    0.0;
	SP_wind_rand     		=    0.0;	
	SP_wind_none     		=  false;
	SP_wind_random   		=  false;
	SP_kickable_margin		=    0.7;	
        SP_control_radius               =    1.0;       // ???
        SP_ckick_margin                 =    1.0;
	SP_dash_power_rate		=    0.006;        
	SP_kick_power_rate		=    0.016;
	SP_visible_angle 		=   90.0;
        SP_visible_distance             =    3.0;
	SP_audio_cut_dist		=   50.0;	
	SP_quantize_step    		=    0.1;
	SP_quantize_step_l    		=    0.01;
	SP_maxpower     		=  100;
	SP_minpower     		= -100;
	SP_maxmoment    		=  180;
	SP_minmoment    		= -180;
        SP_maxneckang                   =   90;
        SP_minneckang                   =  -90;
	SP_port	    		        = 6000;
	SP_coach_port    		= 6001;
        SP_olcoach_port                 = 6002;
        SP_coach_cnt_max                =  128;
        SP_coach_msg_size               =  128;
        SP_send_vi_step                 =  100;
	SP_simulator_step		=  100;
	SP_send_step     		=  150;
	SP_recv_step     		=   10;
	SP_half_time     		=  300;
	SP_say_msg_size  		=  512;
	SP_use_offside   		= true;
	SP_offside_active_area_size     = 9.15;
	SP_forbid_kick_off_offside      = true;
	SP_verbose	 		= false;
        SP_record_version               = 2;
        SP_record_log                   = false;
        SP_send_log                     = true;
        SP_log_file[0]                  = 0;
        SP_record[0]                    = 0;
        SP_replay[0]                    = 0;
        SP_coach                        = false;
        SP_coach_w_referee              = false;
        SP_old_coach_hear               = false;

	SP_kickable_area = SP_kickable_margin + SP_ball_size + SP_player_size;

        // variable types and names
        option_t opt[] =
{{ "pitch_length",              (void *) &SP_pitch_length,      V_DOUBLE },
 { "pitch_width",               (void *) &SP_pitch_width,       V_DOUBLE },
 { "pitch_margin",              (void *) &SP_pitch_margin,      V_DOUBLE },
 { "penalty_area_length",       (void *) &SP_penalty_area_length, V_DOUBLE },
 { "penalty_area_width",        (void *) &SP_penalty_area_width, V_DOUBLE },
 { "goal_area_length",          (void *) &SP_goal_area_length,  V_DOUBLE },
 { "goal_area_width",           (void *) &SP_goal_area_width,   V_DOUBLE },
 { "penalty_spot_dist",         (void *) &SP_penalty_spot_dist, V_DOUBLE },
 { "corner_arc_r",              (void *) &SP_corner_arc_r,      V_DOUBLE },
 { "free_kick_buffer",          (void *) &SP_free_kick_buffer,  V_DOUBLE },
 { "after_goal_wait",           (void *) &SP_after_goal_wait,   V_INT },
 { "feel_distance",             (void *) &SP_feel_distance,     V_DOUBLE },
 { "num_lines",                 (void *) &SP_num_lines,         V_INT },
 { "num_markers",               (void *) &SP_num_markers,       V_INT },
 { "unum_far_length",           (void *) &SP_unum_far_length,   V_DOUBLE },
 { "unum_too_far_length",       (void *) &SP_unum_too_far_length, V_DOUBLE },
 { "team_far_length",           (void *) &SP_team_far_length,   V_DOUBLE },
 { "team_too_far_length",       (void *) &SP_team_too_far_length, V_DOUBLE },
 { "version",                   (void *) &SP_version,           V_DOUBLE },
 { "team_size",                 (void *) &SP_team_size,         V_INT },
 { "half",                      (void *) &SP_half,              V_INT },
 { "host",                      (void *) &SP_host,              V_STRING },  
 { "goal_width",                (void *) &SP_goal_width,        V_DOUBLE },
 { "player_size",               (void *) &SP_player_size,       V_DOUBLE },
 { "player_decay",              (void *) &SP_player_decay,      V_DOUBLE },
 { "player_rand",               (void *) &SP_player_rand,       V_DOUBLE },
 { "player_weight",             (void *) &SP_player_weight,     V_DOUBLE },
 { "player_speed_max",          (void *) &SP_player_speed_max,  V_DOUBLE },
 { "stamina_max",               (void *) &SP_stamina_max,       V_DOUBLE },
 { "stamina_inc_max",           (void *) &SP_stamina_inc_max,   V_DOUBLE },
 { "recover_dec_thr",           (void *) &SP_recover_dec_thr,   V_DOUBLE },
 { "recover_dec",               (void *) &SP_recover_dec,       V_DOUBLE },
 { "recover_min",               (void *) &SP_recover_min,       V_DOUBLE },
 { "effort_dec_thr",            (void *) &SP_effort_dec_thr,    V_DOUBLE },
 { "effort_dec",                (void *) &SP_effort_dec,        V_DOUBLE },
 { "effort_inc_thr",            (void *) &SP_effort_inc_thr,    V_DOUBLE },
 { "effort_inc",                (void *) &SP_effort_inc,        V_DOUBLE },
 { "effort_min",                (void *) &SP_effort_min,        V_DOUBLE },
 { "hear_max",                  (void *) &SP_hear_max,          V_INT },
 { "hear_inc",                  (void *) &SP_hear_inc,          V_INT },
 { "hear_decay",                (void *) &SP_hear_decay,        V_INT },
 { "inertia_moment",            (void *) &SP_inertia_moment,    V_DOUBLE },
 { "sense_body_step",           (void *) &SP_sense_body_step,   V_INT },
 
 { "catchable_area_l",          (void *) &SP_catchable_area_l,  V_DOUBLE },
 { "catchable_area_w",          (void *) &SP_catchable_area_w,  V_DOUBLE },
 { "catch_probability",         (void *) &SP_catch_probability, V_DOUBLE },
 { "catch_ban_cycle",           (void *) &SP_catch_ban_cycle,   V_DOUBLE },
 
 { "ball_size",                 (void *) &SP_ball_size,         V_DOUBLE },
 { "ball_decay",                (void *) &SP_ball_decay,        V_DOUBLE },
 { "ball_rand",                 (void *) &SP_ball_rand,         V_DOUBLE },
 { "ball_weight",               (void *) &SP_ball_weight,       V_DOUBLE },
 { "ball_speed_max",            (void *) &SP_ball_speed_max,    V_DOUBLE },
 
 { "wind_force",                (void *) &SP_wind_force,        V_DOUBLE },
 { "wind_dir",                  (void *) &SP_wind_dir,          V_DOUBLE },
 { "wind_rand",                 (void *) &SP_wind_rand,         V_DOUBLE },
 { "wind_none",                 (void *) &SP_wind_none,         V_BOOL },
 { "wind_random",               (void *) &SP_wind_random,       V_BOOL },
 { "kickable_margin",           (void *) &SP_kickable_margin,   V_DOUBLE },
 { "control_radius",            (void *) &SP_control_radius,    V_DOUBLE },
 { "ckick_margin",              (void *) &SP_ckick_margin,      V_DOUBLE },
 { "dash_power_rate",           (void *) &SP_dash_power_rate,   V_DOUBLE },
 { "kick_power_rate",           (void *) &SP_kick_power_rate,   V_DOUBLE },
 { "visible_angle",             (void *) &SP_visible_angle,     V_DOUBLE },
 { "visible_distance",          (void *) &SP_visible_distance,  V_DOUBLE },
 { "audio_cut_dist",            (void *) &SP_audio_cut_dist,    V_DOUBLE },
 { "quantize_step",             (void *) &SP_quantize_step,     V_DOUBLE },
 { "quantize_step_l",           (void *) &SP_quantize_step_l,   V_DOUBLE },
 { "maxpower",                  (void *) &SP_maxpower,          V_DOUBLE },
 { "minpower",                  (void *) &SP_minpower,          V_DOUBLE },
 { "maxmoment",                 (void *) &SP_maxmoment,         V_DOUBLE },
 { "minmoment",                 (void *) &SP_minmoment,         V_DOUBLE },
 { "maxneckang",                (void *) &SP_maxneckang,        V_DOUBLE },
 { "minneckang",                (void *) &SP_minneckang,        V_DOUBLE },
 { "port",                      (void *) &SP_port,              V_INT },
 { "coach_port",                (void *) &SP_coach_port,        V_INT },
 { "olcoach_port",              (void *) &SP_olcoach_port,      V_INT },
 { "coach_cnt_max",             (void *) &SP_coach_cnt_max,     V_INT },
 { "coach_msg_size",            (void *) &SP_coach_msg_size,    V_INT },
 { "send_vi_step",              (void *) &SP_send_vi_step,      V_INT },
 { "simulator_step",            (void *) &SP_simulator_step,    V_INT },
 { "send_step",                 (void *) &SP_send_step,         V_INT },
 { "recv_step",                 (void *) &SP_recv_step,         V_INT },
 { "half_time",                 (void *) &SP_half_time,         V_INT },
 { "say_msg_size",              (void *) &SP_say_msg_size,      V_INT },
 { "use_offside",               (void *) &SP_use_offside,       V_ONOFF },
 { "offside_active_area_size",  (void *) &SP_offside_active_area_size, V_DOUBLE },
 { "forbid_kick_off_offside",   (void *) &SP_forbid_kick_off_offside, V_ONOFF },
 { "verbose",                   (void *) &SP_verbose,           V_ONOFF },
 { "record_version",            (void *) &SP_record_version,    V_INT },
 { "record_log",                (void *) &SP_record_log,        V_BOOL },
 { "send_log",                  (void *) &SP_send_log,          V_BOOL },
 { "log_file",                  (void *) &SP_log_file,          V_STRING },
 { "record",                    (void *) &SP_record,            V_STRING },
 { "replay",                    (void *) &SP_replay,            V_STRING },
 
 { "coach",                     (void *) &SP_coach,             V_BOOL },
 { "coach_w_referee",           (void *) &SP_coach_w_referee,   V_BOOL },
 { "old_coach_hear",            (void *) &SP_old_coach_hear,    V_BOOL },
 // server.conf ends here

 { "kickable_area",             (void *) &SP_kickable_area,     V_DOUBLE },
 { "\000", 0, V_NONE }
        };

        int i = 0;
        while (opt[i].vptr != 0 && i < optsize) {
                strcpy(options[i].name, opt[i].name);
                options[i].vptr = opt[i].vptr;
                options[i].vsize = opt[i].vsize;
                i++;
        }


        RASSERT1(opt[i].vptr == 0, ROBOBUG_ERROR, PARAMETER_CLASS,
                 "Option array too small (%u)", optsize);

        i = (i < optsize - 1) ? i : optsize - 1;
        options[i].vptr = 0;
}

// --- public -----------------------------------------------------------------
void Parameter::read_options(const char *filename, int verbose)
{
        if (filename == 0) return;

        FILE *file;

        file = fopen(filename, "r");

        if (! ferror(file)) {
                if (verbose > 0) {
                        fprintf(stderr, "can't open (%s)\n",
                                filename);
                } else {
                        ROBOBUG1(ROBOBUG_WARN, PARAMETER_CLASS,
                                 "can't open (%s)", filename);
                }
                
                return;
        }

        char linebuf[256];
        char *t;
        int  i = 0;
        
        while (fgets(linebuf, 256, file) != 0) {
                i++;
                if (verbose == 1 && i % 10 == 0)
                        fprintf(stderr, ".");
                
                if (verbose > 2) {
                        fprintf(stderr,linebuf);
                        fflush(stderr);
                }

                if (linebuf[0] == '#'
                    || linebuf[0] == '\n'
                    || linebuf[0] == '\0')
                        continue;

                t = linebuf;
                // replace the ':' by space
                while (*t != '\0') {
                        if (*t == ':') *t = ' ';
                        t++;
                }
                int n, p ;
                char com[256];
                char onoff[16] ;

		n = sscanf(linebuf,"%s", com) ;

		if (n < 1) {
                        ROBOBUG1(ROBOBUG_WARN, PARAMETER_CLASS,
                                 "Read illegal line: %s", linebuf);
                        continue;
                }

                for (p = 0 ; options[p].vptr != 0; p++) {
                        if (strcmp(com, options[p].name))
                                continue;

                        /* match */
                        switch(options[p].vsize) {
                        case V_INT:
                                n = sscanf(linebuf, "%s %d",
                                           com, (int *) options[p].vptr);
                                break;
                        case V_STRING:
                                n = sscanf(linebuf, "%s %s",
                                           com, (char *) options[p].vptr);
                                break;
                        case V_DOUBLE:
                                n = sscanf(linebuf, "%s %lf", com,
                                           (double *) options[p].vptr);
                                break;
                        case V_BOOL:
                                n = 2;
                                *((bool *) options[p].vptr) = true;
                                break;
                        case V_ONOFF:
                                n = sscanf(linebuf, "%s %s", com, onoff);
                                *((bool *) options[p].vptr) =
                                        (!strcmp(onoff, "on")) ? true : false;
                                break;
                        case V_NONE:
                                n = 0;
                        }

                        if (n < 2) {
                                if (verbose > 0) {
                                        fprintf(stderr, "Illegal line (%s)\n",
                                                com);
                                        fflush(stderr);
                                } else {
                                        ROBOBUG1(ROBOBUG_WARN, PARAMETER_CLASS,
                                                 "Read illegal line (%s)",
                                                 com);
                                }
                                break;
                        }
                }
                
                if (options[p].vptr == 0) {
                        if (verbose > 0) {
                                fprintf(stderr, "Unknown option (%s)\n", com);
                                fflush(stderr);
                        } else {
                                ROBOBUG1(ROBOBUG_WARN, PARAMETER_CLASS,
                                         "Unknown option (%s)", com);
                        }
                }
        }
}
