/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   write error messages
   Copyright (C) Apr 16 1999 Oliver Obst
   $Id: robobug.C,v 1.1 1999/05/05 15:02:35 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdarg.h>
#include <stdio.h>

#include "robobug.h"

FILE *file0 = 0;
FILE *file1 = 0;
FILE *file2 = 0;
FILE *file3 = 0;

struct bugarea 
{
        unsigned short a;
        bugarea *next;
};

bugarea *area0 = 0;


bool in_area(unsigned short area) 
{
        bool b = (area0 == 0);
        bugarea *ap = area0;
        while (ap != 0) {
                if (ap->a >= area) {
                        ap = 0;
                        b = (ap->a == area);
                } else ap = ap->next;
        }

        return b;
}

void delete_areas(bugarea *ap0)
{
        bugarea *ap1;
        
        while (ap0 != 0) {
                ap1 = ap0->next;
                delete ap0;
                ap0 = ap1;
        }
}

void delete_bugareas()
{
        delete_areas(area0);
}

void robobug_area(unsigned short a1)
{
        bugarea *ap0 = area0;

        if (ap0 != 0 && ap0->a > a1) area0 = 0;

        if (area0 == 0) {
                area0 = new(bugarea);
                area0->a = a1;
                area0->next = ap0;
        } else {
                bugarea *ap1 = ap0->next;                
                while (ap1->a < a1 && ap1->next != 0) {
                        ap0 = ap1;
                        ap1 = ap1->next;
                }
                
                if (ap1->next == 0) {
                        ap1->next = new(bugarea);
                        ap1->next->a = a1;
                        ap1->next->next = 0;
                } else {
                        if (ap1->a > a1) {
                                ap1 = new(bugarea);
                                ap1->a = a1;
                                ap1->next = ap0->next;
                                ap0->next = ap1;
                        }
                }
        }
}

int bugclose(FILE *f) 
{
        if (f != 0 && f != stdout && f != stderr)
                return fclose(f);
        else return 0;
}                

void robobug(DebugLevel level, unsigned short area, const char *fmt, ...) 
{
        va_list ap;
        
        va_start(ap, fmt);
        
        switch (level) {
        case ROBOBUG_INFO:
                if (file0 != 0 && in_area(area)) {
                        fprintf(file0, "Info (robobug area: %u): ", area);
                        fprintf(file0, fmt, ap);
                        fprintf(file0, "\n");
                        fflush(file0);
                }
                break;
        case ROBOBUG_WARN:
                if (file1 != 0 && in_area(area)) {
                        fprintf(file1, "Warning (robobug area: %u): ", area);
                        fprintf(file1, fmt, ap);
                        fprintf(file1, "\n");
                        fflush(file1);
                }
                break;
        case ROBOBUG_ERROR:
                if (file2 != 0 && in_area(area)) {
                        fprintf(file2, "Error (robobug area: %u): ", area);
                        fprintf(file2, fmt, ap);
                        fprintf(file2, "\n");
                        fflush(file2);
                }
                break;
        case ROBOBUG_FATAL:
        default:
                if (file3 != 0 && in_area(area)) {
                        fprintf(file3,
                                "Fatal Error (robobug area: %u): ", area);
                        fprintf(file3, fmt, ap);
                        fprintf(file3, "\n");
                        fflush(file3);
                }
        }

        va_end(ap);
        
        if (level >= ROBOBUG_FATAL) {
                bugclose(file0);
                bugclose(file1);
                bugclose(file2);
                bugclose(file3);
                exit(-1);
        }        
}


void bugfile(FILE *f, unsigned char levels) 
{
        if (levels & ROBOBUG_INFO == ROBOBUG_INFO) {
                bugclose(file0);
                file0 = f;
        }
        if (levels & ROBOBUG_WARN == ROBOBUG_WARN) {
                bugclose(file1);
                file1 = f;
        }
        if (levels & ROBOBUG_ERROR == ROBOBUG_ERROR) {
                bugclose(file3);
                file2 = f;
        }
        if (levels & ROBOBUG_FATAL == ROBOBUG_FATAL) {
                bugclose(file3);
                file3 = f;
        }
}

void bugfile(const char *filename, unsigned char levels)
{
        FILE *f = 0;
        
        if (levels & ROBOBUG_INFO == ROBOBUG_INFO) {
                bugclose(file0);
                if (f == 0) f = fopen(filename, "a");
                file0 = f;
        }
        if (levels & ROBOBUG_WARN == ROBOBUG_WARN) {
                bugclose(file1);
                if (f == 0) f = fopen(filename, "a");
                file1 = f;
        }
        if (levels & ROBOBUG_ERROR == ROBOBUG_ERROR) {
                bugclose(file3);
                if (f == 0) f = fopen(filename, "a");
                file2 = f;
        }
        if (levels & ROBOBUG_FATAL == ROBOBUG_FATAL) {
                bugclose(file3);
                if (f == 0) f = fopen(filename, "a");
                file3 = f;
        }
}
