/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Mar 03 1998 Oliver Obst
   main file
   $Id: robolog.C,v 1.10 1999/05/05 14:46:18 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "soccerconst.h"
#include "constants.h"
#include "sclient.h"
#include "control.h"
#include "simple.h"
#include "objinfo.h"
#include "objarray.h"
#include "msginfo.h"
#include "msgarray.h"
#include "knowledge.h"
#include "listproc.h"
#include "ring.h"
#include "self.h"
#include "maths.h"
#include "robolog.h"
#include "debug.h"

Sclient *sclient;
long int myclock;
int steps;
int timeout;

#ifndef SIGIO
# define SIGIO SIGPOLL
#endif

extern "C" RETSIGTYPE sigalrm_handler(int)
{
        ROBOBUG(ROBOBUG_INFO, ROBOLOG_INIT, "sigalrm start");
        myclock++;
        sclient->action_update(steps);
        if (++timeout >= 50) {
                fprintf(stderr,
                        "Cast a cold eye on Life, on Death. "
                        "Horsemen pass by!\n");
                RSC_stop();
                exit(-1);
        }
        ROBOBUG(ROBOBUG_INFO, ROBOLOG_INIT, "sigalrm end");
        return 0;
}

extern "C" RETSIGTYPE sigio_handler(int)
{
        ROBOBUG(ROBOBUG_INFO, ROBOLOG_INIT, "sigio start");        
        timeout = 0;
        sclient->view_update();
        ROBOBUG(ROBOBUG_INFO, ROBOLOG_INIT, "sigio end");
        return 0;
}


extern "C" void clock_start(int irqcycle, int irqsteps)
{
        steps = irqsteps;
        myclock = 0;
#if 0
        bugfile(stderr, ROBOBUG_INFO | ROBOBUG_WARN | ROBOBUG_ERROR | ROBOBUG_FATAL);
        robobug_area(ROBOLOG_INIT);
#endif

#if 0      // def HAVE_SIGSET        
        sigset(SIGALRM, ((void (*) (int)) irq));
#else
        sigset_t mask;
        struct sigaction vec;

        sigfillset(&mask);             // block all signals during timer event
        sigdelset(&mask, SIGINT);      // except INT
        sigdelset(&mask, SIGTERM);     // and TERM

        vec.sa_handler = (RETSIGTYPE (*) (int)) sigalrm_handler;
        vec.sa_mask = mask;
        vec.sa_flags = 0;
        if (sigaction(SIGALRM, &vec, 0) == -1) {
                ROBOBUG(ROBOBUG_FATAL,ROBOLOG_INIT,
                        "sigaction error (SIGALRM)");
        }

        vec.sa_handler = (RETSIGTYPE (*) (int)) sigio_handler;
        if (sigaction(SIGIO, &vec, 0) == -1) {
                ROBOBUG(ROBOBUG_FATAL,ROBOLOG_INIT,
                        "sigaction error (SIGIO)");
        }

        // unblock
        sigemptyset(&mask);
        sigaddset(&mask, SIGALRM);
        sigaddset(&mask, SIGIO);
        sigprocmask(SIG_UNBLOCK, &mask, 0);
#endif
        
        itimerval zeit;
        zeit.it_value.tv_sec = 0;
        zeit.it_value.tv_usec = irqcycle * 1000;
        zeit.it_interval.tv_sec = 0;
        zeit.it_interval.tv_usec = irqcycle * 1000;
        setitimer(ITIMER_REAL, &zeit, 0);

        ROBOBUG(ROBOBUG_INFO, ROBOLOG_INIT, "Timer setup done");
}

extern "C" void clock_stop()
{
        itimerval zeit;
        zeit.it_value.tv_sec = 0;
        zeit.it_value.tv_usec = 0;
        zeit.it_interval.tv_sec = 0;
        zeit.it_interval.tv_usec = 0;
        setitimer(ITIMER_REAL, &zeit, NULL);
        
#if 0   // def HAVE_SIGSET        
        sigset(SIGALRM, SIG_DFL);
#else
        sigset_t mask;
        struct sigaction vec;
        sigemptyset(&mask);
        vec.sa_handler = SIG_DFL;
        vec.sa_mask = mask;
        vec.sa_flags = 0;
        sigaction(SIGALRM, &vec, 0);
        sigaction(SIGIO, &vec, 0);
#endif
}


// ----------------------------------------------------------------------------
// atan2 
extern "C" double RSC_atan2(double y, double x)
{
        return atan2(y,x);
}

        
// ----------------------------------------------------------------------------
// deg2rad
extern "C" double RSC_deg2rad(double deg)
{
        return deg2rad(deg);
}
        

// ----------------------------------------------------------------------------
// rad2deg
extern "C" double RSC_rad2deg(double rad)
{
        return rad2deg(rad);
}        

// ----------------------------------------------------------------------------
// server registration
extern "C" void RSC_init_player(const char *teamname, const char *servername,
                                int portnr, double version)
{
        sclient = new Sclient(servername, portnr, version);
        sclient->simple->normal_init(teamname);
        clock_start(50,2);
        sclient->refresh();
        printf("%s, Version %s.\n%s\n",PROG_NAME,VERSION,COPYRIGHT);
}

// ----------------------------------------------------------------------------
// server registration for goalkeeper
extern "C" void RSC_init_goalie(const char *teamname, const char *servername,
                                int portnr, double version)
{
        sclient = new Sclient(servername, portnr, version);
        sclient->simple->goalie_init(teamname);
        clock_start(50,2);
        sclient->refresh();
        printf("%s, Version %s.\n%s\n",PROG_NAME,VERSION,COPYRIGHT);
}

// ----------------------------------------------------------------------------
// stop interrupts and delete client
extern "C" void RSC_stop()
{
        clock_stop();
        delete sclient;
}

// ----------------------------------------------------------------------------
extern "C" int RSC_open_log(const char *filename)
{        
        return (sclient->simple->start_logfile(filename));
}

// ----------------------------------------------------------------------------
extern "C" void RSC_close_log()
{
        sclient->simple->stop_logfile();
}

// ----------------------------------------------------------------------------
extern "C" int RSC_open_audiolog(const char *filename)
{        
        return (sclient->simple->start_audiologfile(filename));
}

// ----------------------------------------------------------------------------
extern "C" void RSC_close_audiolog()
{
        sclient->simple->stop_audiologfile();
}

// ----------------------------------------------------------------------------
extern "C" int RSC_start_viewserver(int base)
{
        int i = RSC_uniformnumber();

        if (i > 0 && i < 12) {
                sclient->simple->start_server(i,base);
                i = 0;
        }
        else i = -1;

        return i;
}

// ----------------------------------------------------------------------------
extern "C" void RSC_stop_viewserver()
{
        sclient->simple->stop_server();
}

// ----------------------------------------------------------------------------
// refresh workspace
extern "C" void RSC_refresh()
{
        sclient->refresh();
}

// ----------------------------------------------------------------------------
// read working space timer
extern "C" long int RSC_simstep()
{
        return sclient->get_time();
}

// ----------------------------------------------------------------------------
// read working space timer
extern "C" long int RSC_roboclock()
{
        return myclock;
}

// ----------------------------------------------------------------------------
// wait for new data and refresh workspace
extern "C" void RSC_await_refresh()
{
        sclient->set_updated(false);
        while (! sclient->get_updated());
        sclient->refresh();
}

// ----------------------------------------------------------------------------
// wait until outq is empty
extern "C" void RSC_wait_for_robo(int outq_size)
{
        sclient->simple->wait4outq(outq_size);
}

// ----------------------------------------------------------------------------
// clear the outq
extern "C" void RSC_dequeue()
{
        sclient->simple->dequeue();
}

// ----------------------------------------------------------------------------
// return number of commands in the outq
extern "C" int RSC_outqlen()
{
        return sclient->simple->outqlen();
}

// ----------------------------------------------------------------------------
// return team name for own/other team (0/1)
extern "C" char* RSC_teamname(int team)
{
        return sclient->knowledge(-1).get_teamname(team);
}

// ----------------------------------------------------------------------------
// return uniform number
extern "C" int RSC_uniformnumber() 
{
        return sclient->knowledge(-1).get_number();
}

// ----------------------------------------------------------------------------
// return true if player is the goalie
extern "C" int RSC_is_goalie()
{
        return sclient->knowledge(-1).is_goalie();
}

// ----------------------------------------------------------------------------
// return uniform number of the goalie for own/other team (0/1)
extern "C" int RSC_goalie(int team)
{
        return sclient->knowledge(-1).get_goalie(team);
}

// ----------------------------------------------------------------------------
// set uniform number of the goalie for own/other team (0/1)
extern "C" void RSC_set_goalie(int nr, int team)
{
        sclient->knowledge(-1).set_goalie(nr, team);
}

// ----------------------------------------------------------------------------
// return side
extern "C" int RSC_myside()
{
        return sclient->knowledge(-1).get_side();
}

// ----------------------------------------------------------------------------
// set/get view width
extern "C" void RSC_view_1(int *width, int set)
{
        Angle angle;

        if (set) {
                switch (*width) {
                case v_narrow :
                        angle = narrow;
                        break;
                case v_wide :
                        angle = wide;
                        break;
                default :
                        angle = normal;
                }
                
                sclient->simple->
                        changeview(angle,
                                   sclient->knowledge(-1).get_view_quality());
                sclient->knowledge(-1).set_view_width(angle);
        } else {
                switch (sclient->knowledge(-1).get_view_width()) {
                case narrow :
                        *width = v_narrow;
                        break;
                case wide :
                        *width = v_wide;
                        break;
                default :
                        *width = v_normal;
                }
        }
}

// ----------------------------------------------------------------------------
// set/get view width and quality
extern "C" void RSC_view_2(int *width, int *i_quality,
                         int set_width, int set_quality)
{
        Angle angle;
        Quality quality;

        if (set_width) {
                switch (*width) {
                case v_narrow :
                        angle = narrow;
                        break;
                case v_wide :
                        angle = wide;
                        break;
                default :
                        angle = normal;
                }
                
                sclient->knowledge(-1).set_view_width(angle);
        } else {
                switch (sclient->knowledge(-1).get_view_width()) {
                case narrow :
                        *width = v_narrow;
                        break;
                case wide :
                        *width = v_wide;
                        break;
                default :
                        *width = v_normal;
                }
        }

        if (set_quality) {
                if (*i_quality == v_low)
                        quality = low;
                else quality = high;
                
                sclient->knowledge(-1).set_view_quality(quality);
        } else {
                if (sclient->knowledge(-1).get_view_quality() == low)
                        *i_quality = v_low;
                else *i_quality = v_high;
        }

        
        if (set_width || set_quality)
                sclient->simple->
                        changeview(sclient->knowledge(-1).get_view_width(),
                                   sclient->knowledge(-1).get_view_quality());
}

// ----------------------------------------------------------------------------
// return referee judgement
extern "C" int RSC_referee()
{
        return sclient->knowledge(-1).get_status();
}

// ----------------------------------------------------------------------------
// return players stamina
extern "C" double RSC_stamina()
{
        return sclient->knowledge(-1).get_stamina();
}

// ----------------------------------------------------------------------------
// return players effort
extern "C" double RSC_effort()
{
        return sclient->knowledge(-1).get_effort();
}

// ----------------------------------------------------------------------------
// return players effort
extern "C" double RSC_recovery()
{
        return sclient->knowledge(-1).get_recovery();
}

// ----------------------------------------------------------------------------
// return players neck angle
extern "C" double RSC_neck(int time)
{
        return (sclient->knowledge(time).get_version() < 5.0 ?
                0 : sclient->knowledge(time).get_neck());
}

// ----------------------------------------------------------------------------
// return players body angle
extern "C" double RSC_body_angle(int time)
{
        return 0; // (sclient->knowledge(time).get_body());
}

// ----------------------------------------------------------------------------
// return players speed
extern "C" double RSC_speed()
{
        return sclient->knowledge(-1).get_speed();
}

// ----------------------------------------------------------------------------
// return number of kick commands send to the server
extern "C" int RSC_kicks()
{
        return sclient->knowledge(-1).get_kicks();
}

// ----------------------------------------------------------------------------
// return number of dash commands send to the server
extern "C" int RSC_dashs()
{
        return sclient->knowledge(-1).get_dashs();
}

// ----------------------------------------------------------------------------
// return number of turn commands send to the server
extern "C" int RSC_turns()
{
        return sclient->knowledge(-1).get_turns();
}

// ----------------------------------------------------------------------------
// return number of turn commands send to the server
extern "C" int RSC_turn_necks()
{
        return (sclient->knowledge(-1).get_version() < 5.0 ?
                0 : sclient->knowledge(-1).get_turn_necks());
}

// ----------------------------------------------------------------------------
// return number of say commands send to the server
extern "C" int RSC_says()
{
        return sclient->knowledge(-1).get_says();
}

// ----------------------------------------------------------------------------
// return number of goals for left/right side (0/1)
extern "C" int RSC_goals(int time, int side)
{
        return sclient->knowledge(time).get_goals(side);
}

// ----------------------------------------------------------------------------
// self(+Time, ?X, ?Y, ?Bodydir, ?Headdir)
// return/set position
extern "C" void RSC_self(int time, double *xpos, double *ypos,
                         double *angle1, double *angle2, int set)
{
        if (set) {
                sclient->calc_abs(*xpos, *ypos, *angle1, *angle2, time);
        } else {
                *xpos = sclient->objects(time).mypos().x();
                *ypos = sclient->objects(time).mypos().y();
                *angle1 = sclient->objects(time).mybodyangle();
                *angle2 = sclient->objects(time).myheadangle();

        }
}

// ----------------------------------------------------------------------------
// dist(-X, -Y, +Dist)
// return distance to (X,Y), fail if own position unknown
extern "C" int RSC_dist(double xpos, double ypos, double *dist)
{
        double x0  = sclient->objects().mypos().x();
        double y0  = sclient->objects().mypos().y();
        
        if (x0 < -9000 || y0 < -9000) {
                return false;
        } else {
                double x = x0 - xpos;
                double y = y0 - ypos;
                *dist = sqrt(x*x + y*y);
                return true;
        }
}

// ----------------------------------------------------------------------------
// absangle(-X,-Y,+A)
// get absolute angle
extern "C" int RSC_absangle(double xpos, double ypos, double *angle)
{
        double x0  = sclient->objects().mypos().x();
        double y0  = sclient->objects().mypos().y();
        
        if (x0 < -9000 || y0 < -9000) {
                return false;
        } else {
                double x = xpos - x0;
                double y = ypos - y0;
                *angle = rad2deg(atan2(y,x));
                return true;
        }
}

// math line stuff ------------------------------------------------------------

// ----------------------------------------------------------------------------
// true if point is on the line
extern "C" int RSC_point_on_line(double px, double py, double x0, double y0,
                                 double x1, double y1)
{
        Line l = LineFromTwoPoints(Cvector(x0,y0),Cvector(x1,y1));
        return l.PointOnLine(px,py);
}

// ----------------------------------------------------------------------------
extern "C" int RSC_point_between(double px, double py, double x0, double y0,
                                 double x1, double y1)
{
        Cvector e0(x0,y0);
        Cvector e1(x1,y1);
        
        return LineFromTwoPoints(e0,e1).InBetween(Cvector(px,py),e0,e1);
}

// ----------------------------------------------------------------------------
extern "C" double RSC_dist_point_line(double px, double py,
                                      double x0, double y0,
                                      double x1, double y1)
{
        Line l = LineFromTwoPoints(Cvector(x0,y0),Cvector(x1,y1));
        return l.dist(Cvector(px,py));
}

// ----------------------------------------------------------------------------
extern "C" double RSC_line_angle(double x0, double y0, double x1, double y1)
{
        return LineFromTwoPoints(Cvector(x0,y0),Cvector(x1,y1)).angle();
}

// math rectangle -------------------------------------------------------------

// ----------------------------------------------------------------------------
// check if point is within a rectangular area
extern "C" int RSC_point_in_box(double px, double py, double x0, double y0,
                                double x1, double y1)
{
        return Rectangle(x0,y0,x1,y1).IsWithin(Cvector(px,py));
}

// ----------------------------------------------------------------------------
// return nearest edge
extern "C" void RSC_nearest_edge(double px, double py, double x0, double y0,
                                 double x1, double y1, double *x2, double *y2)
{
        Cvector edge = Rectangle(x0,y0,x1,y1).nearestEdge(Cvector(px,py));
        *x2 = edge.x();
        *y2 = edge.y();
}

// ----------------------------------------------------------------------------
// return distance to edge
extern "C" double RSC_distance_to_edge(double px, double py, double x0,
                                       double y0, double x1, double y1)
{
        return Rectangle(x0,y0,x1,y1).DistanceToEdge(Cvector(px,py));
}

// ----------------------------------------------------------------------------
// return closest point within the rectangular area
extern "C" void RSC_closest_point_inside(double px, double py, double x0,
                                         double y0, double x1, double y1,
                                         double *x2, double *y2)
{
        Cvector inside = Rectangle(x0,y0,x1,y1).AdjustToWithin(Cvector(px,py));

        *x2 = inside.x();
        *y2 = inside.y();
}

// ----------------------------------------------------------------------------
// calculate a quality for a direction
extern "C" int RSC_dir_quality(double dir)
{
        int q;
        
        if (fabs(dir) > 9000) return -1;
        
        while (dir > 180) dir -= 360;
        while (dir <= -180) dir += 360;

        if (dir > -22.5) {
                if (dir > 22.5) {
                        if (dir > 112.5) {
                                if (dir > 157.5)
                                        q = 4;  // sueden
                                else
                                        q = 3;  // sued-ost
                        } else {
                                if (dir > 67.5)
                                        q = 2; // ost
                                else
                                        q = 1; // nord-ost
                        }
                }
                else
                        q = 0; // nord
        } else {
                if (dir > -112.5) {
                        if (dir > - 67.5)
                                q = 7; // nord-west
                        else
                                q = 6; // west
                } else {
                        if (dir > -157.5)
                                q = 5; // sued-west
                        else
                                q = 4; // sued
                }
        }
        return q;
}

// ----------------------------------------------------------------------------
// calculate pred. quality
extern "C" int RSC_pred_quality(int q0, int qmax, int circular)
{
        int q1;

        if (q0 < 0 || q0 > qmax) {
                q1 = -1;
        } else {
                if (q0 == 0) {
                        if (circular)
                                q1 = qmax;
                        else
                                q1 = 0;
                }
                else q1 = q0 - 1;
        }
        
        return q1;
}

// ----------------------------------------------------------------------------
// calculate succ. quality
extern "C" int RSC_succ_quality(int q0, int qmax, int circular)
{
        int q1;

        if (q0 < 0 || q0 > qmax) {
                q1 = -1;
        } else {
                if (q0 == qmax) {
                        if (circular)
                                q1 = 0;
                        else
                                q1 = qmax;
                }
                else q1 = q0 + 1;
        }
        
        return q1;
}

// ----------------------------------------------------------------------------
// return distance quality
extern "C" int RSC_dist_quality(double dist)
{
        int q;
        if (dist < 20) {
                if (dist < kickable_area) {
                        if (dist < 0) 
                                q = -1;
                        else
                                q = 0;
                }
                else q = 1;
        } else {
                if (dist < 60) {
                        if (dist < 40)
                                q = 2;
                        else
                                q = 3;
                }
                else q = 4;
        }

        return q;
}

// ----------------------------------------------------------------------------
// return simulation step time of current audio messages
extern "C" int RSC_msgidx()
{
        return sclient->get_msgidx();
}

// ----------------------------------------------------------------------------
// return simulation step time of next audio messages
extern "C" int RSC_nextmsgidx()
{
        return sclient->get_nextmsgidx();
}

// ----------------------------------------------------------------------------
// return number of messages of sender s at simulation step t
extern "C" int RSC_n_message(int sender, int time)
{
        int n;

        switch (sender) {
        case 0 :
                n = sclient->get_messages(self, time);
                break;
        case 1 :
                n = sclient->get_messages(referee, time);
                break;
        case 2 :
                n = sclient->get_messages(other, time);
                break;
        default:
                n = sclient->get_messages(all, time);
        }
        
        return n;
}

// ----------------------------------------------------------------------------
// return message info (txt,dir) number i of sender s at time t
extern "C" void RSC_message(int sender, int time, int i, char *txt, double *d)
{
        char msg[MaxStringSize];
        
        switch (sender) {
        case 0 :
                sclient->message(self, i, time, (char *) msg, d);
                break;
        case 1 :
                sclient->message(referee, i, time, (char *) msg, d);
                break;
        case 2 :
                sclient->message(other, i, time, (char *) msg, d);
                break;
        default:
                sclient->message(all, i, time, (char *) msg, d);
        }
        
        sclient->msg2text((char *) msg, txt);
}

// ----------------------------------------------------------------------------
// return object info for object number i at time t
extern "C" int RSC_object(int time, int i, object_data_t *od)
{
        int found = (i < sclient->objects().get_entries());
        
        if (found) {
                ObjInfo oi = sclient->objects(time)[i];
                switch (oi.name()) {
                case unknown :
                        od->name = o_unknown;
                        break;
                case ball :
                        od->name = o_ball;
                        break;
                case player :
                        od->name = o_player;
                        break;
                case partner :
                        od->name = o_partner;
                        break;
                case opponent :
                        od->name = o_opponent;
                        break;
                case goal :
                        od->name = o_goal;
                        break;
                case flag :
                        od->name = o_flag;
                        break;
                case line :
                        od->name = o_line;
                }

                od->id = oi.id();
                od->x = oi.x();
                od->y = oi.y();
                od->dir = oi.dist();
                od->dir = oi.dir();
                od->ddist = oi.ddist();
                od->ddir = oi.ddir();
                od->bodydir = oi.bodydir();
                od->facedir = oi.facedir();
        }

        return found;
}

extern "C" int RSC_n_goal(int time)
{
        return sclient->get_objects(goal, time);
}

// ----------------------------------------------------------------------------
extern "C" int RSC_goal(int time, int goalid, object_data_t *od)
{
        position gp;

        if (goalid != id_goal_left && goalid != id_goal_right)
                return 0;

        gp = pos_convert(sclient->knowledge(-1).get_side(), goals[goalid]);

        // if no view to requested goal exists, we have
        // to calculate the relpos first :-(          )
        if (sclient->get_objects(goal, time) == 0
            || (sclient->get_objects(goal, time) == 1
                && sclient->object(goal,0,time).id() != goalid)) {
                double gdir = sclient->objects(time).myheadangle() - 180;
                double gxdist = sclient->objects(time).mypos().x() - gp.x;
                double gydist = sclient->objects(time).mypos().y() - gp.y;
                
                if (sclient->objects(time).mypos().x() > -9000
                    && sclient->objects(time).mypos().y() > -9000) {
                        od->dist = sqrt(gxdist * gxdist
                                                 - gydist * gydist);
                        if (gdir < 9000) {
                                gdir += 57.29578 * atan2(gydist, gxdist);
                                gdir = gdir < -180 ? gdir + 360
                                        : gdir > 180 ? gdir - 360 : gdir;
                                od->dir = gdir;
                        } else {
                                od->dir = gdir;
                        }
                } else {
                        od->dist = bearing_unknown;
                        od->dir = bearing_unknown;
                }
                od->ddist = bearing_unknown;
                od->ddir = bearing_unknown;
        } else {
                // we can retrive the relpos from the view
                if (sclient->object(goal,0,time).id() == goalid) {
                        od->dist = sclient->object(goal,0,time).dist();
                        od->dir  = sclient->object(goal,0,time).dir();
                        od->ddist = sclient->object(goal,0,time).ddist();
                        od->ddir = sclient->object(goal,0,time).ddir();
                } else {
                        od->dist = sclient->object(goal,1,time).dist();
                        od->dir  = sclient->object(goal,1,time).dir();
                        od->ddist = sclient->object(goal,1,time).ddist();
                        od->ddir = sclient->object(goal,1,time).ddir();
                }
        }

        od->name = o_goal;
        od->id = goalid;
        od->x = gp.x;
        od->y = gp.y;
        od->bodydir = od->facedir = bearing_unknown;
        
        return -1;
}

extern "C" int RSC_n_flag(int time)
{
        return sclient->get_objects(flag, time);
}

extern "C" int RSC_flag(int time, int flagid, object_data_t *od)
{
        int i;
        int found;
        position fp;

        found = (flagid >= 0 || flagid <= 52);

        if (found) {
                i = 0;
                int j = sclient->get_objects(flag, time);

                found = 0;
                fp = pos_convert(sclient->knowledge(-1).get_side(),
                                 flags[flagid]);

                while (!found && i < j) {
                        found = (sclient->object(flag,i,time).id() == flagid);
                        if (!found) i++;
                }
        }
        
        if (found) {
                od->dist = sclient->object(flag,i,time).dist();
                od->dir = sclient->object(flag,i,time).dir();
                od->ddist = sclient->object(flag,i,time).ddist();
                od->ddir = sclient->object(flag,i,time).ddir();
        } else {
                od->dist = od->dir = bearing_unknown;
                od->ddist = od->ddir = bearing_unknown;
        }

        od->name = o_flag;
        od->id = flagid;
        od->x = fp.x;
        od->y = fp.y;
        od->bodydir = od->facedir = bearing_unknown;
        
        return found;
}

extern "C" int RSC_n_line(int time)
{
        return sclient->get_objects(line, time);
}

extern "C" int RSC_line(int time, int lineid, object_data_t *od)
{
        int i;
        int found;
        position lp;

        found = (lineid >= 0 || lineid <= 3);
        
        if (found)
        {
                i = 0;
                int j = sclient->get_objects(line, time);

                found = 0;
                lp = pos_convert(sclient->knowledge(-1).get_side(),
                                 lines[lineid]);

                while (!found && i < j) {
                        found = (sclient->object(line,i,time).id() == lineid);
                        if (!found) i++;
                }
        }
        
        if (found) {
                od->dist = sclient->object(line,i,time).dist();
                od->dir = sclient->object(line,i,time).dir();
                od->ddist = sclient->object(line,i,time).ddist();
                od->ddir = sclient->object(line,i,time).ddir();
        } else {
                od->dist = od->dir = bearing_unknown;
                od->ddist = od->ddir = bearing_unknown;
        }

        od->name = o_line;
        od->id = lineid;
        od->x = lp.x;
        od->y = lp.y;
        od->bodydir = od->facedir = bearing_unknown;        
        
        return found;
}

extern "C" int RSC_n_ball(int time)
{
        return sclient->get_objects(ball, time);
}

extern "C" void RSC_ball(int time, object_data_t *od)
{
        od->name = o_ball;
        od->id = 0;
        od->bodydir = od->facedir = bearing_unknown;
        od->dist = sclient->object(ball,0,time).dist();
        od->dir = sclient->object(ball,0,time).dir();
        od->ddist = sclient->object(ball,0,time).ddist();
        od->ddir = sclient->object(ball,0,time).ddir();
        od->x = sclient->object(ball,0,time).x();
        od->y = sclient->object(ball,0,time).y();
}

extern "C" int RSC_n_partner(int time)
{
        return sclient->get_objects(partner,time);
}

extern "C" int RSC_partner(int time, int i, object_data_t *od)
{
        int found;

        found = (i < sclient->get_objects(partner,time));
        
        if (found) {
                od->id = sclient->object(partner,i,time).id();
                od->dist = sclient->object(partner,i,time).dist();
                od->dir = sclient->object(partner,i,time).dir();
                od->ddist = sclient->object(partner,i,time).ddist();
                od->ddir = sclient->object(partner,i,time).ddir();
                od->x = sclient->object(partner,i,time).x();
                od->y = sclient->object(partner,i,time).y();
                od->facedir = sclient->object(partner,i,time).facedir();
        };

        od->name = o_partner;

        return found;
}


/* opponents */
extern "C" int RSC_n_opponent(int time)
{
        return sclient->get_objects(opponent,time);
}

extern "C" int RSC_opponent(int time, int i, object_data_t *od)
{
        int found = (i < sclient->get_objects(opponent,time));
        
        if (found) {
                od->id = sclient->object(opponent,i,time).id();
                od->dist = sclient->object(opponent,i,time).dist();
                od->dir = sclient->object(opponent,i,time).dir();
                od->ddist = sclient->object(opponent,i,time).ddist();
                od->ddir = sclient->object(opponent,i,time).ddir();
                od->x = sclient->object(opponent,i,time).x();
                od->y = sclient->object(opponent,i,time).y();
                od->bodydir = sclient->object(opponent,i,time).bodydir();
                od->facedir = sclient->object(opponent,i,time).facedir();
        }

        od->name = o_opponent;

        return found;
}


/* unknown players */
extern "C" int RSC_n_player(int time)
{
        return sclient->get_objects(player,time);
}

extern "C" int RSC_player(int time, int i, object_data_t *od)
{
        int found = (i < sclient->get_objects(player,time));
        
        if (found) {
                od->id = sclient->object(player,i,time).id();
                od->dist = sclient->object(player,i,time).dist();
                od->dir = sclient->object(player,i,time).dir();
                od->ddist = sclient->object(player,i,time).ddist();
                od->ddir = sclient->object(player,i,time).ddir();
                od->x = sclient->object(player,i,time).x();
                od->y = sclient->object(player,i,time).y();
                od->bodydir = sclient->object(player,i,time).bodydir();
                od->facedir = sclient->object(player,i,time).facedir();
        }

        od->name = o_player;

        return found;
}


/* sense body predicates */
extern "C" void RSC_sense_body_0()
{
        sclient->simple->sense_body();
}

extern "C" void RSC_sense_body_9(int *vwidth, int *vquality, double *stamina,
                               double *effort, double *speed,
                               long int *kicks, long int *dashs,
                               long int *turns, long int *says)
{
        sclient->simple->wait4outq(0);
        sclient->simple->sense_body();
        RSC_await_refresh();

        *vwidth = sclient->knowledge(-1).get_view_width();
        *vquality = sclient->knowledge(-1).get_view_quality();
        *stamina = sclient->knowledge(-1).get_stamina();
        *effort = sclient->knowledge(-1).get_effort();
        *speed = sclient->knowledge(-1).get_speed();
        *kicks = sclient->knowledge(-1).get_kicks();
        *dashs = sclient->knowledge(-1).get_dashs();
        *turns = sclient->knowledge(-1).get_turns();
        *says = sclient->knowledge(-1).get_says();
}

/* client actions */
extern "C" void RSC_moveto(double xpos, double ypos)
{
        sclient->simple->moveto(xpos, ypos);
}

extern "C" void RSC_turn(double angle)
{
        sclient->simple->turn(angle);
}

extern "C" void RSC_turn_neck(double angled)
{
        if (sclient->knowledge(-1).get_version() >= 5.0)
                sclient->simple->turn_neck(angled);
}

extern "C" void RSC_dash(double power)
{
        sclient->simple->dash(power);
}

extern "C" void RSC_kick(double power, double angle)
{
        sclient->simple->kick(power, angle);
}

// say command
extern "C" void RSC_say(const char *text)
{
        char msg[MaxStringSize];
        sclient->text2msg(text, (char *) msg);
        sclient->simple->say((char *) msg);
}

// goalie catch command
extern "C" void RSC_catch(double angle)
{
        sclient->simple->goalie_catch(angle);
}


Cvector bvel;
double  bddist;
double  bddir;
double  kick_margin(double ddir);
int     dokick(double dir, double dist, bool doit);


int dokick(double dist, double dir, bool doit)
{
        double  dir_diff;
        
        dir_diff = normalize_angle(sclient->object(ball, 0, -1).dir(false)
                                   -deg2rad(dir), false);
        dir_diff = fabs(dir_diff);
        double bdist = sclient->object(ball, 0, -1).dist()
                - player_size - ball_size;
        double kpr = kick_power_rate * (1 - 0.25 * dir_diff / M_PI
                                        - 0.25 * bdist / kickable_margin);
                
        double v0    = dist * (1 - ball_decay);
        double power = v0  / kpr;

        if (bvel != 0) {
                Cvector tmp;
                tmp.set((v0 > 100 ? 100 : v0), AngleDeg(dir));
                tmp -= bvel;
                dir = rad2deg(tmp.dir());
                power = tmp.len() / kpr;
        }

        if (power > 100) {
                double d = kick_margin(dir);
                if (d / kpr >= 50) {
                        double pow = d / kpr - 10;
                        if (power - 100 < pow) pow = power - 100;
                        if (pow > 100) {
                                double pw = (pow - 100) / 2;
                                if (pw > 100) pw = 100;
                                if (pw > 50) {
                                        sclient->simple->kick(pw, dir);
                                }
                                pow = 100;
                        }
                        sclient->simple->kick(pow, dir);
                        if (power - pow < 100) power -=pow;
                        else power = 100;
                }
                if (power > 100) power = 100;
        }

        if (power > 0.5 || doit) {
                sclient->simple->kick(power, dir);
                return 1;
        }
        else return 0;
}



double kick_margin(double ddir) {

        double x, y, a, s;

        Cvector bdest(sclient->object(ball, 0, -1).dist() + bddist,
                      sclient->object(ball, 0, -1).dir() + bddir);

        if (bdest.len() > kickable_area
            || sclient->object(ball, 0, -1).dist() < 0)
                return 0;
        if (bdest.len() < player_size + ball_size) {
                double r = ball_size + player_size;
                double d = bdest.len();
                double th = fabs(bddir - rad2deg(bdest.dir()));

                double l1 = d * cos(deg2rad(th));
                double h = d * sin(deg2rad(th));
                double cosp = h / r;
                double sinp = sqrt(1.0 - sqr(deg2rad(cosp)));
                double l2 = r * sinp;
                bdest += bvel*(-(l1+l2)/max(bvel.len(), 1.0e-10));
        }

        if (fabs(ddir) <= 45 || fabs(ddir) >= 135) {
                x = bdest.x();
                y = bdest.y();
                a = tan(deg2rad(ddir));
                s = fabs(ddir) <= 45 ? 1 : -1;
        } else {
                x = bdest.y();
                y = bdest.x();
                a = 1 / tan(deg2rad(ddir));
                s = (ddir >= 0 ? 1 : -1);
        }

        double b = y - a*x;
        double c = a*a + 1;
        double delta = a*a * b*b - c * ( b*b - kickable_area*kickable_area);
        if (delta < 0) return 0;
        else {
                double dx = (-a*b + s*sqrt(delta)) / c;
                double dy = a*dx + b;
                return hypot(dx - x, dy - y);
        }
}


extern "C" void RSC_kick2(double dist, double dir) 
{
        dokick(dist, dir, true);
}


extern "C" void RSC_kick3(double dist, double dir, long int wise) 
{
        bddist = normalize_value(sclient->object(ball, 0, -1).ddist());
        bddir  = normalize_value(sclient->object(ball, 0, -1).ddir(false));
                
        bvel.set(bddist, sclient->object(ball, 0, -1).dist() * tan(bddir));
        bvel.rotate(sclient->object(ball, 0, -1).dir(false), false);
        
        double  tmpdir;
        Cvector brpos, btraj;
        
        brpos.set(sclient->object(ball, 0, -1).dist(),
                  AngleRad(sclient->object(ball, 0, -1).dir(false)));
        btraj.set(dist, AngleDeg(dir));
        btraj -= brpos;

        double ndir = rad2deg(btraj.dir());
        double bdir = rad2deg(brpos.dir());
        ndir = normalize_angle(ndir - bdir, true);
        tmpdir = 90 + ACos((player_size + 2 * ball_size) / brpos.len(), true);
        if (fabs(ndir) < tmpdir
            || btraj.len() < (brpos.sqrlen()
                              - sqr(player_size + 2 * ball_size))) {
                tmpdir = normalize_angle(ndir + bdir, true);
                dokick(btraj.len(), tmpdir, true);
// return 1
        }       
                
        if (wise != 1) {
                if (normalize_angle(bdir) < 0)
                        wise = 1;
                else
                        wise = -1;
        }
                
        tmpdir = (wise >= 0 ? normalize_angle(tmpdir + bdir)
                  : normalize_angle(-tmpdir + bdir));
        
        double turnkick = 10;
                
        dokick(turnkick, tmpdir, 0);
}

/**
 * debugging stuff
 */
extern "C" void RSC_debug(FILE *file, unsigned char levels)
{
        bugfile(file, levels);
}

extern "C" void RSC_debugfile(const char *filename, unsigned char levels)
{
        bugfile(filename, levels);
}

extern "C" void RSC_debugarea(unsigned short area)
{
        robobug_area(area);
}


/** ------------------------------------------------------------ *
 *  for old prolog functions -- depreciated, do not use          *
 ** ------------------------------------------------------------ */

extern "C" void RSC_ball_dep(object_data_t *od)
{
        od->name = o_ball;
        od->id = 0;
        od->facedir = bearing_unknown;
        od->dist = sclient->object(ball,0).dist();
        od->dir = sclient->object(ball,0).dir();
        od->ddist = sclient->object(ball,0).ddist();
        od->ddir = sclient->object(ball,0).ddir();
        od->x = sclient->object(ball,0).x();
        od->y = sclient->object(ball,0).y();
}

extern "C" int RSC_goal_dep(int goalid, object_data_t *od)
{ 
        position gp;

        if (goalid != id_goal_left && goalid != id_goal_right)
                return 0;

        gp = goals[goalid];

        // if no view to requested goal exists, we have
        // to calculate the relpos first :-(          )
        if (sclient->get_objects(goal) == 0
            || (sclient->get_objects(goal) == 1
                && sclient->object(goal,0).id() != goalid)) {
                double gdir = 180 - sclient->objects().myheadangle();
                double gxdist = sclient->objects().mypos().x() - gp.x;
                double gydist = sclient->objects().mypos().y() - gp.y;
                
                if (sclient->objects().mypos().x() > -9000
                    && sclient->objects().mypos().y() > -9000) {
                        od->dist = sqrt(gxdist * gxdist
                                                 - gydist * gydist);
                        if (gdir < 9000) {
                                gdir += 57.29578 * atan2(gydist, gxdist);
                                gdir = gdir < -180 ? gdir + 360
                                        : gdir > 180 ? gdir - 360 : gdir;
                                od->dir = gdir;
                        } else {
                                od->dir = gdir;
                        }
                } else {
                        od->dist = bearing_unknown;
                        od->dir = bearing_unknown;
                }
                od->ddist = bearing_unknown;
                od->ddir = bearing_unknown;
        } else {
                // we can retrieve the relpos from the view
                if (sclient->object(goal,0).id() == goalid) {
                        od->dist = sclient->object(goal,0).dist();
                        od->dir  = sclient->object(goal,0).dir();
                        od->ddist = sclient->object(goal,0).ddist();
                        od->ddir = sclient->object(goal,0).ddir();
                } else {
                        od->dist = sclient->object(goal,1).dist();
                        od->dir  = sclient->object(goal,1).dir();
                        od->ddist = sclient->object(goal,1).ddist();
                        od->ddir = sclient->object(goal,1).ddir();
                }
        }

        od->name = o_goal;
        od->id = goalid;
        od->x = gp.x;
        od->y = gp.y;
        od->facedir = bearing_unknown;
        
        return -1;
}

extern "C" int RSC_n_player_dep()
{
        return sclient->get_objects(player);
}

extern "C" int RSC_player_dep(int i, object_data_t *od)
{
        int found = (i < sclient->get_objects(player));
        
        if (found) {
                od->id = sclient->object(player,i).id();
                od->dist = sclient->object(player,i).dist();
                od->dir = sclient->object(player,i).dir();
                od->ddist = sclient->object(player,i).ddist();
                od->ddir = sclient->object(player,i).ddir();
                od->x = sclient->object(player,i).x();
                od->y = sclient->object(player,i).y();
                od->facedir = sclient->object(player,i).facedir();
        }

        od->name = o_player;

        return found;
}

extern "C" int RSC_n_partner_dep()
{
        return sclient->get_objects(partner);
}

extern "C" int RSC_partner_dep(int i, object_data_t *od)
{
        int found = (i < sclient->get_objects(partner));
        
        if (found) {
                od->id = sclient->object(partner,i).id();
                od->dist = sclient->object(partner,i).dist();
                od->dir = sclient->object(partner,i).dir();
                od->ddist = sclient->object(partner,i).ddist();
                od->ddir = sclient->object(partner,i).ddir();
                od->x = sclient->object(partner,i).x();
                od->y = sclient->object(partner,i).y();
                od->facedir = sclient->object(partner,i).facedir();
        }

        od->name = o_partner;

        return found;
}

extern "C" int RSC_n_opponent_dep()
{
        return sclient->get_objects(opponent);
}

extern "C" int RSC_opponent_dep(int i, object_data_t *od)
{
        int found = (i < sclient->get_objects(opponent));
        
        if (found) {
                od->id = sclient->object(opponent,i).id();
                od->dist = sclient->object(opponent,i).dist();
                od->dir = sclient->object(opponent,i).dir();
                od->ddist = sclient->object(opponent,i).ddist();
                od->ddir = sclient->object(opponent,i).ddir();
                od->x = sclient->object(opponent,i).x();
                od->y = sclient->object(opponent,i).y();
                od->bodydir = sclient->object(opponent,i).bodydir();
                od->facedir = sclient->object(opponent,i).facedir();
        }

        od->name = o_opponent;

        return found;
}


// ----------------------------------------------------------------------------
// self(?X, ?Y, ?Dir)
// return/set position
extern "C" void RSC_self_dep(double *xpos, double *ypos,
                             double *angle, int set)
{
        int time = sclient->last_refresh();
        
        if (set) {
                sclient->calc_abs(*xpos, *ypos, *angle, *angle, time);
        } else {
                *xpos = sclient->objects(time).mypos().x();
                *ypos = sclient->objects(time).mypos().y();
                *angle = sclient->objects(time).myheadangle();
        }
}


/*
  simulator 4.19 kick

  void Player::kick(Value power, Angle dir) 
{
	MPObject* ball = stadium->ball ;
	PVector tmp ;
	Value dir_diff ;
	Value dist_ball ;

	alive |= KICK ;

	if (stadium->mode == PM_BeforeKickOff ||
		stadium->mode == PM_AfterGoal_Left ||
		stadium->mode == PM_AfterGoal_Right  ||
		stadium->mode == PM_OffSide_Left ||
		stadium->mode == PM_OffSide_Right )
		return ;

	if(pos.distance(ball->pos) > stadium->kickable_area) {
		alive |= KICK_FAULT ;
		return ;
	}

	if (stadium->useoffside)
		if (stadium->mark_offside_mark(this))
			return ;

	dir_diff = fabs(this->vangle(*ball)) ;
	tmp = ball->pos - this->pos ;
	dist_ball = tmp.r() - stadium->psize - stadium->bsize ;

	power *= stadium->kprate *
			(1 - 0.25*dir_diff/PI - 0.25*dist_ball/stadium->kmargin) ;

	ball->push(Polar2PVector(power, dir+angle)) ;
	ball->angle = dir+angle ;
	stadium->last_touch = this ;
	stadium->last_kicker = this ;
}



inline PVector Polar2PVector(Value r, Angle ang)
{
	return PVector(r * cos(ang), r * sin(ang)) ;
}


*/

