/* -*- mode: c -*-
   
   this file is part of the robolog libraries
   Copyright (C) Jul 29 1998 Oliver Obst
   header file for the robolog c library
   $Id: robolog.h,v 1.5 1999/05/05 14:52:08 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Robolog Soccer Client interface

   suggestions -> <frvit@uni-koblenz.de>. thanks.
*/
#ifndef _ROBOLOG_H
#define _ROBOLOG_H

#include <stdio.h>

typedef struct 
{
        int name;
        int id;
        double x, y;
        double dist, dir;
        double ddist, ddir;
        double bodydir;
        double facedir;
} object_data_t;

enum DebugLevel { ROBOBUG_INFO = 1,
                  ROBOBUG_WARN = 2,
                  ROBOBUG_ERROR = 4,
                  ROBOBUG_FATAL = 8 };

#ifdef __cplusplus
extern "C" {
#endif

#if 0 // only to switch of stupid xemacs indenting
}
#endif

// Robolog Soccer Client Functions
extern void RSC_init_player(const char *teamname, const char *servername,
                            int portnr, double version);
extern void RSC_init_goalie(const char *teamname, const char *servername,
                            int portnr, double version);
extern void RSC_stop();

extern int RSC_open_log(const char *filename);
extern void RSC_close_log();
extern int RSC_open_audiolog(const char *filename);
extern void RSC_close_audiolog();
extern int RSC_start_viewserver(int base);
extern void RSC_stop_viewserver();

extern long int RSC_simstep();
extern long int RSC_roboclock();

extern void RSC_refresh();
extern void RSC_await_refresh();
extern void RSC_wait_for_robo(int outq_size);
extern void RSC_dequeue();
extern int RSC_outqlen();

extern char* RSC_teamname(int team);
extern int RSC_uniformnumber();
extern int RSC_is_goalie();
extern int RSC_goalie(int team);

extern double RSC_stamina();
extern double RSC_effort();
extern double RSC_recovery();        
extern double RSC_speed();
extern int RSC_kicks();
extern int RSC_dashs();
extern int RSC_turns();
extern int RSC_turn_necks();
extern int RSC_says();
extern int RSC_goals(int time, int side);
extern int RSC_myside();
extern void RSC_self(int time, double *xpos, double *ypos,
                     double *angle1, double *angle2, int set);
extern double RSC_neck(int time);
extern double RSC_body_angle(int time);
        
extern double RSC_atan2(double y, double x);
extern double RSC_deg2rad(double deg);
extern double RSC_rad2deg(double rad);
extern int RSC_dist(double xpos, double ypos, double *dist);
extern int RSC_absangle(double xpos, double ypos, double *angle);

/* maths line stuff */
extern int RSC_point_on_line(double px, double py,
                             double x0, double y0, double x1, double y1);
extern int RSC_point_between(double px, double py,
                             double x0, double y0, double x1, double y1);
extern double RSC_dist_point_line(double px, double py,
                                  double x0, double y0, double x1, double y1);
extern double RSC_line_angle(double x0, double y0, double x1, double y1);

/* maths rectangle stuff */
extern int RSC_point_in_box(double px, double py, double x0, double y0,
                            double x1, double y1);
extern void RSC_nearest_edge(double px, double py, double x0, double y0,
                             double x1, double y1, double *x2, double *y2);
extern double RSC_distance_to_edge(double px, double py, double x0,
                                   double y0, double x1, double y1);
extern void RSC_closest_point_inside(double px, double py, double x0,
                                     double y0, double x1, double y1,
                                     double *x2, double *y2);

extern int RSC_dir_quality(double dir);
extern int RSC_dist_quality(double dist);
extern int RSC_pred_quality(int q0, int qmax, int circular);
extern int RSC_succ_quality(int q0, int qmax, int circular);

extern int RSC_msgidx();
extern int RSC_nextmsgidx();
extern int RSC_n_message(int sender, int time);
extern void RSC_message(int sender, int time, int i, char *txt, double *dir);

extern int RSC_object(int time, int i, object_data_t *od);
extern int RSC_goal(int time, int goalid, object_data_t *od);
extern int RSC_flag(int time, int flagid, object_data_t *od);
extern int RSC_line(int time, int lineid, object_data_t *od);
extern int RSC_partner(int time, int i, object_data_t *od);
extern int RSC_opponent(int time, int i, object_data_t *od);
extern int RSC_player(int time, int i, object_data_t *od);
extern void RSC_ball(int time, object_data_t *od);


extern int RSC_n_ball(int time);
extern int RSC_n_goal(int time);        
extern int RSC_n_line(int time);
extern int RSC_n_flag(int time);
extern int RSC_n_partner(int time);
extern int RSC_n_opponent(int time);
extern int RSC_n_player(int time);

extern void RSC_sense_body_0();
extern void RSC_sense_body_9(int *vwidth, int *vquality, double *stamina,
                             double *effort, double *speed, long int *kicks,
                             long int *dashs, long int *turns, long int *says);

extern int RSC_referee();
extern void RSC_view_1(int *width, int set);
extern void RSC_view_2(int *width, int *quality,
                       int set_width, int set_quality);

extern void RSC_moveto(double xpos, double ypos);
extern void RSC_turn(double angle);
extern void RSC_turn_neck(double angled);
        
extern void RSC_dash(double power);
extern void RSC_kick(double power, double angle);
extern void RSC_say(const char *text);
extern void RSC_catch(double angle);

extern void RSC_kick2(double dist, double dir);
extern void RSC_kick3(double dist, double dir, long int wise);
        
extern void RSC_debug(FILE *file, unsigned char levels);
extern void RSC_debugfile(const char *filename, unsigned char levels);
extern void RSC_debugarea(unsigned short area);
        
/* for old prolog functions -- depreciated, do not use */
extern void RSC_ball_dep(object_data_t *od);
extern int RSC_goal_dep(int goalid, object_data_t *od);
extern int RSC_player_dep(int i, object_data_t *od);
extern int RSC_partner_dep(int i, object_data_t *od);
extern int RSC_opponent_dep(int i, object_data_t *od);
extern int RSC_n_player_dep();
extern int RSC_n_partner_dep();
extern int RSC_n_opponent_dep();
extern void RSC_self_dep();        

#ifdef __cplusplus
}
#endif

/* some constants */
#define kickable_area   (0.7 + 0.3 + 0.085)
#define MaxStringSize   4096

/* states of a game (referee messages) */
#define st_before_kick_off             1
#define st_kick_off_l                  2
#define st_kick_off_r                  3
#define st_kick_in_l                   4
#define st_kick_in_r                   5
#define st_corner_kick_l               6
#define st_corner_kick_r               7
#define st_goal_kick_l                 8
#define st_goal_kick_r                 9
#define st_free_kick_l                10
#define st_free_kick_r                11
#define st_play_on                    12
#define st_half_time                  13
#define st_time_up                    14
#define st_extend                     15
#define st_foul_l_                    16
#define st_foul_r_                    17
#define st_goal_l_                    18
#define st_goal_r_                    19
#define st_goalie_catch_ball_l        20
#define st_goalie_catch_ball_r        21
/* sides of the field */
#define no_side                        0
#define left_side                      1
#define right_side                     2
/* set or get ? */
#define SET                            1
#define GET                            0
/* view widths */
#define v_normal                       2
#define v_wide                         3
#define v_narrow                       1
/* view qualities */
#define v_high                         2
#define v_low                          1 
/* object types */
#define o_unknown                      0
#define o_ball                         1
#define o_player                       2
#define o_partner                      3
#define o_opponent                     4
#define o_goal                         5
#define o_flag                         6
#define o_line                         7 
/* object ID's */
#define bearing_unknown            -9999
#define  id_goal_left                   0
#define  id_goal_right                  1
#define  id_flag_l_t                    0
#define  id_flag_l_b                    1
#define  id_flag_c_t                    2
#define  id_flag_c_b                    3
#define  id_flag_r_t                    4
#define  id_flag_r_b                    5
#define  id_pflag_l_t                   6
#define  id_pflag_l_c                   7
#define  id_pflag_l_b                   8
#define  id_pflag_r_t                   9
#define  id_pflag_r_c                  10
#define  id_pflag_r_b                  11
#define  id_flag_c                     12
#define  id_gflag_l_t                  13
#define  id_gflag_l_b                  14
#define  id_gflag_r_t                  15
#define  id_gflag_r_b                  16
#define  id_oflag_t_l_10               17
#define  id_oflag_t_l_20               18
#define  id_oflag_t_l_30               19
#define  id_oflag_t_l_40               20
#define  id_oflag_t_l_50               21
#define  id_oflag_t_r_10               22
#define  id_oflag_t_r_20               23
#define  id_oflag_t_r_30               24
#define  id_oflag_t_r_40               25
#define  id_oflag_t_r_50               26
#define  id_oflag_b_l_10               27
#define  id_oflag_b_l_20               28
#define  id_oflag_b_l_30               29
#define  id_oflag_b_l_40               30
#define  id_oflag_b_l_50               31
#define  id_oflag_b_r_10               32
#define  id_oflag_b_r_20               33
#define  id_oflag_b_r_30               34
#define  id_oflag_b_r_40               35
#define  id_oflag_b_r_50               36
#define  id_oflag_l_t_10               37
#define  id_oflag_l_t_20               38
#define  id_oflag_l_t_30               39
#define  id_oflag_l_b_10               40
#define  id_oflag_l_b_20               41
#define  id_oflag_l_b_30               42
#define  id_oflag_r_t_10               43
#define  id_oflag_r_t_20               44
#define  id_oflag_r_t_30               45
#define  id_oflag_r_b_10               46
#define  id_oflag_r_b_20               47
#define  id_oflag_r_b_30               48
#define  id_oflag_t_0                  49
#define  id_oflag_b_0                  50
#define  id_oflag_l_0                  51
#define  id_oflag_r_0                  52
#define  id_line_l                      0
#define  id_line_r                      1
#define  id_line_t                      2
#define  id_line_b                      3
#define  id_unknown                 -9999

#endif /* _ROBOLOG_H */
