/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (c) Apr 14 1998 Oliver Obst
   soccer client class
   $Id: sclient.C,v 1.4 1999/05/05 14:37:42 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#if STDC_HEADERS
# include <string.h>
# include <stdlib.h>
#else
# if HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#include "constants.h"
#include "soccerconst.h"
#include "sclient.h"
#include "control.h"
#include "simple.h"
#include "objinfo.h"
#include "objarray.h"
#include "msginfo.h"
#include "msgarray.h"
#include "knowledge.h"
#include "listproc.h"
#include "ring.h"
#include "self.h"

Sclient::Sclient(const char *server, int port, double ver, int memsize)
{
        step = 0;
        idx = -1;
        oa = new ObjArray;
        ma = new MsgArray(MaxMessage);
        k = new Knowledge;
        k->set_version(ver);
        self = new Self(oa, k);
        sensor = new Listproc(oa, ma, k);
        simple  = new Simple(self, sensor, k, server, port, ver);
        objring = new ObjRing(memsize);
        msgring = new MsgRing(memsize);
        knowring = new KnowRing(memsize);
}

Sclient::~Sclient()
{
        delete knowring;
        delete msgring;
        delete objring;
        delete simple;
        delete sensor;
        delete k;
        delete self;
        delete ma;
        delete oa;
}

// --- public -----------------------------------------------------------------
void Sclient::calc_abs(double xpos, double ypos,
                       double alpha, double beta, int t)
{
        objring->peek(t).calc_abs(xpos, ypos, alpha, beta);
}

// --- public -----------------------------------------------------------------
ObjArray& Sclient::objects() 
{
        return objring->peek(idx);
}

// --- public -----------------------------------------------------------------
ObjArray& Sclient::objects(int t) 
{
        return objring->peek(t);
}

// --- public -----------------------------------------------------------------
MsgArray& Sclient::messages() 
{
        return msgring->peek(idx);
}

// --- public -----------------------------------------------------------------
MsgArray& Sclient::messages(int t) 
{
        return msgring->peek(t);
}

// --- public -----------------------------------------------------------------
Knowledge& Sclient::knowledge()
{
        return knowring->peek(idx);
}

// --- public -----------------------------------------------------------------
Knowledge& Sclient::knowledge(int time)
{
        return knowring->peek(time);
}

// --- public -----------------------------------------------------------------
ObjInfo& Sclient::object(Objtype name, int i)
{
        return object(name, i, idx);
}

// --- public -----------------------------------------------------------------
ObjInfo& Sclient::object(Objtype name, int i, int time)
{
        ObjArray &o = objring->peek(time);

        switch (name) {
        case line :
                if (i < o.get_lines()) return o.lines(i);
                break;
        case flag :
                if (i < o.get_flags()) return o.flags(i);
                break;
        case goal :
                if (i < o.get_goals()) return o.goals(i);
                break;
        case opponent :
                if (i < o.get_opponents()) return o.opponents(i);
                break;
        case partner :
                if (i < o.get_partners()) return o.partners(i);
                break;
        case player :
                if (i < o.get_players()) return o.players(i);
                break;
        case ball :
                if (i < o.get_balls()) return o.balls(i);
                break;
        default :
                return empty_obj;
        }
        return empty_obj;
}

// --- public -----------------------------------------------------------------
MsgInfo& Sclient::message(Msgtype name, int i)
{
        return message(name, i, idx);
}

// --- public -----------------------------------------------------------------
MsgInfo& Sclient::message(Msgtype name, int i, int time)
{
        MsgArray &m = msgring->peek(time);

        switch (name) {
        case ::self :
                if (i < m.get_ownmsgs()) return m.self(i);
                else return empty_msg;
                break;
        case referee :
                if (i < m.get_refmsgs()) return m.referee(i);
                else return empty_msg;
                break;
        case other :
                if (i < m.get_othermsgs()) return m.others(i);
                else return empty_msg;
                break;
        default :
                return m[i];
        }
}

// --- public -----------------------------------------------------------------
void Sclient::message(Msgtype name, int i, int t, char *txt, double *dir) 
{
        message(name, i, t).get(txt,dir);
}

// --- public -----------------------------------------------------------------
int Sclient::get_objects(Objtype name)
{
        return get_objects(name, idx);
}

// --- public -----------------------------------------------------------------
int Sclient::get_objects(Objtype name, int time)
{
        ObjArray o;
        objring->peek(o, time);

        switch (name) {
        case line :
                return o.get_lines();
                break;
        case flag :
                return o.get_flags();
                break;
        case goal :
                return o.get_goals();
                break;
        case opponent :
                return o.get_opponents();
                break;
        case partner :
                return o.get_partners();
                break;
        case player :
                return o.get_players();
                break;
        case ball :
                return o.get_balls();
                break;
        default :
                return 0;
        }
}


// --- public -----------------------------------------------------------------
int Sclient::get_messages(Msgtype name)
{
        return get_messages(name, idx);
}

// --- public -----------------------------------------------------------------
int Sclient::get_messages(Msgtype name, int time)
{
        MsgArray m;
        int i;

        msgring->peek(m, time);
        switch (name) {
        case ::self :
                i = m.get_ownmsgs();
                break;
        case referee :
                i = m.get_refmsgs();
                break;
        case other :
                i = m.get_othermsgs();
                break;
        default :
                i = m.get_entries();
        }
        
        return i;
}

// --- public -----------------------------------------------------------------
int Sclient::get_nextmsgidx()
{
        int last, next;

        last = msgring->get_nextkey();
        msgring->inc_nextkey();
        next = msgring->get_nextkey();

        return (last == next ? -1 : next);
}

// --- public -----------------------------------------------------------------
void Sclient::refresh()
{
        idx = sensor->get_time();
}

// --- public -----------------------------------------------------------------
void Sclient::update(int steps)
{
        char data[ MaxStringSize ];
        if (++step == steps) {
                step = 0;
                sensor->set_moved(false);

                simple->transceive();
                self->forget();

                
                if (simple->get_data(data)) {
                        sensor->parse(data);
                        
                        int now = sensor->get_time();
                        
                        if (sensor->get_newobj()) {
                                self->locate();
                                objring->store(*oa, now);
                                updated = true;
                        }
                        if (sensor->get_newmsg()) {
                                msgring->store(*ma, now);
                        }

                        estimate_stamina(now - knowring->last_key());
                        knowring->store(*k, now);
                }
        }
}

// --- public -----------------------------------------------------------------
void Sclient::view_update()
{
        char data[ MaxStringSize ];

        bool b = true;

        while (b) {
                simple->receive();
                b = simple->get_data(data);
                
                if (b) {
                        sensor->parse(data);
                        int now = sensor->get_time();
                        if (sensor->get_newobj()) {
                                self->locate();
                                objring->store(*oa, now);
                                updated = true;
                        }
                        if (sensor->get_newmsg()) {
                                msgring->store(*ma, now);
                        }
                        estimate_stamina(now - knowring->last_key());
                        knowring->store(*k, now);
                }
        }
}

// --- public -----------------------------------------------------------------
void Sclient::action_update(int steps)
{
        if (++step == steps) {
                sensor->set_moved(false);
                step = 0;
        }
        
        simple->dispatch();
        self->forget();
}

void Sclient::text2msg(const char *text, char *msg)
{
        char *s = text;
        msg[0] = '\0';

        if (text) {
                size_t i = 0;
                size_t j = 0;
                while (i < strlen(text)) {
                        j = strcspn(s, "?,'\"%&#");
                        if (j > 0) {
                                strncat(msg, s, j);
                                i += j;
                                s += j;
                        } else {
                                switch (s[0]) {
                                case '?' :
                                        strcat(msg, "?00");
                                        break;
                                case ',' :
                                        strcat(msg, "?01");
                                        break;
                                case '\'' :
                                        strcat(msg, "?02");
                                        break;
                                case '' :
                                        strcat(msg, "?03");
                                        break;
                                case '\"' :
                                        strcat(msg, "?04");
                                        break;
                                case '%' :
                                        strcat(msg, "?05");
                                        break;
                                case '&' :
                                        strcat(msg, "?06");
                                        break;
                                case '#' :
                                        strcat(msg, "?07");
                                        break;
                                default:
                                        strcat(msg, "?FF");
                                }
                                i++;
                                s++;
                        }
                }
        }
}
        
void Sclient::msg2text(const char *msg, char *text)
{
        char tmp[3];
        char *s = msg;

        text[0] = '\0';
        tmp[2] = '\0';

        if (msg) {
                int i = 0;
                int j = 0;
                int k = strlen(msg);
                while (i < k) {
                        j = strcspn(s, "?");
                        if (j > 0) {
                                strncat(text, s, j);
                                i += j;
                                s += j;
                        } else {
                                i++;
                                if (i < k) {
                                        s++;
                                        if (i + 1 < k) j = 2;
                                        else j = 1;
                                        strncpy(tmp, s, j);
                                        i += j;
                                        s += j;
                                        switch (strtol(tmp,0,16)) {
                                        case  0 :
                                                strcat(text, "?");
                                                break;
                                        case  1 :
                                                strcat(text, ",");
                                                break;
                                        case  2 :
                                                strcat(text, "'");
                                                break;
                                        case  3 :
                                                strcat(text, "");
                                                break;
                                        case  4 :
                                                strcat(text, "\"");
                                                break;
                                        case  5 :
                                                strcat(text, "%");
                                                break;
                                        case  6 :
                                                strcat(text, "&");
                                                break;
                                        case  7 :
                                                strcat(text, "#");
                                                break;
                                        default:
                                                strcat(text, "???");
                                        }
                                }
                        }
                }
        }
}

void Sclient::estimate_stamina(int simsteps)
{
        double stamina = k->get_stamina();
        double effort = k->get_effort();
        double recovery = k->get_recovery();
        
        for (int i=0; i < simsteps; i++) {
                if (stamina <= recovery_dec_thr * stamina_max)
                        if (recovery > recovery_min)
                                recovery -= recovery_dec;

                if (stamina <= effort_dec_thr * stamina_max)
                        if (effort > effort_min) 
                                effort -= effort_dec;

                if (stamina >= effort_inc_thr * stamina_max) 
                        if (effort < 1.0)
                                effort += effort_inc;

                if (stamina < stamina_max)
                        stamina += recovery * stamina_max_inc;
        }
        
        k->set_stamina(stamina);
        k->set_effort(effort);
        k->set_recovery(recovery);
}

               
