/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Oct 21 1997 Oliver Obst
   a soccer client class
   $Id: sclient.h,v 1.4 1999/05/05 14:35:28 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _SCLIENT_H
#define _SCLIENT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "constants.h"
#include "control.h"
#include "simple.h"
#include "objinfo.h"
#include "objarray.h"
#include "msginfo.h"
#include "msgarray.h"
#include "knowledge.h"
#include "listproc.h"
#include "ring.h"
#include "self.h"

class Sclient
{
public:
        Sclient(const char *server, int port, double ver, int memsize=100);
        virtual ~Sclient();

        void            refresh();
        void            update(int steps);
        void            view_update();
        void            action_update(int steps);
        
        void            calc_abs(double xpos, double ypos,
                                 double alpha, double beta, int t);

        ObjArray&       objects();
        ObjArray&       objects(int t);
        MsgArray&       messages();
        MsgArray&       messages(int t);
        Knowledge&      knowledge();
        Knowledge&      knowledge(int t);

        ObjInfo&        object(Objtype name, int i);
        ObjInfo&        object(Objtype name, int i, int t);
        MsgInfo&        message(Msgtype name, int i);
        MsgInfo&        message(Msgtype name, int i, int t);
        void            message(Msgtype name, int i, int t,
                                char *txt, double *dir);

        int             get_objects(Objtype name);
        int             get_objects(Objtype name, int t);

        int             get_messages(Msgtype name);
        int             get_messages(Msgtype name, int t);

        inline int      get_msgidx() { return msgring->get_nextkey(); }
        int             get_nextmsgidx();

        inline void     set_updated(bool update) { updated = update; }
        inline volatile bool get_updated() { return updated; }
        inline int      get_time() { return sensor->get_time(); }
        inline int      last_refresh() { return idx; }

        void            text2msg(const char *text, char *msg);
        void            msg2text(const char *msg, char *text);
        
        Simple          *simple;

private:
        void            estimate_stamina(int simsteps);
        
        ObjInfo empty_obj;
        MsgInfo empty_msg;
        Knowledge brain;

        Listproc *sensor;
        ObjArray *oa;
        MsgArray *ma;
        Knowledge *k;
        Self *self;
        ObjRing *objring;
        MsgRing *msgring;
        KnowRing *knowring;

        int step;
        int idx;
        
        volatile bool updated;
};
#endif /* _SCLIENT_H */
