/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Dec 01 1997 Oliver Obst
   methods for robot self localisation
   $Id: self.C,v 1.5 1999/05/05 14:39:42 frvit Exp $

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <math.h>
#include <stdlib.h>
#include <iostream.h>

#include "maths.h"
#include "self.h"
#include "sfield.h"
#include "objarray.h"
#include "knowledge.h"

Self::Self(ObjArray *oa0, Knowledge *k0) : Sfield(k0)
{
        oa = oa0;
        k = k0;
        insects = 0;
        definite = -1;
        lastangle = bearing_unknown;
        lastpos.set(bearing_unknown, bearing_unknown);
        angle = bearing_unknown;
        pos.set(bearing_unknown, bearing_unknown);
        remember = 0;
}


Self::~Self()
{
}


// --- public -----------------------------------------------------------------
// bestimmt die ausrichtung des spielers. das klappt hier nur, wenn die
// eigene absolute position klar ist.
// das kann aber auch sonst klappen, wenn man die linien auch beruecksichtigt.
// in dieser version werden nur punkte beruecksichtigt.
void Self::direction()
{
        int n = 0;
        double gamma_s = 0;                

        if (pos.x() != bearing_unknown && pos.y() != bearing_unknown) {
                double alpha, gamma;
                Cvector d;

                for (int i=0; i<oa->get_points(); i++) {
                        if (oa->points(i).id() != id_unknown) {
                                // get angle in rad
                                alpha = oa->points(i).dir(false);
                                if (alpha != bearing_unknown) {
                                        d = getpos(oa->points(i)) - pos;

                                        gamma = d.dir() - alpha;
                                        if ( n == 0 ) {
                                                gamma_s += gamma;
                                                n++;
                                        }
                                }
                        }
                        i++;
                }
        }

        if ( n == 0 )
                angle = bearing_unknown;
        else
                angle = gamma_s / n;
}


void Self::intersections()
{
        // berechnet noch keine schnittpunkte mit linien...
        double  a, b, c, h;
        double  asqr, bsqr, csqr, hsqr;
        double  a1, a2, d1, e1, dh, eh, sh;

        Cvector vd;
        Cvector vdsqr;
        Cvector v0, v1;

        definite = -1;
        insects = oa->get_points() - 1;
        
        // zum berechnen von schnittpunkten mit linien
        // ist die erste bedingung ".. && insects < 1)"
        if ((lp == 0 && insects < 0) || (insects < 0)) {
                // zu wenig schnittpunkte
                insects = 0;
                return;
        }

        // Referenzpunkt ist der erste feste Punkt, von dem auch
        // die Objekt ID bekannt ist.
        int anf = -1;
        bool unknown;
        do {
                anf++;
                c = oa->points(anf).dist();
                unknown = (oa->points(anf).id() == id_unknown);
        }
        while (anf < insects && unknown );

        // Gibt es jetzt nicht mehr genug Schnittpunkte, war es das schon
        if (insects - anf < 1) {
                insects = 0;
                return;
        }

        csqr = c * c;
        v0 = getpos(oa->points(anf));

        for (int i=0; i < insects; i++) {
                if (oa->nonmoving(i).id() == id_unknown) {
                        intersec[0][i].set_authentic(false);
                        intersec[1][i].set_authentic(false);
                } else {
                        b = oa->nonmoving(i).dist();
                        v1 = getpos(oa->nonmoving(i));
                        vd = v0 - v1;

                        bsqr = b * b;
                        vdsqr.set(vd.x()*vd.x(), vd.y()*vd.y());

                        asqr = vdsqr.x() + vdsqr.y();

                        a = v1.x() < v0.x() ? sqrt(asqr) : -sqrt(asqr);
                        a2 = (asqr + bsqr - csqr) / (2 * a);


                        a1 = a - a2;
                
                        hsqr = bsqr - a2 * a2;
                        h = sqrt(hsqr);
                
                        if (vd.y() == 0) {
                                intersec[0][i].set(v1.x() + a2, v1.y() + h);
                                intersec[1][i].set(v1.x() + a2, v1.y() - h);
                        } else {
                                sh = -vd.x() / vd.y();
                                eh = sqrt(hsqr / (sh * sh + 1 ));
                                dh = sh * eh;
                                e1 = a1 * vd.x() / a;
                                d1 = a1 * vd.y() / a;
                        
                                intersec[ 0 ][i].set(v0.x() - e1 + eh,
                                                       v0.y() - d1 + dh);
                                intersec[ 1 ][i].set(v0.x() - e1 - eh,
                                                       v0.y() - d1 - dh);
                        }

//                           cout << i << "Schnittpunkt 0: " << intersec[0][i]
//                                << "     Schnittpunkt 1: " << intersec[1][i]
//                                << "\n";

                        for (int k=0; k < 2; k++) {
                                if (intersec[k][i].x() > Xmin
                                    && intersec[k][i].x() < Xmax
                                    && intersec[k][i].y() > Ymin
                                    && intersec[k][i].y() < Ymax) {
                                
                                        intersec[k][i].set_authentic(true);
                                        if (k == 1 &&
                                            !intersec[0][i].authentic())
                                                definite = i;
                                } else {
                                        intersec[k][i].set_authentic(false);
                                        if (k == 1 &&
                                            intersec[0][i].authentic())
                                                definite = i;
                                }
                        }
                        
                }
        }  
}
                        
                        
void Self::linecheck()
{
        // berechnet in dieser version nur moegliche x- oder y- positionen,
        // nicht aber den richtungswinkel des spielers.
        lp = oa->get_lines();
        
        for (int i=0; i < lp; i++) {
                linecalc(i);
        }
}


void Self::linecalc(int i)
{
        // beta ist der (gemessene) Winkel zwischen Randlinie und Sichtlinie
        // tau ist der (feste) Winkel der Randlinie
        // d ist die Lnge der Sichtlinie

        // h = d * sin( beta ) ist die Entfernung des Robos von der Randlinie
        // phi = beta + tau    ist der Richtungswinkel des Robos

        double dx, dy;
        double beta = oa->lines(i).dir(false);
        double d = oa->lines(i).dist();
                
        if (beta != bearing_unknown) {
                switch (oa->lines(i).id()) {
                case id_line_l :
                        dx = d * sin(beta);
                        linepos[0][i].set(lines[ id_line_l ].x + dx,
                                          bearing_unknown);
                        linepos[1][i].set(lines[ id_line_l ].x - dx,
                                          bearing_unknown);
                        break;
                case id_line_b :
                        dy = d * sin(beta);
                        linepos[0][i].set(bearing_unknown,
                                          lines[ id_line_b ].y + dy);
                        linepos[0][i].set(bearing_unknown,
                                          lines[ id_line_b ].y - dy);
                        break;
                case id_line_r :
                        dx = d * sin(beta);
                        linepos[0][i].set(lines[ id_line_r ].x + dx,
                                          bearing_unknown);
                        linepos[0][i].set(lines[ id_line_r ].x - dx,
                                          bearing_unknown);
                        break;                        
                case id_line_t :
                        dy = d * sin(beta);
                        linepos[0][i].set(bearing_unknown,
                                              lines[ id_line_t ].y + dy);
                        linepos[0][i].set(bearing_unknown,
                                              lines[ id_line_t ].y - dy);
                        break;
                }
        }
}

// calculates an agent position.
// gets a point and a line with full information and calculates two
// possible positions. If the point is not on the line, the right position 
// out of the two possible positions can be calculated. If the point is on
// the line, we estimate the new position to be the one closer to the last
// known position.
void Self::hq_point_line(ObjInfo &point, ObjInfo &line,
                         Cvector &pos, double &theta)
{
        double theta1, theta2;
        double alpha1, alpha2;
        double err1, err2;
        double x1, x2;
        double y1, y2;
        double phi1, phi2;

        double x = bearing_unknown;
        double y = bearing_unknown;
        
        phi1 = point.dir(false);
        phi2 = line.dir(false);
                
       if (line.id() == id_line_l || line.id() == id_line_r) {
                // seen a short line
                x1 = getpos(line).x() - sgn(phi2) * line.dist() * sin(phi2);
                x2 = getpos(line).x() + sgn(phi2) * line.dist() * sin(phi2);
                // direction of the agent: theta = +/- PI/2 - phi_2
                
                theta1 = sgn(phi2) * M_PI_2 - phi2;
                theta2 = -sgn(phi2) * M_PI_2 - phi2;
                alpha1 = -phi1 - theta1;
                alpha2 = -phi1 - theta2;
                
                if (getpos(line).x() != getpos(point).x()) {
#ifdef DEBUG_SELF_PL                        
                        cout << "PNOL: ";
#endif                        
                        // the point is not on the line:
                        // calculate agent position via error minimization
                        err1 = getpos(point).x()
                                - point.dist() * cos(alpha1) - x1;
                        err2 = getpos(point).x()
                                - point.dist() * cos(alpha2) - x2;
                        if (abs(err1) < abs(err2)) {
                                theta = theta1;
                                x = x1;
                                y = getpos(point).y()
                                        + point.dist() * sin(alpha1);
                        } else {
                                theta = theta2;
                                x = x2;
                                y = getpos(point).y()
                                        + point.dist() * sin(alpha2);
                        }
#ifdef DEBUG_SELF_PL                        
                        cout << "Line.dir(): " << line.dir() << ", "
                             << "Line.x(): " << getpos(line).x() << ", "
                             << "(x,y,th): (" << x << ", "
                             << y << ", " << rad2deg(theta) << ")\n";
#endif                        
                } else {
#ifdef DEBUG_SELF_PL                        
                        cout << "POL: ";
#endif                        
                        // point is on the line:
                        // position estimation by calculating differences
                        // to the last known agent position
                        y1 = getpos(point).y() + point.dist() * sin(alpha1);
                        y2 = getpos(point).y() + point.dist() * sin(alpha2);


                        if (abs(lastpos.y()) < abs_unknown) {
#ifdef DEBUG_SELF_PL                        
                                cout << "(LX,LY): ("
                                     << lastpos.x() << ", "
                                     << lastpos.y() << ") ";
#endif                                
                                // there is a last known y position
                                err1 = lastpos.y() - y1;
                                err2 = lastpos.y() - y2;

                                if (abs(err1) < abs(err2)) {
                                        theta = theta1;
                                        x = x1;
                                        y = y1;
                                } else {
                                        theta = theta2;
                                        x = x2;
                                        y = y2;
                                }
                        } else {
                                if (abs(lastangle) < abs_unknown) {
                                        // there is a last known direction
#ifdef DEBUG_SELF_PL                        
                                        cout << "LA: " << rad2deg(lastangle)
                                             << " ";
#endif                                        
                                        err1 = lastangle - theta1;
                                        err2 = lastangle - theta2;
                                        
                                        if (abs(err1) < abs(err2)) {
                                                theta = theta1;
                                                x = x1;
                                                y = y1;
                                        } else {
                                                theta = theta2;
                                                x = x2;
                                                y = y2;
                                        }
                                } else {
                                // we don't know our last y position
                                // hence nothing can be estimated
                                y = bearing_unknown;
                                }
                        }
#ifdef DEBUG_SELF_PL                        
                        cout << "Line.dir(): " << line.dir() << ", "
                             << "Line.x(): " << getpos(line).x() << ", "
                             << "(x,y,th): (" << x << ", "
                             << y << ", " << rad2deg(theta) << ")\n";
#endif                        
                }
        } else {
                // seen a long line
                y1 = getpos(line).y() - sgn(phi2) * line.dist() * sin(phi2);
                y2 = getpos(line).y() + sgn(phi2) * line.dist() * sin(phi2);
                // direction of the agent: theta = +/- PI/2 - phi_2
                theta1 = (phi2 < 0) ? - phi2 : M_PI - phi2;
                theta2 = (phi2 < 0) ? -M_PI - phi2 : -phi2;
                alpha1 = -phi1 - theta1;
                alpha2 = -phi1 - theta2;

                if (getpos(line).y() != getpos(point).y()) {
#ifdef DEBUG_SELF_PL                        
                        cout << "-PNOL: ";
#endif                        
                        // if point is not on the line:
                        // calculate x position via error minimization
                        err1 = getpos(point).y()
                                + point.dist() * sin(alpha1) - y1;
                        err2 = getpos(point).y()
                                + point.dist() * sin(alpha2) - y2;
                        if (abs(err1) < abs(err2)) {
                                x = getpos(point).x()
                                        - point.dist() * cos(alpha1);
                                y = y1;
                                theta = theta1;
                        } else {
                                x = getpos(point).x()
                                        - point.dist() * cos(alpha2);
                                y = y2;
                                theta = theta2;
                        }
#ifdef DEBUG_SELF_PL                        
                        cout << "Line.dir(): " << line.dir() << ", "
                             << "Line.y(): " << getpos(line).y() << ", "
                             << "(x,y,th): (" << x << ", "
                             << y << ", " << rad2deg(theta) << ")\n";
#endif                        
                } else {
#ifdef DEBUG_SELF_PL                        
                        cout << "-POL: ";
#endif                        
                        // point is on the line:
                        // position estimation by calculating differences
                        // to the last known agent position
                        x1 = getpos(point).x() - point.dist() * cos(alpha1);
                        x2 = getpos(point).x() - point.dist() * cos(alpha2);

                        if (abs(lastpos.x()) < abs_unknown) {
#ifdef DEBUG_SELF_PL                        
                                cout << "(LX,LY): (";
#endif                                
                                // there is a last known x position
                                err1 = lastpos.x() - x1;
                                err2 = lastpos.x() - x2;

                                if (abs(err1) < abs(err2)) {
                                        theta = theta1;
                                        x = x1;
                                        y = y1;
                                } else {
                                        theta = theta2;
                                        x = x2;
                                        y = y2;
                                }
                        } else {
                                if (abs(lastangle) < abs_unknown) {
#ifdef DEBUG_SELF_PL                        
                                        cout << "LA: " << rad2deg(lastangle)
                                             << " ";
#endif                                        
                                        // there is a last known direction
                                        err1 = lastangle - theta1;
                                        err2 = lastangle - theta2;
                                        
                                        if (abs(err1) < abs(err2)) {
                                                theta = theta1;
                                                x = x1;
                                                y = y1;
                                        } else {
                                                theta = theta2;
                                                x = x2;
                                                y = y2;
                                        }
                                } else {
                                // we don't know our last x position
                                // hence nothing can be estimated
                                        x = bearing_unknown;
                                }
                        }
#ifdef DEBUG_SELF_PL                        
                        cout << "Line.dir(): " << line.dir() << ", "
                             << "Line.y(): " << getpos(line).y() << ", "
                             << "(x,y,th): (" << x << ", "
                             << y << ", " << rad2deg(theta) << ")\n";
#endif                        
                }
        }
        
        pos.set(x,y);
}

// calculates agent position
// gets two points with full information and calls lq_point_point()
// to calculate two possible agent positions. disambiguation is done
// by using the angles from the points information: only on the real
// agent position both angles from the points result in the same agent
// direction.
void Self::hq_point_point(ObjInfo &p1, ObjInfo &p2,
                          Cvector &pos, double &theta)
{
        Cvector delta1, delta2;
        Cvector pos1, pos2;
        Cvector alpha11, alpha12, alpha21, alpha22;
        double  theta11, theta12, theta21, theta22;
        double  phi1, phi2;
        double  thetad1, thetad2;


        lq_point_point(p1,p2,pos1,pos2);

        phi1 = p1.dir(false);
        phi2 = p2.dir(false);

        alpha11 = getpos(p1) - pos1; // 1st possibility, 1st point
        alpha12 = getpos(p2) - pos1; // 1st possibility, 2nd point
        alpha21 = getpos(p1) - pos2; // 2nd possibility, 1st point
        alpha22 = getpos(p2) - pos2; // 2nd possibility, 2nd point

        theta11 = phi1 - alpha11.dir(false);
        theta12 = phi2 - alpha12.dir(false);
        theta21 = phi1 - alpha21.dir(false);
        theta22 = phi2 - alpha22.dir(false);

        thetad1 = normalize_angle(theta12 - theta11, false);
        thetad2 = normalize_angle(theta22 - theta21, false);

        if (abs(thetad1) < abs(thetad2)) {
                theta = normalize_angle(theta11 + thetad1 / 2, false);
                pos = pos1;
        } else {
                theta = normalize_angle(theta21 + thetad2 / 2, false);
                pos = pos2;
        }
//        cout << "Winkel: " << theta << "\n";
}


// calculates two possible agent positions.
// gets two points with distance information and calculates two possible
// positions for the agent.
void Self::lq_point_point(ObjInfo &p1, ObjInfo &p2,
                          Cvector &pos1, Cvector &pos2) 
{
        double a,a1,a2,ap,apsqr,asqr;
        double b,b1,bp;
        double c,c1,cp;
        double d1sqr,d2sqr;
        double x1, x2;
        double y1, y2;

        double p1x, p2x;
        double p1y, p2y;

        p1x = getpos(p1).x();
        p2x = getpos(p2).x();
        p1y = getpos(p1).y();
        p2y = getpos(p2).y();        

        c = p1x - p2x;
        b = p1y - p2y;

        asqr = b*b + c*c;

        if (asqr != 0) {
                // ok: the points are different
                d1sqr = p1.dist() * p1.dist();
                d2sqr = p2.dist() * p2.dist();

                a = p2x < p1x ? sqrt(asqr) : - sqrt(asqr);
                
                a2 = (asqr + d2sqr - d1sqr) / (2*a);
                a1 = a - a2;
                apsqr = d2sqr - a2*a2;
                ap = apsqr > 0 ? sqrt(apsqr) : 0;

                c1 = c * a1 / a;
                b1 = b * a1 / a;
                cp = c * ap / a;
                bp = b * ap / a;

                x1 = getpos(p1).x() - c1 + bp;
                y1 = getpos(p1).y() - b1 - cp;
                x2 = getpos(p1).x() - c1 - bp;
                y2 = getpos(p1).y() - b1 + cp;
        } else {
                // oops: point1 == point2. that won't work.
                x1 = y1 = bearing_unknown;
                x2 = y2 = bearing_unknown;
        }

        pos1.set(x1,y1);
        pos2.set(x2,y2);
}


double Self::sqrdist(Cvector v0, Cvector v1)
{
        Cvector d = v1 - v0;        
        return d.sqrlen();
}


int Self::fuzzcmp(double d0, double d1)
{
        int dd = 0;
        if ( d0 < d1 - 0.2 * pitch_length)
                dd = -1;
        if ( d0 > d1 + 0.2 * pitch_length)
                dd = 1;
        return dd;
}


void Self::selection()
{
        Xvector r;
        int     npos;

//         cout << "definite: " << definite << "\n";
//         cout << "insects : " << insects  << "\n"
//              << "--------------\n";
        
        
        if (definite < 0 && insects > 1) {
                for (int i=1; i < insects; i++) {
//                         cout << intersec[0][0] << " [0][0]  ||"
//                              << intersec[0][i] << " [0][" << i << "]  ||"
//                              << intersec[1][0] << " [1][0]  || "
//                              << intersec[1][i] << " [1][" << i << "]\n";
                        
                        if (fuzzcmp(0, sqrdist(intersec[ 0 ][ 0 ],
                                               intersec[ 0 ][ i ]))
                             != 0 &&
                             fuzzcmp(0, sqrdist(intersec[ 1 ][ 0 ],
                                                intersec[ 0 ][ i ]))
                             != 0)
                                intersec[ 0 ][ i ].set_authentic(false);

                        if (fuzzcmp(0, sqrdist(intersec[ 0 ][ 0 ],
                                               intersec[ 1 ][ i ]))
                            != 0 &&
                            fuzzcmp(0, sqrdist(intersec[ 1 ][ 0 ],
                                               intersec[ 1 ][ i ]))
                            != 0 )
                                intersec[ 1 ][ i ].set_authentic(false);
                        
                        if (intersec[ 0 ][ i ].authentic()
                            ^ intersec[ 1 ][ i ].authentic())
                                definite = i;
                }
        }

        pos.set(0, 0);
        npos = 0;
        
        if (definite > 0 && insects > 1) {
                r = intersec[ 0 ][ definite ].authentic() ?
                        intersec[ 0 ][ definite ] : intersec[ 1 ][ definite ];

                for (int i=0; i < insects; i++) {
//                         cout << intersec[0][0] << " [0][0]  ||"
//                              << intersec[0][i] << " [0][" << i << "]  ||"
//                              << intersec[1][0] << " [1][0]  || "
//                              << intersec[1][i] << " [1][" << i << "]\n";
                        
                        if ((intersec[ 0 ][ i ].authentic())
                            && (fuzzcmp(0, sqrdist(intersec[ 0 ][ i ], r))
                                == 0)) {
                                pos += intersec[ 0 ][ i ];
                                npos++;
                        }

                        if ((intersec[ 1 ][ i ].authentic())
                            && (fuzzcmp(0, sqrdist(intersec[ 1 ][ i ], r))
                                == 0 )) {
                                pos += intersec[ 1 ][ i ];
                                npos++;  
                        }
                }
        }

//        cout << "Berechnete Positionen: " << npos << "\n";

        if ( npos > 0 ) {
                pos = pos / npos;
        } else {
                pos.set(bearing_unknown, bearing_unknown);
        }
}


// adds angle (in deg) to clients estimated heading. to be used when turning
// the client.
void Self::turn(double angle)
{
        if (abs(lastangle) < abs_unknown) {
                odoangle = lastangle - deg2rad(angle);
                remember = 10;
                while (odoangle > M_PI) odoangle = odoangle - M_PI - M_PI;
                while (odoangle < -M_PI) odoangle = odoangle + M_PI + M_PI;
        }
}

// set estimated position to x,y.
void Self::moveto(double x, double y)
{
        odopos.set(x,y);
        remember = 10;
}

// decrease odometer remembrance value
void Self::forget()
{
        if (remember > 0) remember--;
}

// compute position with 2 points
void Self::locate_2p(ObjInfo &p0, ObjInfo &p1, Cvector &pa, double &theta)
{
        Cvector p[2];
        
        if (abs(p0.dir()) < abs_unknown
            && abs(p1.dir()) < abs_unknown) {
                // full information: enough to calculate a position 
                hq_point_point(p0, p1, pa, theta);
        } else {
                // calculate two points and estimate one of the positions
                // by comparing to the last known position.
                lq_point_point(p0, p1, p[0], p[1]);
                if (sqrdist(p[0], lastpos) < sqrdist(p[1], lastpos))
                        pa = p[0];
                else
                        pa = p[1];

                theta = lastangle;
        }        
}


// compute position with 3 or more points for input
void Self::locate_3p(ObjInfo &p0, ObjInfo &p1, ObjInfo &p2,
                     Cvector &pa, double &theta)
{
        int i, j;
        Cvector p[4];
        double  d[4];
        
        if (abs(p0.dir()) < abs_unknown
            && abs(p1.dir()) < abs_unknown
            && abs(p2.dir()) < abs_unknown) {
                // full information: we could estimate our position exactly
                hq_point_point(p0, p1, pa, theta);
        } else {
                // only distance information -- pffffh. Lets estimate X and Y
                lq_point_point(p0, p1, p[0], p[1]);
                lq_point_point(p0, p2, p[2], p[3]);

                d[0] = sqrdist(p[0], p[2]);
                d[1] = sqrdist(p[0], p[3]);
                d[2] = sqrdist(p[1], p[2]);
                d[3] = sqrdist(p[1], p[3]);

                i = 0;
                for (j=1; j<4; j++)
                        if (d[i] > d[j]) i = j;
                        
                pa = p[i];
                theta = lastangle;
        }
}



void Self::locate()
{
        if (remember > 0) {
                lastpos = odopos;
                lastangle = odoangle;
        }


        if (oa->get_known_points() > 2) {
                // 3 or more known points: very well
                int i,j,k;
                
                i = 0;
                while (oa->points(i).id() == id_unknown) i++;
                j = i + 1;
                while (oa->points(j).id() == id_unknown) j++;
                k = j + 1;
                while (oa->points(k).id() == id_unknown) k++;
                
                locate_3p(oa->points(i), oa->points(j), oa->points(k),
                          pos, angle);
        } else {
                // 2 points
                if (oa->get_known_points() > 1) {
                        int i, j;
                        
                        i = 0;
                        while (oa->points(i).id() == id_unknown) i++;
                        j = i + 1;
                        while (oa->points(j).id() == id_unknown) j++;

                        locate_2p(oa->points(i), oa->points(j), pos, angle);
                
                } else {
                        // 1 point and a line
                        if (oa->get_known_lines() > 0
                            && oa->get_known_points() > 0) {
                                int i, j;
                                i = j = 0;

                                bool b = false;                
                                while (!b && i < oa->get_points()) {
                                        j=0;
                                        while (!b && j < oa->get_lines()) {
                                                if (oa->lines(j).id() == id_line_l
                                                    || oa->lines(j).id() == id_line_r) {
                                                        if ((j + 1 < oa->get_lines())
                                                            && (oa->points(i).x()
                                                                == oa->lines(j).x()))
                                                                j++;
                                                        else
                                                                b = true;
                                                } else {
                                                        if ((j + 1 < oa->get_lines())
                                                            && (oa->points(i).y()
                                                                == oa->lines(j).y()))
                                                                j++;
                                                        else
                                                                b = true;
                                                }
                                        }
                                        if (!b && (i + 1 < oa->get_points())) i++;
                                }

                                hq_point_line(oa->points(i), oa->lines(j), pos, angle);

                        } else {
                                angle = bearing_unknown;
                                pos.set(bearing_unknown, bearing_unknown);
                        }
                }
        }

        if (angle != bearing_unknown) {
                headangle = -angle;
                bodyangle = headangle - deg2rad(k->get_neck());
        } else {
                headangle = angle;
                bodyangle = angle;
        }

        lastangle = angle;
        lastpos = pos;
        
        oa->calc_abs(pos.x(), pos.y(), bodyangle, headangle);
}
