/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   locate nonmovable objects on the soccer field
   Copyright (C) Oct 21 1997 Oliver Obst
   $Id: sfield.C,v 1.3 1999/02/03 07:29:12 frvit Exp $

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "sfield.h"
#include "soccerconst.h"
#include "maths.h"
#include "objarray.h"
#include "objinfo.h"
#include "knowledge.h"

Sfield::Sfield(Knowledge *k0)
{
        k = k0;
}


Sfield::~Sfield()
{
}


// --- public -----------------------------------------------------------------
Cvector Sfield::getpos(ObjInfo &obj)
{
        position p = getobj(obj);

        Cvector cv(p.x, p.y);
        return cv;
}


// --- public -----------------------------------------------------------------
double Sfield::xsize(ObjInfo &obj)
{
        position p = getobj(obj);
        return p.xsize;
}


// --- public -----------------------------------------------------------------
double Sfield::ysize(ObjInfo &obj)
{
        position p = getobj(obj);
        return p.ysize;
}


// --- private ----------------------------------------------------------------
position Sfield::getobj(ObjInfo &obj)
{
        position p;
        
        switch (obj.name()) {
        case goal :
                if (obj.id() >= 0 && obj.id() < max_goals)
                        p = pos_convert(k->get_side(), goals[ obj.id() ]);
                else
                        p.x = p.y = bearing_unknown;                
                break;
        case line :
                if (obj.id() >= 0 && obj.id() < max_lines) 
                        p = pos_convert(k->get_side(), lines[ obj.id() ]);
                else
                        p.x = p.y = bearing_unknown;
                break;
        case flag :
                if (obj.id() >= 0 && obj.id() < max_flags)
                        p = pos_convert(k->get_side(), flags[ obj.id() ]);
                else
                        p.x = p.y = bearing_unknown;
                break;
        default :
                p.x = p.y = bearing_unknown;
        }

        return p;
}
