/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   simple robot actions
   Copyright (c) Apr 09 1998 Oliver Obst
   $Id: simple.C,v 1.3 1999/02/03 07:25:03 frvit Exp $

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>

#include "simple.h"
#include "self.h"

Simple::Simple(Self *s, Listproc *l, Knowledge *k0,
               const char *server, int port, double ver)
        : Control(l, server, port, ver)
{
        self = s;
        k = k0;
}

Simple::~Simple()
{
}

// --- public -----------------------------------------------------------------
void Simple::normal_init(const char *teamname)
{
        sensor->set_teamname(teamname, false);
        init(false);
}

// --- public -----------------------------------------------------------------
void Simple::goalie_init(const char *teamname)
{
        sensor->set_teamname(teamname, true);
        init(true);
}

// --- public -----------------------------------------------------------------
// works only in 'before_kick_off' mode!
void Simple::moveto(double x, double y) 
{
        sprintf(tmpstr, "(move %f %f)", x, y);
        outq.write(tmpstr, true);
        self->moveto(x,y);
}

// --- public -----------------------------------------------------------------
void Simple::turn(double moment)
{
        if (moment < min_turn_moment)
                moment += 360.0;
        if (moment > max_turn_moment)
                moment -= 360.0;

        if (moment >=  min_turn_moment && moment <= max_turn_moment) {
                sprintf(tmpstr, "(turn %f)", moment);
                outq.write(tmpstr, true);
                self->turn(moment);
                k->turn();
        }
}

// --- public -----------------------------------------------------------------
void Simple::turn_neck(double angled)
{
        if (angled >= 2*min_neck_angle && angled <= 2*max_neck_angle) {
                sprintf(tmpstr, "(turn_neck %f)", angled);
                outq.write(tmpstr, false);
                k->turn_neck(angled);
        }
}

// --- public -----------------------------------------------------------------
void Simple::dash(double power)
{
        if (power < min_dash_power)
                power = min_dash_power;
        if (power > max_dash_power)
                power = max_dash_power;

        sprintf(tmpstr, "(dash %f)", power);
        outq.write(tmpstr, true);
        k->dash(power);
}

// --- public -----------------------------------------------------------------
void Simple::kick(double power, double direction) 
{
        if (direction < min_kick_moment)
                direction += 360.0;
        if (direction > max_kick_moment)
                direction -= 360.0;
        if (power < min_kick_power)
                power = min_kick_power;
        if (power > max_kick_power)
                power = max_kick_power;

        if (direction >= min_kick_moment && direction <= max_kick_moment) {
                sprintf(tmpstr, "(kick %f %f)", power, direction);
                outq.write(tmpstr, true);
                k->kick();
        }
}

// --- public -----------------------------------------------------------------
void Simple::say(const char *text)
{
        if (text != 0 && strlen(text) > 0) {
                sprintf(tmpstr, "(say %s)", text);
                outq.write(tmpstr, true);
                k->say();
        }
}
