/* -*- mode: c++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) Oct 21 1997 Oliver Obst
   soccer server constants 
   $Id: soccerconst.h,v 1.4 1999/05/05 14:42:33 frvit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _SOCCERCONST_H
#define _SOCCERCONST_H

#include "constants.h"

// used in sfield
struct position
{
        double  x;              // x center of an object
        double  y;              // y center of an object
        double  xsize;          // total length of an object
        double  ysize;          // total depth of an object
};
        

// timings (in ms)
const int       simulator_step  = 100;
const int       send_step       = 150;
const int       recv_step       =  10;
const int       timer_step      =  10;

// min. and max. dash
const double    min_dash_power  = -30.0;
const double    max_dash_power  = 100.0;

// min. and max. kick
const double    min_kick_power  = -30.0;
const double    max_kick_power  = 100.0;

// min. and max. moment (turn)
const double    min_turn_moment = -180.0;
const double    max_turn_moment = 180.0;

// min. and max. neck angle (turn neck)
const double    min_neck_angle  = -90.0;
const double    max_neck_angle  = 90.0;

// min. and max. moment (kick)
const double    min_kick_moment = -180.0;
const double    max_kick_moment = 180.0;

// min. and max. moment (catch)
const double    min_catch_moment = -180.0;
const double    max_catch_moment = 180.0;

// player parameter (default 1.0, 0.5, 0.1, 60.0, 32.0)
const double    player_size     = 0.3;
const double    player_decay	= 0.4;
const double    player_rand	= 0.1;
const double    player_weight   = 60.0;
const double    player_speed_max= 1.0;

// ball parameter (default 0.15, 0.8, 0.2, 0.2, 32.0)
const double    ball_size       = 0.085;
const double    ball_decay      = 0.94;
const double    ball_rand       = 0.05;         // random parameter
const double    ball_weight     = 0.2;
const double    ball_speed_max  = 2.7;

// misc. player constants
const double    kickable_margin = 0.7;
const double    kickable_area   = kickable_margin + ball_size + player_size;
const double    kick_power_rate = 0.016;
const double    stamina_max     = 3500;
const double    stamina_max_inc = 20;
const double    recovery_dec_thr= 0.3;
const double    recovery_dec    = 0.002;
const double    recovery_min    = 0.5;
const double    effort_dec_thr  = 0.3;
const double    effort_dec      = 0.005;
const double    effort_min      = 0.6;
const double    effort_inc_thr  = 0.6;
const double    effort_inc      = 0.01;

// used in sfield
const double    pitch_length            = 105.0;
const double    pitch_width             =  68.0;
const double    pitch_margin            =   5.0;
const double    center_circle_r         =   9.15;
const double    penalty_area_length     =  16.5;
const double    penalty_area_width      =  40.32;
const double    goal_area_length        =   5.5;
const double    goal_area_width         =  18.32;
const double    goal_width              =  14.64;       // 7.32
const double    goal_depth              =   2.44;

const double    pitch_hlength   = pitch_length / 2;
const double    pitch_hwidth    = pitch_width / 2;
const double    penalty_hwidth  = penalty_area_width / 2;
const double    penalty_lstart  = penalty_area_length - pitch_hlength;
const double    penalty_rstart  = pitch_hlength - penalty_area_length;
const double    goal_hwidth     = goal_width / 2;
const double    Xmin            = - pitch_hlength - pitch_margin;
const double    Xmax            = pitch_hlength + pitch_margin;
const double    Ymin            = - pitch_hwidth - pitch_margin;
const double    Ymax            = pitch_hwidth + pitch_margin;

const int max_balls               = 1;
const int max_teamplayers         = 11;
const int max_players             = max_teamplayers + max_teamplayers;

const position  field           = { 0, 0, pitch_length, pitch_width }; 
const position  goals[]  = {{ -pitch_hlength, 0, 0, goal_width },
                                   { pitch_hlength, 0, 0, goal_width }};
#define max_goals (sizeof(::goals)/sizeof(::goals[0]))
const position  flags[]  =
      {{ -pitch_hlength, -pitch_hwidth, 0, 0 },         // id_flag_l_t
       { -pitch_hlength, pitch_hwidth, 0, 0 },          // id_flag_l_b
       { 0, -pitch_hwidth, 0, 0 },                      // id_flag_c_t
       { 0, pitch_hwidth, 0, 0 },                       // id_flag_c_b 
       { pitch_hlength, -pitch_hwidth, 0, 0 },          // id_flag_r_t
       { pitch_hlength, pitch_hwidth, 0, 0 },           // id_flag_r_b
       { penalty_lstart, -penalty_hwidth, 0, 0 },       // id_pflag_l_t
       { penalty_lstart, 0, 0, 0 },                     // id_pflag_l_c
       { penalty_lstart, penalty_hwidth, 0, 0 },        // id_pflag_l_b
       { penalty_rstart, -penalty_hwidth, 0, 0 },       // id_pflag_r_t
       { penalty_rstart, 0, 0, 0 },                     // id_pflag_r_c
       { penalty_rstart, penalty_hwidth, 0, 0 },        // id_pflag_r_b
       { 0, 0, 0, 0 },                                  // id_flag_c
       { -pitch_hlength, -goal_hwidth, 0, 0 },          // id_gflag_l_t
       { -pitch_hlength, goal_hwidth, 0, 0 },           // id_gflag_l_b
       { pitch_hlength, -goal_hwidth, 0, 0 },           // id_gflag_r_t
       { pitch_hlength, goal_hwidth, 0, 0 },            // id_gflag_r_b
       { -10, -pitch_hwidth - pitch_margin, 0, 0 },     // id_oflag_t_l_10
       { -20, -pitch_hwidth - pitch_margin, 0, 0 },     // id_oflag_t_l_20
       { -30, -pitch_hwidth - pitch_margin, 0, 0 },     // id_oflag_t_l_30
       { -40, -pitch_hwidth - pitch_margin, 0, 0 },     // id_oflag_t_l_40
       { -50, -pitch_hwidth - pitch_margin, 0, 0 },     // id_oflag_t_l_50
       { 10, -pitch_hwidth - pitch_margin, 0, 0 },      // id_oflag_t_r_10
       { 20, -pitch_hwidth - pitch_margin, 0, 0 },      // id_oflag_t_r_20
       { 30, -pitch_hwidth - pitch_margin, 0, 0 },      // id_oflag_t_r_30
       { 40, -pitch_hwidth - pitch_margin, 0, 0 },      // id_oflag_t_r_40
       { 50, -pitch_hwidth - pitch_margin, 0, 0 },      // id_oflag_t_r_50
       { -10, pitch_hwidth + pitch_margin, 0, 0 },      // id_oflag_b_l_10
       { -20, pitch_hwidth + pitch_margin, 0, 0 },      // id_oflag_b_l_20
       { -30, pitch_hwidth + pitch_margin, 0, 0 },      // id_oflag_b_l_30
       { -40, pitch_hwidth + pitch_margin, 0, 0 },      // id_oflag_b_l_40
       { -50, pitch_hwidth + pitch_margin, 0, 0 },      // id_oflag_b_l_50
       { 10, pitch_hwidth + pitch_margin, 0, 0 },       // id_oflag_b_r_10
       { 20, pitch_hwidth + pitch_margin, 0, 0 },       // id_oflag_b_r_20
       { 30, pitch_hwidth + pitch_margin, 0, 0 },       // id_oflag_b_r_30
       { 40, pitch_hwidth + pitch_margin, 0, 0 },       // id_oflag_b_r_40
       { 50, pitch_hwidth + pitch_margin, 0, 0 },       // id_oflag_b_r_50
       { -pitch_hlength - pitch_margin, -10, 0, 0 },    // id_oflag_l_t_10
       { -pitch_hlength - pitch_margin, -20, 0, 0 },    // id_oflag_l_t_20
       { -pitch_hlength - pitch_margin, -30, 0, 0 },    // id_oflag_l_t_30
       { -pitch_hlength - pitch_margin, 10, 0, 0 },     // id_oflag_l_b_10
       { -pitch_hlength - pitch_margin, 20, 0, 0 },     // id_oflag_l_b_20
       { -pitch_hlength - pitch_margin, 30, 0, 0 },     // id_oflag_l_b_30
       { pitch_hlength + pitch_margin, -10, 0, 0 },     // id_oflag_r_t_10
       { pitch_hlength + pitch_margin, -20, 0, 0 },     // id_oflag_r_t_20
       { pitch_hlength + pitch_margin, -30, 0, 0 },     // id_oflag_r_t_30
       { pitch_hlength + pitch_margin, 10, 0, 0 },      // id_oflag_r_b_10
       { pitch_hlength + pitch_margin, 20, 0, 0 },      // id_oflag_r_b_20
       { pitch_hlength + pitch_margin, 30, 0, 0 },      // id_oflag_r_b_30
       { 0, -pitch_hwidth - pitch_margin, 0, 0 },       // id_oflag_t_0
       { 0, pitch_hwidth + pitch_margin, 0, 0 },        // id_oflag_b_0
       { -pitch_hlength - pitch_margin, 0, 0, 0 },      // id_oflag_l_0
       { pitch_hlength + pitch_margin, 0, 0, 0 }};      // id_oflag_r_0
#define max_flags (sizeof(::flags)/sizeof(::flags[0]))

const position lines[]   = {{ -pitch_hlength, 0, 0, pitch_width },
                                   { pitch_hlength, 0, 0, pitch_width },
                                   { 0, -pitch_hwidth, pitch_length, 0 },
                                   { 0, pitch_hwidth, pitch_length, 0 }};
#define max_lines (sizeof(::lines)/sizeof(::lines[0]))


inline position pos_convert(char side, position p) 
{
        if (side != right_side) return p;
        else
	  {
	    position converted = {-p.x, -p.y, p.xsize, p.ysize};
	    return converted;
	  }
}


#endif
