/* -*- mode: c++ -*-
   this file is part of the robolog libraries
   a robot timer
   Copyright (C) Apr 06 1999 Oliver Obst
   $Id: timer.C,v 1.1 1999/05/05 15:02:37 frvit Exp $
   
 * most of the stuff in this file is taken from or based upon
 * CMUnited98 (soccer client for Robocup98)
 * Peter Stone <pstone@cs.cmu.edu>
 * Computer Science Department
 * Carnegie Mellon University
 * Copyright (C) 1998 Peter Stone
 *
 * CMUnited-98 was created by Peter Stone, Manuela Veloso, and Patrick Riley
 *
 * You may copy and distribute this program freely as long as you retain
 * this notice.
 * If you make any changes or have any comments we would appreciate a message.
 *
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "timer.h"
#include "debug.h"

#if 0
Timer LastStartClockTime = 0;
Timer SecondLastStartClockTime = 0;
Timer CurrentTime = 0;


// --- public -----------------------------------------------------------------
Timer Timer::operator -(const int &a)
{
        // this is the default case: no stopped cycles and time progressed
        if (s == 0 && t - a > LastStartClockTime.t)
                return Timer(t - a, 0);

        // substract from stopped cycles
        if (s  > 0) {
                if (a <= s) return Timer(t, s - a);
                else {
                        Timer new_time = Timer(t-(a-s), 0);
                        if (new_time < LastStartClockTime)
                                ROBOBUG(ROBOBUG_WARN, TIMER_CLASS,
                                        "can't be sure of "
                                        "this subtraction (1)");
                        return new_time;
                }
        } else {
                // t-a <= Mem->LastStartClockTime.t
                int stopped_time =  a - (t - LastStartClockTime.t);

                if (stopped_time > LastStartClockTime.s ) {
                        // if == 0, OK --- account for players
                        // starting at different times
                        if ( !LastStartClockTime.t ) return Timer(0,0);

                        int tmp = LastStartClockTime.t -
                                (stopped_time - LastStartClockTime.s);

                        if (SecondLastStartClockTime.t)
                                ROBOBUG4(ROBOBUG_WARN, TIMER_CLASS,
                                        "can't be sure of this "
                                        "subtraction (2) - %d, %d, %d, %d",
                                         t,s,a,LastStartClockTime.s);
                        return Timer( tmp, 0 );
                }

                return Timer(LastStartClockTime.t,
                            LastStartClockTime.s - stopped_time);
        }
}

// --- public -----------------------------------------------------------------
Timer Timer::operator +(const int &a)
{
        // this is the default case: no stopped cycles and time progressed
        if (s==0 && t > LastStartClockTime.t && t + a < CurrentTime.t)
                return Timer(t + a, 0);
        
        Timer new_time;

        // we have no stopped cycles
        if (s == 0) {
                // but missed steps propably
                if (LastStartClockTime.t > t) {
                        ROBOBUG2(ROBOBUG_WARN, TIMER_CLASS,
                                 "Can't be sure of this addition (1) - %d, %d",
                                 LastStartClockTime.t, t);
                        new_time = Timer(t + a, 0);
                }

                if (t + a < CurrentTime.t)
                        new_time = Timer(t + a,0);
                else
                  // t+a >= CurrentTime.t 
                        new_time = Timer(CurrentTime.t,
                                        a - (CurrentTime.t - t));
        } else {
                if (s > 0) {
                        // clock still stopped
                        if (t == CurrentTime.t) 
                                new_time = Timer(t, s + a);
                        else {
                                // Could've missed one already
                                if (LastStartClockTime.t != t)
                                        ROBOBUG(ROBOBUG_WARN, TIMER_CLASS,
                                                "Can't be sure "
                                                "of this addition (2)");
                                new_time = Timer(t + (a - (LastStartClockTime.s
                                                          - s)), 0);
                        }
                }
                else /* s<0 */
                        ROBOBUG(ROBOBUG_WARN, TIMER_CLASS,
                                "s shouldn't be < 0");
        }

        /* clock might stop */
        if (new_time > CurrentTime)
                ROBOBUG(ROBOBUG_WARN, TIMER_CLASS,
                        "Can't be sure of this addition (3)");

        return new_time;
}

// --- public -----------------------------------------------------------------
int Timer::operator -(const Timer &a)
{
        if (s == 0) {
                if (a.t < LastStartClockTime.t) {
                        if (a.t <= SecondLastStartClockTime.t)
                                ROBOBUG4(ROBOBUG_WARN, TIMER_CLASS,
                                         "Can't be sure of this subtraction "
                                         "(3) - %d, %d, - %d, %d",
                                         t,s,a.t,a.s);
                        return (t - a.t) + LastStartClockTime.s;
                }

                if (a.t > LastStartClockTime.t) return (t - a.t);

                if (a.t == LastStartClockTime.t)
                        return (t - a.t) + (LastStartClockTime.s - a.s);
        } else {
                if ( s > 0 ) {
                        /* If they're equal, it's OK */
                        if (a.t <= SecondLastStartClockTime.t)
                                ROBOBUG4(ROBOBUG_WARN, TIMER_CLASS,
                                         "Can't be sure of this subtraction "
                                         "(4) - %d, %d, %d, %d",
                                         t,s,a.t,a.s);

                        if (a.t < LastStartClockTime.t) 
                                return LastStartClockTime.s + (t - a.t);
                        else {
                                /* a is during the last stopped interval */
                                if (a.t == LastStartClockTime.t && a.t != t)
                                        return (s +
                                                (LastStartClockTime.s - a.s ))
                                                + (t - a.t);
                        }
                        
                        return (s - a.s) + (t - a.t);
                }
                else // s<0
                        ROBOBUG(ROBOBUG_WARN, TIMER_CLASS,
                                "s shouldn't be < 0");
        }
        
        return 0;
}
#endif
